@@ -18,16 +18,33 @@ package org.gradle.api.plugins.quality;
 
 import org.gradle.api.reporting.ReportContainer;
 import org.gradle.api.reporting.SingleFileReport;
+import org.gradle.api.tasks.Internal;
 
 /**
- * The reporting configuration for the {@link Checkstyle} test.
+ * The reporting configuration for the {@link Checkstyle} task.
  */
 public interface CheckstyleReports extends ReportContainer<SingleFileReport> {
 
+    /**
+     * The checkstyle HTML report.
+     * <p>
+     * This report IS enabled by default.
+     * <p>
+     * Enabling this report will also cause the XML report to be generated, as the HTML is derived from the XML.
+     *
+     * @return The checkstyle HTML report
+     * @since 2.10
+     */
+    @Internal
+    SingleFileReport getHtml();
+
     /**
      * The checkstyle XML report
+     * <p>
+     * This report IS enabled by default.
      *
      * @return The checkstyle XML report
      */
+    @Internal
     SingleFileReport getXml();
 }
