@@ -16,25 +16,40 @@
 
 package org.gradle.api.internal;
 
-import org.gradle.api.file.FileCollection;
+import com.google.common.collect.ImmutableSortedSet;
+import org.gradle.api.NonNullApi;
 import org.gradle.api.internal.tasks.TaskOutputFilePropertySpec;
+import org.gradle.api.internal.tasks.TaskValidationContext;
+import org.gradle.api.internal.tasks.ValidatingValue;
 import org.gradle.api.specs.Spec;
+import org.gradle.api.tasks.TaskOutputFilePropertyBuilder;
 import org.gradle.api.tasks.TaskOutputs;
 
-import java.util.SortedSet;
+import javax.annotation.Nullable;
+import java.io.File;
+import java.util.Set;
 
+@NonNullApi
 public interface TaskOutputsInternal extends TaskOutputs {
 
     Spec<? super TaskInternal> getUpToDateSpec();
 
-    SortedSet<TaskOutputFilePropertySpec> getFileProperties();
+    ImmutableSortedSet<TaskOutputFilePropertySpec> getFileProperties();
+
+    TaskOutputFilePropertyBuilder file(ValidatingValue path);
+
+    TaskOutputFilePropertyBuilder dir(ValidatingValue path);
+
+    TaskOutputFilePropertyBuilder files(ValidatingValue paths);
+
+    TaskOutputFilePropertyBuilder dirs(ValidatingValue paths);
 
     /**
-     * Returns the output files recorded during the previous execution of the task.
+     * Returns the output files and directories recorded during the previous execution of the task.
      */
-    FileCollection getPreviousOutputFiles();
+    Set<File> getPreviousOutputFiles();
 
-    void setHistory(TaskExecutionHistory history);
+    void setHistory(@Nullable TaskExecutionHistory history);
 
     /**
      * Yields information about the cacheability of the outputs.
@@ -45,4 +60,6 @@ public interface TaskOutputsInternal extends TaskOutputs {
      * Returns whether the task has declared any outputs.
      */
     boolean hasDeclaredOutputs();
+
+    void validate(TaskValidationContext context);
 }
