@@ -18,19 +18,13 @@ package org.gradle.api.internal.artifacts.dependencies;
 
 import org.gradle.api.artifacts.Configuration;
 import org.gradle.api.artifacts.ProjectDependency;
-import org.gradle.api.internal.tasks.TaskDependencyInternal;
-
-import java.util.Map;
 
 public interface ProjectDependencyInternal extends ProjectDependency {
 
     /**
      * This method is called when the project dependency is resolved
      */
-    public void beforeResolved();
-
-    TaskDependencyInternal getTaskDependency(Map<String, String> attributes);
-
-    Configuration findProjectConfiguration(Map<String, String> attributes);
+    void beforeResolved();
 
+    Configuration findProjectConfiguration();
 }
