@@ -16,19 +16,34 @@
 
 package org.gradle.api.internal.changedetection.state;
 
-import java.util.List;
+import java.util.Collection;
 
 /**
  * Represents the state of a directory tree.
  */
-public class DirectoryTreeDetails {
+public class DirectoryTreeDetails implements FileTreeSnapshot {
     // Interned path
-    final String path;
+    private final String path;
     // All elements, not just direct children
-    final List<FileDetails> elements;
+    private final Collection<FileSnapshot> descendants;
 
-    public DirectoryTreeDetails(String path, List<FileDetails> elements) {
+    public DirectoryTreeDetails(String path, Collection<FileSnapshot> descendants) {
         this.path = path;
-        this.elements = elements;
+        this.descendants = descendants;
+    }
+
+    @Override
+    public String getPath() {
+        return path;
+    }
+
+    @Override
+    public Collection<FileSnapshot> getDescendants() {
+        return descendants;
+    }
+
+    @Override
+    public String toString() {
+        return path + " (" + descendants.size() + " descendants)";
     }
 }
