@@ -16,14 +16,15 @@
 package org.gradle.api.internal.changedetection.state;
 
 import org.gradle.api.file.FileCollection;
+import org.gradle.api.tasks.FileNormalizer;
 import org.gradle.internal.serialize.SerializerRegistry;
+import org.gradle.normalization.internal.InputNormalizationStrategy;
 
 public interface FileCollectionSnapshotter {
     /**
      * The type used to refer to this snapshotter in the {@link FileCollectionSnapshotterRegistry}.
-     * Must be a super-type of the actual implementation.
      */
-    Class<? extends FileCollectionSnapshotter> getRegisteredType();
+    Class<? extends FileNormalizer> getRegisteredType();
 
     /**
      * Registers the serializer(s) that can be used to serialize the {@link FileCollectionSnapshot} implementations produced by this snapshotter.
@@ -32,11 +33,6 @@ public interface FileCollectionSnapshotter {
 
     /**
      * Creates a snapshot of the contents of the given collection.
-     *
-     * @param files The files to snapshot.
-     * @param compareStrategy How to compare this collection snapshot to others.
-     * @param snapshotNormalizationStrategy How to normalize file snapshots.
-     * @return The snapshot.
      */
-    FileCollectionSnapshot snapshot(FileCollection files, TaskFilePropertyCompareStrategy compareStrategy, SnapshotNormalizationStrategy snapshotNormalizationStrategy);
+    FileCollectionSnapshot snapshot(FileCollection files, PathNormalizationStrategy pathNormalizationStrategy, InputNormalizationStrategy inputNormalizationStrategy);
 }
