@@ -17,13 +17,12 @@
 package org.gradle.api.internal.changedetection.state;
 
 import com.google.common.base.Objects;
-import org.gradle.caching.internal.BuildCacheKeyBuilder;
-import org.gradle.internal.hash.HashUtil;
+import org.gradle.caching.internal.BuildCacheHasher;
 
 public class IgnoredPathFileSnapshot implements NormalizedFileSnapshot {
-    private final IncrementalFileSnapshot snapshot;
+    private final FileContentSnapshot snapshot;
 
-    public IgnoredPathFileSnapshot(IncrementalFileSnapshot snapshot) {
+    public IgnoredPathFileSnapshot(FileContentSnapshot snapshot) {
         this.snapshot = snapshot;
     }
 
@@ -33,13 +32,13 @@ public class IgnoredPathFileSnapshot implements NormalizedFileSnapshot {
     }
 
     @Override
-    public IncrementalFileSnapshot getSnapshot() {
+    public FileContentSnapshot getSnapshot() {
         return snapshot;
     }
 
     @Override
-    public void appendToCacheKey(BuildCacheKeyBuilder builder) {
-        builder.putBytes(snapshot.getContentMd5().asBytes());
+    public void appendToHasher(BuildCacheHasher hasher) {
+        hasher.putHash(snapshot.getContentMd5());
     }
 
     @Override
@@ -47,7 +46,7 @@ public class IgnoredPathFileSnapshot implements NormalizedFileSnapshot {
         if (!(o instanceof IgnoredPathFileSnapshot)) {
             return -1;
         }
-        return HashUtil.compareHashCodes(getSnapshot().getContentMd5(), o.getSnapshot().getContentMd5());
+        return getSnapshot().getContentMd5().compareTo(o.getSnapshot().getContentMd5());
     }
 
     @Override
