@@ -52,6 +52,7 @@ import org.gradle.api.internal.NoConventionMapping;
 import org.gradle.api.internal.ProcessOperations;
 import org.gradle.api.internal.artifacts.Module;
 import org.gradle.api.internal.artifacts.configurations.DependencyMetaDataProvider;
+import org.gradle.api.internal.file.DefaultProjectLayout;
 import org.gradle.api.internal.file.FileOperations;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.file.SourceDirectorySetFactory;
@@ -64,8 +65,12 @@ import org.gradle.api.internal.project.taskfactory.ITaskFactory;
 import org.gradle.api.internal.tasks.TaskContainerInternal;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
+import org.gradle.api.model.ObjectFactory;
 import org.gradle.api.plugins.Convention;
 import org.gradle.api.plugins.ExtensionContainer;
+import org.gradle.api.provider.PropertyState;
+import org.gradle.api.provider.Provider;
+import org.gradle.api.provider.ProviderFactory;
 import org.gradle.api.resources.ResourceHandler;
 import org.gradle.api.tasks.WorkResult;
 import org.gradle.configuration.ScriptPluginFactory;
@@ -81,6 +86,7 @@ import org.gradle.internal.logging.StandardOutputCapture;
 import org.gradle.internal.metaobject.BeanDynamicObject;
 import org.gradle.internal.metaobject.DynamicObject;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.internal.resource.TextResourceLoader;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
 import org.gradle.internal.typeconversion.TypeConverter;
@@ -99,6 +105,7 @@ import org.gradle.model.internal.manage.instance.ManagedProxyFactory;
 import org.gradle.model.internal.manage.schema.ModelSchemaStore;
 import org.gradle.model.internal.registry.ModelRegistry;
 import org.gradle.model.internal.type.ModelType;
+import org.gradle.normalization.InputNormalizationHandler;
 import org.gradle.process.ExecResult;
 import org.gradle.process.ExecSpec;
 import org.gradle.process.JavaExecSpec;
@@ -107,6 +114,7 @@ import org.gradle.util.ConfigureUtil;
 import org.gradle.util.DeprecationLogger;
 import org.gradle.util.Path;
 
+import javax.annotation.Nullable;
 import javax.inject.Inject;
 import java.io.File;
 import java.net.URI;
@@ -117,10 +125,12 @@ import java.util.Map;
 import java.util.Set;
 import java.util.TreeMap;
 import java.util.TreeSet;
+import java.util.concurrent.Callable;
 
+import static com.google.common.base.Strings.isNullOrEmpty;
 import static java.util.Collections.singletonMap;
+import static org.gradle.util.ConfigureUtil.configureUsing;
 import static org.gradle.util.GUtil.addMaps;
-import static org.gradle.util.GUtil.isTrue;
 
 @NoConventionMapping
 public class DefaultProject extends AbstractPluginAware implements ProjectInternal, DynamicObjectAware {
@@ -144,6 +154,8 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
 
     private final File projectDir;
 
+    private final File buildFile;
+
     private final ProjectInternal parent;
 
     private final String name;
@@ -166,10 +178,6 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
 
     private AntBuilder ant;
 
-    private Object buildDir = Project.DEFAULT_BUILD_DIR_NAME;
-
-    private File buildDirCached;
-
     private final int depth;
 
     private TaskContainerInternal taskContainer;
@@ -190,8 +198,9 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     private Path identityPath;
 
     public DefaultProject(String name,
-                          ProjectInternal parent,
+                          @Nullable ProjectInternal parent,
                           File projectDir,
+                          File buildFile,
                           ScriptSource buildScriptSource,
                           GradleInternal gradle,
                           ServiceRegistryFactory serviceRegistryFactory,
@@ -199,9 +208,9 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
                           ClassLoaderScope baseClassLoaderScope) {
         this.classLoaderScope = selfClassLoaderScope;
         this.baseClassLoaderScope = baseClassLoaderScope;
-        assert name != null;
         this.rootProject = parent != null ? parent.getRootProject() : this;
         this.projectDir = projectDir;
+        this.buildFile = buildFile;
         this.parent = parent;
         this.name = name;
         this.state = new ProjectStateInternal();
@@ -230,6 +239,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         populateModelRegistry(services.get(ModelRegistry.class));
     }
 
+    @SuppressWarnings("unused")
     static class BasicServicesRules extends RuleSource {
         @Hidden @Model
         SourceDirectorySetFactory sourceDirectorySetFactory(ServiceRegistry serviceRegistry) {
@@ -314,10 +324,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return "Project.<init>." + path + "()";
     }
 
+    @Override
     public ProjectInternal getRootProject() {
         return rootProject;
     }
 
+    @Override
     public GradleInternal getGradle() {
         return gradle;
     }
@@ -334,56 +346,69 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public ScriptHandler getBuildscript() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public File getBuildFile() {
-        return getBuildscript().getSourceFile();
+        return buildFile;
     }
 
+    @Override
     public void setScript(groovy.lang.Script buildScript) {
         extensibleDynamicObject.addObject(new BeanDynamicObject(buildScript).withNoProperties().withNotImplementsMissing(),
             ExtensibleDynamicObject.Location.BeforeConvention);
     }
 
+    @Override
     public ScriptSource getBuildScriptSource() {
         return buildScriptSource;
     }
 
+    @Override
     public File getRootDir() {
         return rootProject.getProjectDir();
     }
 
+    @Override
     public ProjectInternal getParent() {
         return parent;
     }
 
+    @Override
     public ProjectIdentifier getParentIdentifier() {
         return parent;
     }
 
+    @Override
     public DynamicObject getAsDynamicObject() {
         return extensibleDynamicObject;
     }
 
+    @Override
     public DynamicObject getInheritedScope() {
         return extensibleDynamicObject.getInheritable();
     }
 
+    @Override
     public String getName() {
         return name;
     }
 
+    @Override
     public String getDescription() {
         return description;
     }
 
+    @Override
     public void setDescription(String description) {
         this.description = description;
     }
 
+    @Override
     public Object getGroup() {
         if (group != null) {
             return group;
@@ -394,42 +419,52 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return group;
     }
 
+    @Override
     public void setGroup(Object group) {
         this.group = group;
     }
 
+    @Override
     public Object getVersion() {
         return version == null ? DEFAULT_VERSION : version;
     }
 
+    @Override
     public void setVersion(Object version) {
         this.version = version;
     }
 
+    @Override
     public Object getStatus() {
         return status == null ? DEFAULT_STATUS : status;
     }
 
+    @Override
     public void setStatus(Object status) {
         this.status = status;
     }
 
+    @Override
     public Map<String, Project> getChildProjects() {
         return childProjects;
     }
 
+    @Override
     public List<String> getDefaultTasks() {
         return defaultTasks;
     }
 
+    @Override
     public void setDefaultTasks(List<String> defaultTasks) {
         this.defaultTasks = defaultTasks;
     }
 
+    @Override
     public ProjectStateInternal getState() {
         return state;
     }
 
+    @Override
     public FileResolver getFileResolver() {
         if (fileResolver == null) {
             fileResolver = services.get(FileResolver.class);
@@ -445,6 +480,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         this.ant = ant;
     }
 
+    @Override
     public ArtifactHandler getArtifacts() {
         if (artifactHandler == null) {
             artifactHandler = services.get(ArtifactHandler.class);
@@ -457,11 +493,13 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public RepositoryHandler getRepositories() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ConfigurationContainer getConfigurations() {
         if (configurationContainer == null) {
             configurationContainer = services.get(ConfigurationContainer.class);
@@ -473,10 +511,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         this.configurationContainer = configurationContainer;
     }
 
+    @Override
     public Convention getConvention() {
         return extensibleDynamicObject.getConvention();
     }
 
+    @Override
     public String getPath() {
         return path.toString();
     }
@@ -493,20 +533,24 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return identityPath;
     }
 
+    @Override
     public int getDepth() {
         return depth;
     }
 
     @Inject
+    @Override
     public ProjectRegistry<ProjectInternal> getProjectRegistry() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public int depthCompare(Project otherProject) {
         return new Integer(getDepth()).compareTo(otherProject.getDepth());
     }
 
+    @Override
     public int compareTo(Project otherProject) {
         int depthCompare = depthCompare(otherProject);
         if (depthCompare == 0) {
@@ -516,6 +560,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         }
     }
 
+    @Override
     public String absoluteProjectPath(String path) {
         return this.path.absolutePath(path);
     }
@@ -530,15 +575,27 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return path;
     }
 
+    @Override
+    public Path getBuildPath() {
+        return gradle.getIdentityPath();
+    }
+
     @Override
     public Path projectPath(String name) {
         return path.child(name);
     }
 
+    @Override
+    public boolean isScript() {
+        return false;
+    }
+
+    @Override
     public String relativeProjectPath(String path) {
         return this.path.relativePath(path);
     }
 
+    @Override
     public ProjectInternal project(String path) {
         ProjectInternal project = findProject(path);
         if (project == null) {
@@ -547,29 +604,35 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return project;
     }
 
+    @Override
     public ProjectInternal findProject(String path) {
-        if (!isTrue(path)) {
+        if (isNullOrEmpty(path)) {
             throw new InvalidUserDataException("A path must be specified!");
         }
         return getProjectRegistry().getProject(absoluteProjectPath(path));
     }
 
+    @Override
     public Set<Project> getAllprojects() {
         return new TreeSet<Project>(getProjectRegistry().getAllProjects(getPath()));
     }
 
+    @Override
     public Set<Project> getSubprojects() {
         return new TreeSet<Project>(getProjectRegistry().getSubProjects(getPath()));
     }
 
+    @Override
     public void subprojects(Action<? super Project> action) {
-        configure(getSubprojects(), action);
+        getProjectConfigurator().subprojects(getSubprojects(), action);
     }
 
+    @Override
     public void allprojects(Action<? super Project> action) {
-        configure(getAllprojects(), action);
+        getProjectConfigurator().allprojects(getAllprojects(), action);
     }
 
+    @Override
     public <T> Iterable<T> configure(Iterable<T> objects, Action<? super T> configureAction) {
         for (T object : objects) {
             configureAction.execute(object);
@@ -577,6 +640,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return objects;
     }
 
+    @Override
     public AntBuilder getAnt() {
         if (ant == null) {
             ant = createAntBuilder();
@@ -584,6 +648,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return ant;
     }
 
+    @Override
     public AntBuilder createAntBuilder() {
         return getAntBuilderFactory().create();
     }
@@ -591,10 +656,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     /**
      * This method is used when scripts access the project via project.x
      */
+    @Override
     public Project getProject() {
         return this;
     }
 
+    @Override
     public DefaultProject evaluate() {
         getProjectEvaluator().evaluate(this, state);
         return this;
@@ -610,10 +677,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return this;
     }
 
+    @Override
     public TaskContainerInternal getTasks() {
         return taskContainer;
     }
 
+    @Override
     public void defaultTasks(String... defaultTasks) {
         if (defaultTasks == null) {
             throw new InvalidUserDataException("Default tasks must not be null!");
@@ -627,26 +696,32 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         }
     }
 
+    @Override
     public void addChildProject(ProjectInternal childProject) {
         childProjects.put(childProject.getName(), childProject);
     }
 
+    @Override
     public File getProjectDir() {
         return projectDir;
     }
 
+    @Override
     public File getBuildDir() {
-        if (buildDirCached == null) {
-            buildDirCached = file(buildDir);
-        }
-        return buildDirCached;
+        return getLayout().getBuildDirectory().getAsFile().get();
     }
 
+    @Override
+    public void setBuildDir(File path) {
+        setBuildDir((Object) path);
+    }
+
+    @Override
     public void setBuildDir(Object path) {
-        buildDir = path;
-        buildDirCached = null;
+        getLayout().setBuildDirectory(path);
     }
 
+    @Override
     public void evaluationDependsOnChildren() {
         for (Project project : childProjects.values()) {
             DefaultProject defaultProjectToEvaluate = (DefaultProject) project;
@@ -654,8 +729,9 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         }
     }
 
+    @Override
     public Project evaluationDependsOn(String path) {
-        if (!isTrue(path)) {
+        if (isNullOrEmpty(path)) {
             throw new InvalidUserDataException("You must specify a project!");
         }
         DefaultProject projectToEvaluate = (DefaultProject) project(path);
@@ -685,10 +761,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return builder.toString();
     }
 
+    @Override
     public String toString() {
         return getDisplayName();
     }
 
+    @Override
     public Map<Project, Set<Task>> getAllTasks(boolean recursive) {
         final Map<Project, Set<Task>> foundTargets = new TreeMap<Project, Set<Task>>();
         Action<Project> action = new Action<Project>() {
@@ -704,8 +782,9 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return foundTargets;
     }
 
+    @Override
     public Set<Task> getTasksByName(final String name, boolean recursive) {
-        if (!isTrue(name)) {
+        if (isNullOrEmpty(name)) {
             throw new InvalidUserDataException("Name is not specified!");
         }
         final Set<Task> foundTasks = new HashSet<Task>();
@@ -734,62 +813,122 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
+    @Inject
+    public ProviderFactory getProviders() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    @Inject
+    public ObjectFactory getObjects() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    @Inject
+    public DefaultProjectLayout getLayout() {
+        // Decoration takes care of the implementation
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
     public File file(Object path) {
         return getFileOperations().file(path);
     }
 
+    @Override
     public File file(Object path, PathValidation validation) {
         return getFileOperations().file(path, validation);
     }
 
+    @Override
     public URI uri(Object path) {
         return getFileOperations().uri(path);
     }
 
+    @Override
     public ConfigurableFileCollection files(Object... paths) {
         return getFileOperations().files(paths);
     }
 
+    @Override
     public ConfigurableFileCollection files(Object paths, Closure closure) {
-        return ConfigureUtil.configure(closure, getFileOperations().files(paths));
+        return ConfigureUtil.configure(closure, files(paths));
     }
 
+    @Override
+    public ConfigurableFileCollection files(Object paths, Action<? super ConfigurableFileCollection> configureAction) {
+        ConfigurableFileCollection files = files(paths);
+        configureAction.execute(files);
+        return files;
+    }
+
+    @Override
     public ConfigurableFileTree fileTree(Object baseDir) {
         return getFileOperations().fileTree(baseDir);
     }
 
+    @Override
     public ConfigurableFileTree fileTree(Object baseDir, Closure closure) {
-        return ConfigureUtil.configure(closure, getFileOperations().fileTree(baseDir));
+        return ConfigureUtil.configure(closure, fileTree(baseDir));
     }
 
+    @Override
+    public ConfigurableFileTree fileTree(Object baseDir, Action<? super ConfigurableFileTree> configureAction) {
+        ConfigurableFileTree fileTree = fileTree(baseDir);
+        configureAction.execute(fileTree);
+        return fileTree;
+    }
+
+    @Override
     public ConfigurableFileTree fileTree(Map<String, ?> args) {
         return getFileOperations().fileTree(args);
     }
 
+    @Override
     public FileTree zipTree(Object zipPath) {
         return getFileOperations().zipTree(zipPath);
     }
 
+    @Override
     public FileTree tarTree(Object tarPath) {
         return getFileOperations().tarTree(tarPath);
     }
 
+    @Override
+    public <T> Provider<T> provider(Callable<T> value) {
+        return getProviders().provider(value);
+    }
+
+    @Override
+    public <T> PropertyState<T> property(Class<T> clazz) {
+        return getProviders().property(clazz);
+    }
+
+    @Override
     public ResourceHandler getResources() {
         return getFileOperations().getResources();
     }
 
+    @Override
     public String relativePath(Object path) {
         return getFileOperations().relativePath(path);
     }
 
+    @Override
     public File mkdir(Object path) {
         return getFileOperations().mkdir(path);
     }
 
+    @Override
     public boolean delete(Object... paths) {
         return getFileOperations().delete(paths);
     }
 
+    @Override
     public WorkResult delete(Action<? super DeleteSpec> action) {
         return getFileOperations().delete(action);
     }
@@ -801,6 +940,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return antBuilderFactory;
     }
 
+    @Override
     public DependencyHandler getDependencies() {
         if (dependencyHandler == null) {
             dependencyHandler = services.get(DependencyHandler.class);
@@ -812,62 +952,76 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         this.dependencyHandler = dependencyHandler;
     }
 
+    @Override
     public ProjectEvaluationListener getProjectEvaluationBroadcaster() {
         return evaluationListener.getSource();
     }
 
+    @Override
     public void beforeEvaluate(Action<? super Project> action) {
         evaluationListener.add("beforeEvaluate", action);
     }
 
+    @Override
     public void afterEvaluate(Action<? super Project> action) {
         evaluationListener.add("afterEvaluate", action);
     }
 
+    @Override
     public void beforeEvaluate(Closure closure) {
         evaluationListener.add(new ClosureBackedMethodInvocationDispatch("beforeEvaluate", closure));
     }
 
+    @Override
     public void afterEvaluate(Closure closure) {
         evaluationListener.add(new ClosureBackedMethodInvocationDispatch("afterEvaluate", closure));
     }
 
+    @Override
     public Logger getLogger() {
         return BUILD_LOGGER;
     }
 
+    @Override
     public StandardOutputCapture getStandardOutputCapture() {
         return getLogging();
     }
 
     @Inject
+    @Override
     public LoggingManagerInternal getLogging() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
     @Inject
+    @Override
     public SoftwareComponentContainer getComponents() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public Object property(String propertyName) throws MissingPropertyException {
         return extensibleDynamicObject.getProperty(propertyName);
     }
 
+    @Override
     public Object findProperty(String propertyName) {
         return hasProperty(propertyName) ? property(propertyName) : null;
     }
 
+    @Override
     public void setProperty(String name, Object value) {
         extensibleDynamicObject.setProperty(name, value);
     }
 
+    @Override
     public boolean hasProperty(String propertyName) {
         return extensibleDynamicObject.hasProperty(propertyName);
     }
 
+    @Override
     public Map<String, ?> getProperties() {
         return DeprecationLogger.whileDisabled(new Factory<Map<String, ?>>() {
             public Map<String, ?> create() {
@@ -876,26 +1030,32 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         });
     }
 
+    @Override
     public WorkResult copy(Closure closure) {
-        return copy(ConfigureUtil.configureUsing(closure));
+        return copy(configureUsing(closure));
     }
 
+    @Override
     public WorkResult copy(Action<? super CopySpec> action) {
         return getFileOperations().copy(action);
     }
 
+    @Override
     public WorkResult sync(Action<? super CopySpec> action) {
         return getFileOperations().sync(action);
     }
 
+    @Override
     public CopySpec copySpec(Closure closure) {
-        return copySpec(ConfigureUtil.configureUsing(closure));
+        return ConfigureUtil.configure(closure, copySpec());
     }
 
+    @Override
     public CopySpec copySpec(Action<? super CopySpec> action) {
         return Actions.with(copySpec(), action);
     }
 
+    @Override
     public CopySpec copySpec() {
         return getFileOperations().copySpec();
     }
@@ -906,60 +1066,79 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ExecResult javaexec(Closure closure) {
-        return javaexec(ConfigureUtil.configureUsing(closure));
+        return javaexec(configureUsing(closure));
     }
 
+    @Override
     public ExecResult javaexec(Action<? super JavaExecSpec> action) {
         return getProcessOperations().javaexec(action);
     }
 
+    @Override
     public ExecResult exec(Closure closure) {
-        return exec(ConfigureUtil.configureUsing(closure));
+        return exec(configureUsing(closure));
     }
 
+    @Override
     public ExecResult exec(Action<? super ExecSpec> action) {
         return getProcessOperations().exec(action);
     }
 
+    @Override
     public ServiceRegistry getServices() {
         return services;
     }
 
+    @Override
     public ServiceRegistryFactory getServiceRegistryFactory() {
         return services.get(ServiceRegistryFactory.class);
     }
 
+    @Override
     public Module getModule() {
         return services.get(DependencyMetaDataProvider.class).getModule();
     }
 
+    @Override
     public AntBuilder ant(Closure configureClosure) {
         return ConfigureUtil.configure(configureClosure, getAnt());
     }
 
-    public void subprojects(Closure configureClosure) {
-        configure(getSubprojects(), configureClosure);
+    @Override
+    public AntBuilder ant(Action<? super AntBuilder> configureAction) {
+        AntBuilder ant = getAnt();
+        configureAction.execute(ant);
+        return ant;
     }
 
+    @Override
+    public void subprojects(Closure configureClosure) {
+        getProjectConfigurator().subprojects(getSubprojects(), configureClosure);
+    }
+
+    @Override
     public void allprojects(Closure configureClosure) {
-        configure(getAllprojects(), configureClosure);
+        getProjectConfigurator().allprojects(getAllprojects(), configureClosure);
     }
 
     @Override
     public Project project(String path, Closure configureClosure) {
-        return ConfigureUtil.configure(configureClosure, project(path));
+        return getProjectConfigurator().project(project(path), configureClosure);
     }
 
     @Override
     public Project project(String path, Action<? super Project> configureAction) {
-        return Actions.with(project(path), configureAction);
+        return getProjectConfigurator().project(project(path), configureAction);
     }
 
+    @Override
     public Object configure(Object object, Closure configureClosure) {
         return ConfigureUtil.configure(configureClosure, object);
     }
 
+    @Override
     public Iterable<?> configure(Iterable<?> objects, Closure configureClosure) {
         for (Object object : objects) {
             configure(object, configureClosure);
@@ -967,26 +1146,37 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return objects;
     }
 
+    @Override
     public void configurations(Closure configureClosure) {
         ((Configurable<?>) getConfigurations()).configure(configureClosure);
     }
 
+    @Override
     public void repositories(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getRepositories());
     }
 
+    @Override
     public void dependencies(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getDependencies());
     }
 
+    @Override
     public void artifacts(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getArtifacts());
     }
 
+    @Override
+    public void artifacts(Action<? super ArtifactHandler> configureAction) {
+        configureAction.execute(getArtifacts());
+    }
+
+    @Override
     public void buildscript(Closure configureClosure) {
         ConfigureUtil.configure(configureClosure, getBuildscript());
     }
 
+    @Override
     public Task task(String task) {
         return taskContainer.create(task);
     }
@@ -995,6 +1185,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return taskContainer.create(task.toString());
     }
 
+    @Override
     public Task task(String task, Closure configureClosure) {
         return taskContainer.create(task).configure(configureClosure);
     }
@@ -1003,6 +1194,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return task(task.toString(), configureClosure);
     }
 
+    @Override
     public Task task(Map options, String task) {
         return taskContainer.create(addMaps(options, singletonMap(Task.TASK_NAME, task)));
     }
@@ -1011,6 +1203,7 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return task(options, task.toString());
     }
 
+    @Override
     public Task task(Map options, String task, Closure configureClosure) {
         return taskContainer.create(addMaps(options, singletonMap(Task.TASK_NAME, task))).configure(configureClosure);
     }
@@ -1020,12 +1213,14 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
     }
 
     @Inject
+    @Override
     public ProjectConfigurationActionContainer getConfigurationActions() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
     }
 
     @Inject
+    @Override
     public ModelRegistry getModelRegistry() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
@@ -1039,10 +1234,16 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
 
     @Override
     protected DefaultObjectConfigurationAction createObjectConfigurationAction() {
-        return new DefaultObjectConfigurationAction(getFileResolver(), getScriptPluginFactory(), getScriptHandlerFactory(), getBaseClassLoaderScope(), this);
+        return new DefaultObjectConfigurationAction(getFileResolver(), getScriptPluginFactory(), getScriptHandlerFactory(), getBaseClassLoaderScope(), getResourceLoader(), this);
     }
 
     @Inject
+    protected TextResourceLoader getResourceLoader() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    @Override
     public PluginManagerInternal getPluginManager() {
         // Decoration takes care of the implementation
         throw new UnsupportedOperationException();
@@ -1060,10 +1261,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Override
     public ClassLoaderScope getClassLoaderScope() {
         return classLoaderScope;
     }
 
+    @Override
     public ClassLoaderScope getBaseClassLoaderScope() {
         return baseClassLoaderScope;
     }
@@ -1075,21 +1278,25 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         return object;
     }
 
+    @Override
     public <T> NamedDomainObjectContainer<T> container(Class<T> type) {
         Instantiator instantiator = getServices().get(Instantiator.class);
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer());
     }
 
+    @Override
     public <T> NamedDomainObjectContainer<T> container(Class<T> type, NamedDomainObjectFactory<T> factory) {
         Instantiator instantiator = getServices().get(Instantiator.class);
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer(), factory);
     }
 
+    @Override
     public <T> NamedDomainObjectContainer<T> container(Class<T> type, Closure factoryClosure) {
         Instantiator instantiator = getServices().get(Instantiator.class);
         return instantiator.newInstance(FactoryNamedDomainObjectContainer.class, type, instantiator, new DynamicPropertyNamer(), factoryClosure);
     }
 
+    @Override
     public ExtensionContainerInternal getExtensions() {
         return (ExtensionContainerInternal) getConvention();
     }
@@ -1110,6 +1317,12 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         throw new UnsupportedOperationException();
     }
 
+    @Inject
+    protected CrossProjectConfigurator getProjectConfigurator() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
     public void addDeferredConfiguration(Runnable configuration) {
         getDeferredProjectConfiguration().add(configuration);
     }
@@ -1119,4 +1332,15 @@ public class DefaultProject extends AbstractPluginAware implements ProjectIntern
         getDeferredProjectConfiguration().fire();
     }
 
+    @Inject
+    @Override
+    public InputNormalizationHandler getNormalization() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void normalization(Action<? super InputNormalizationHandler> configuration) {
+        configuration.execute(getNormalization());
+    }
+
 }
