@@ -16,17 +16,11 @@
 package org.gradle.api.internal.project.taskfactory;
 
 import org.gradle.api.internal.TaskInternal;
+import org.gradle.api.internal.tasks.TaskPropertyValue;
 import org.gradle.api.tasks.OutputFile;
 import org.gradle.api.tasks.TaskOutputFilePropertyBuilder;
 
-import java.io.File;
 import java.lang.annotation.Annotation;
-import java.util.Collection;
-import java.util.concurrent.Callable;
-
-import static org.gradle.api.internal.tasks.TaskOutputsUtil.ensureParentDirectoryExists;
-import static org.gradle.api.internal.tasks.TaskOutputsUtil.validateFile;
-import static org.gradle.util.GUtil.uncheckedCall;
 
 public class OutputFilePropertyAnnotationHandler extends AbstractOutputPropertyAnnotationHandler {
 
@@ -36,21 +30,7 @@ public class OutputFilePropertyAnnotationHandler extends AbstractOutputPropertyA
     }
 
     @Override
-    protected void validate(String propertyName, Object value, Collection<String> messages) {
-        validateFile(propertyName, (File) value, messages);
-    }
-
-
-    @Override
-    protected TaskOutputFilePropertyBuilder createPropertyBuilder(TaskPropertyActionContext context, TaskInternal task, Callable<Object> futureValue) {
+    protected TaskOutputFilePropertyBuilder createPropertyBuilder(TaskPropertyActionContext context, TaskInternal task, TaskPropertyValue futureValue) {
         return task.getOutputs().file(futureValue);
     }
-
-    @Override
-    protected void beforeTask(final Callable<Object> futureValue) {
-        File file = (File) uncheckedCall(futureValue);
-        if (file != null) {
-            ensureParentDirectoryExists(file);
-        }
-    }
 }
