@@ -16,34 +16,26 @@
 
 package org.gradle.api.internal.project.taskfactory;
 
+import org.gradle.api.GradleException;
 import org.gradle.api.internal.TaskInternal;
+import org.gradle.api.internal.tasks.TaskPropertyValue;
+import org.gradle.api.internal.tasks.TaskValidationContext;
+import org.gradle.api.internal.tasks.ValidationAction;
+import org.gradle.api.provider.Provider;
 import org.gradle.internal.Factory;
-import org.gradle.internal.reflect.JavaReflectionUtil;
+import org.gradle.internal.UncheckedException;
+import org.gradle.util.DeferredUtil;
 import org.gradle.util.DeprecationLogger;
 
+import javax.annotation.Nonnull;
 import java.lang.annotation.Annotation;
+import java.lang.reflect.InvocationTargetException;
 import java.lang.reflect.Method;
-import java.util.Collection;
-import java.util.concurrent.Callable;
 
 public class TaskPropertyInfo implements Comparable<TaskPropertyInfo> {
-    private static final ValidationAction NO_OP_VALIDATION_ACTION = new ValidationAction() {
-        public void validate(String propertyName, Object value, Collection<String> messages) {
-        }
-    };
-    private static final TaskPropertyValue NO_OP_VALUE = new TaskPropertyValue() {
-        public Object getValue() {
-            return null;
-        }
-
-        public void checkNotNull(Collection<String> messages) {
-        }
-
-        public void checkValid(Collection<String> messages) {
-        }
-    };
     private static final UpdateAction NO_OP_CONFIGURATION_ACTION = new UpdateAction() {
-        public void update(TaskInternal task, Callable<Object> futureValue) {
+        @Override
+        public void update(TaskInternal task, TaskPropertyValue futureValue) {
         }
     };
 
@@ -51,18 +43,15 @@ public class TaskPropertyInfo implements Comparable<TaskPropertyInfo> {
     private final String propertyName;
     private final Class<? extends Annotation> propertyType;
     private final Method method;
-    private final ValidationAction validationAction;
     private final UpdateAction configureAction;
-    private final boolean optional;
 
-    TaskPropertyInfo(TaskPropertyInfo parent, String propertyName, Class<? extends Annotation> propertyType, Method method, ValidationAction validationAction, UpdateAction configureAction, boolean optional) {
+    TaskPropertyInfo(TaskPropertyInfo parent, String propertyName, Class<? extends Annotation> propertyType, Method method, UpdateAction configureAction) {
         this.parent = parent;
         this.propertyName = propertyName;
         this.propertyType = propertyType;
         this.method = method;
-        this.validationAction = validationAction == null ? NO_OP_VALIDATION_ACTION : validationAction;
+        method.setAccessible(true);
         this.configureAction = configureAction == null ? NO_OP_CONFIGURATION_ACTION : configureAction;
-        this.optional = optional;
     }
 
     @Override
@@ -82,12 +71,12 @@ public class TaskPropertyInfo implements Comparable<TaskPropertyInfo> {
         return configureAction;
     }
 
-    public TaskPropertyValue getValue(Object rootObject) {
+    public PropertyValue getValue(Object rootObject) {
         final Object bean;
         if (parent != null) {
-            TaskPropertyValue parentValue = parent.getValue(rootObject);
+            PropertyValue parentValue = parent.getValue(rootObject);
             if (parentValue.getValue() == null) {
-                return NO_OP_VALUE;
+                return PropertyValue.NO_OP;
             }
             bean = parentValue.getValue();
         } else {
@@ -96,34 +85,55 @@ public class TaskPropertyInfo implements Comparable<TaskPropertyInfo> {
 
         final Object value = DeprecationLogger.whileDisabled(new Factory<Object>() {
             public Object create() {
-                return JavaReflectionUtil.method(Object.class, method).invoke(bean);
+                try {
+                    return method.invoke(bean);
+                } catch (InvocationTargetException e) {
+                    throw UncheckedException.throwAsUncheckedException(e.getCause());
+                } catch (Exception e) {
+                    throw new GradleException(String.format("Could not call %s.%s() on %s", method.getDeclaringClass().getSimpleName(), method.getName(), bean), e);
+                }
             }
         });
 
-        return new TaskPropertyValue() {
+        return new PropertyValue() {
             @Override
             public Object getValue() {
-                return value;
+                return value instanceof Provider ? ((Provider<?>) value).getOrNull() : value;
             }
 
             @Override
-            public void checkNotNull(Collection<String> messages) {
-                if (value == null && !optional) {
-                    messages.add(String.format("No value has been specified for property '%s'.", propertyName));
-                }
-            }
-
-            @Override
-            public void checkValid(Collection<String> messages) {
-                if (value != null) {
-                    validationAction.validate(propertyName, value, messages);
+            public void validate(boolean optional, ValidationAction valueValidator, TaskValidationContext context, TaskValidationContext.Severity severity) {
+                Object unpacked = DeferredUtil.unpack(getValue());
+                if (unpacked == null) {
+                    if (!optional) {
+                        context.recordValidationMessage(severity, String.format("No value has been specified for property '%s'.", propertyName));
+                    }
+                } else {
+                    valueValidator.validate(propertyName, unpacked, context, severity);
                 }
             }
         };
     }
 
     @Override
-    public int compareTo(TaskPropertyInfo o) {
+    public int compareTo(@Nonnull TaskPropertyInfo o) {
         return propertyName.compareTo(o.getName());
     }
+
+    public interface PropertyValue {
+        PropertyValue NO_OP = new PropertyValue() {
+            @Override
+            public Object getValue() {
+                return null;
+            }
+
+            @Override
+            public void validate(boolean optional, ValidationAction valueValidator, TaskValidationContext context, TaskValidationContext.Severity severity) {
+            }
+        };
+
+        Object getValue();
+
+        void validate(boolean optional, ValidationAction valueValidator, TaskValidationContext context, TaskValidationContext.Severity severity);
+    }
 }
