@@ -16,16 +16,10 @@
 
 package org.gradle.api.internal.tasks;
 
-import org.gradle.api.Nullable;
-
+import javax.annotation.Nullable;
 import java.io.File;
 
 public interface CacheableTaskOutputFilePropertySpec extends TaskOutputFilePropertySpec {
-    enum OutputType {
-        FILE, DIRECTORY
-    }
-
     @Nullable
     File getOutputFile();
-    OutputType getOutputType();
 }
