@@ -16,18 +16,13 @@
 
 package org.gradle.api.internal.tasks;
 
-import org.gradle.api.internal.changedetection.state.FileCollectionSnapshotter;
-import org.gradle.api.internal.changedetection.state.SnapshotNormalizationStrategy;
-import org.gradle.api.tasks.PathSensitivity;
+import org.gradle.api.NonNullApi;
+import org.gradle.api.internal.changedetection.state.PathNormalizationStrategy;
 import org.gradle.api.tasks.TaskFilePropertyBuilder;
 
+@NonNullApi
 public interface TaskFilePropertyBuilderInternal extends TaskFilePropertyBuilder {
-    TaskFilePropertyBuilderInternal withSnapshotNormalizationStrategy(SnapshotNormalizationStrategy snapshotNormalizationStrategy);
-
-    TaskFilePropertyBuilderInternal withSnapshotter(Class<? extends FileCollectionSnapshotter> snapshotter);
-
-    @Override
-    TaskFilePropertyBuilderInternal withPathSensitivity(PathSensitivity sensitivity);
+    TaskFilePropertyBuilderInternal withPathNormalizationStrategy(PathNormalizationStrategy pathNormalizationStrategy);
 
     @Override
     TaskFilePropertyBuilderInternal withPropertyName(String propertyName);
