@@ -17,13 +17,11 @@
 package org.gradle.api.internal.tasks;
 
 import org.gradle.api.file.FileCollection;
-import org.gradle.api.internal.changedetection.state.FileCollectionSnapshotter;
-import org.gradle.api.internal.changedetection.state.SnapshotNormalizationStrategy;
-import org.gradle.api.internal.changedetection.state.TaskFilePropertyCompareStrategy;
+import org.gradle.api.internal.changedetection.state.PathNormalizationStrategy;
+import org.gradle.api.tasks.FileNormalizer;
 
 public interface TaskFilePropertySpec extends TaskPropertySpec {
     FileCollection getPropertyFiles();
-    Class<? extends FileCollectionSnapshotter> getSnapshotter();
-    TaskFilePropertyCompareStrategy getCompareStrategy();
-    SnapshotNormalizationStrategy getSnapshotNormalizationStrategy();
+    Class<? extends FileNormalizer> getNormalizer();
+    PathNormalizationStrategy getPathNormalizationStrategy();
 }
