@@ -17,8 +17,8 @@ package org.gradle.api.tasks;
 
 import org.gradle.StartParameter;
 import org.gradle.api.internal.ConventionTask;
-import org.gradle.initialization.GradleLauncher;
 import org.gradle.initialization.NestedBuildFactory;
+import org.gradle.internal.invocation.BuildController;
 
 import java.io.File;
 import java.util.Collection;
@@ -66,6 +66,16 @@ public class GradleBuild extends ConventionTask {
         return getStartParameter().getCurrentDir();
     }
 
+    /**
+     * Sets the project directory for the build.
+     *
+     * @param dir The project directory. Should not be null.
+     * @since 4.0
+     */
+    public void setDir(File dir) {
+        setDir((Object) dir);
+    }
+
     /**
      * Sets the project directory for the build.
      *
@@ -86,6 +96,16 @@ public class GradleBuild extends ConventionTask {
         return getStartParameter().getBuildFile();
     }
 
+    /**
+     * Sets the build file that should be used for this build.
+     *
+     * @param file The build file. May be null to use the default build file for the build.
+     * @since 4.0
+     */
+    public void setBuildFile(File file) {
+        setBuildFile((Object) file);
+    }
+
     /**
      * Sets the build file that should be used for this build.
      *
@@ -105,6 +125,16 @@ public class GradleBuild extends ConventionTask {
         return getStartParameter().getTaskNames();
     }
 
+    /**
+     * Sets the tasks that should be executed for this build.
+     *
+     * @param tasks The task names. May be empty or null to use the default tasks for the build.
+     * @since 4.0
+     */
+    public void setTasks(List<String> tasks) {
+        setTasks((Collection<String>) tasks);
+    }
+
     /**
      * Sets the tasks that should be executed for this build.
      *
@@ -116,11 +146,11 @@ public class GradleBuild extends ConventionTask {
 
     @TaskAction
     void build() {
-        GradleLauncher launcher = nestedBuildFactory.nestedInstanceWithNewSession(getStartParameter());
+        BuildController buildController = nestedBuildFactory.nestedBuildController(getStartParameter());
         try {
-            launcher.run();
+            buildController.run();
         } finally {
-            launcher.stop();
+            buildController.stop();
         }
     }
 }
