@@ -17,73 +17,78 @@
 package org.gradle.initialization;
 
 import com.google.common.collect.ImmutableList;
+import org.gradle.BuildAdapter;
+import org.gradle.BuildResult;
 import org.gradle.StartParameter;
 import org.gradle.api.internal.ExceptionAnalyser;
 import org.gradle.api.internal.GradleInternal;
-import org.gradle.api.internal.changedetection.state.GlobalScopeFileTimeStampInspector;
+import org.gradle.api.internal.tasks.execution.statistics.TaskExecutionStatisticsEventAdapter;
 import org.gradle.api.logging.Logging;
-import org.gradle.api.logging.StandardOutputListener;
 import org.gradle.api.logging.configuration.ShowStacktrace;
-import org.gradle.caching.internal.tasks.TaskExecutionStatisticsEventAdapter;
 import org.gradle.configuration.BuildConfigurer;
-import org.gradle.deployment.internal.DeploymentRegistry;
+import org.gradle.deployment.internal.DefaultDeploymentRegistry;
 import org.gradle.execution.BuildConfigurationActionExecuter;
 import org.gradle.execution.BuildExecuter;
 import org.gradle.internal.buildevents.BuildLogger;
-import org.gradle.internal.buildevents.CacheStatisticsReporter;
+import org.gradle.internal.buildevents.BuildStartedTime;
+import org.gradle.internal.buildevents.ProjectEvaluationLogger;
 import org.gradle.internal.buildevents.TaskExecutionLogger;
+import org.gradle.internal.buildevents.TaskExecutionStatisticsReporter;
 import org.gradle.internal.classpath.ClassPath;
-import org.gradle.internal.cleanup.BuildOutputCleanupListener;
 import org.gradle.internal.concurrent.Stoppable;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.featurelifecycle.LoggingDeprecatedFeatureHandler;
 import org.gradle.internal.featurelifecycle.ScriptUsageLocationReporter;
-import org.gradle.internal.logging.LoggingManagerInternal;
+import org.gradle.internal.invocation.BuildController;
+import org.gradle.internal.invocation.GradleBuildController;
 import org.gradle.internal.logging.progress.ProgressLoggerFactory;
 import org.gradle.internal.logging.text.StyledTextOutputFactory;
-import org.gradle.internal.progress.BuildOperationExecutor;
+import org.gradle.internal.operations.BuildOperationContext;
+import org.gradle.internal.operations.BuildOperationExecutor;
+import org.gradle.internal.operations.CallableBuildOperation;
+import org.gradle.internal.operations.notify.BuildOperationNotificationBridge;
+import org.gradle.internal.progress.BuildOperationDescriptor;
 import org.gradle.internal.progress.BuildProgressFilter;
 import org.gradle.internal.progress.BuildProgressLogger;
 import org.gradle.internal.progress.LoggerProvider;
 import org.gradle.internal.reflect.Instantiator;
-import org.gradle.internal.scan.BuildScanRequest;
-import org.gradle.internal.scan.BuildScanRequestListener;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.BuildScopeServices;
 import org.gradle.internal.service.scopes.BuildSessionScopeServices;
+import org.gradle.internal.service.scopes.BuildTreeScopeServices;
 import org.gradle.internal.service.scopes.GradleUserHomeScopeServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
+import org.gradle.internal.time.Clock;
+import org.gradle.internal.time.Time;
 import org.gradle.invocation.DefaultGradle;
 import org.gradle.profile.ProfileEventAdapter;
 import org.gradle.profile.ReportGeneratingProfileListener;
 import org.gradle.util.DeprecationLogger;
 
+import javax.annotation.Nullable;
 import java.util.List;
 
 public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
-    private final ListenerManager globalListenerManager;
     private final GradleUserHomeScopeServiceRegistry userHomeDirServiceRegistry;
     private final BuildProgressLogger buildProgressLogger;
     private DefaultGradleLauncher rootBuild;
 
     public DefaultGradleLauncherFactory(
         ListenerManager listenerManager, ProgressLoggerFactory progressLoggerFactory, GradleUserHomeScopeServiceRegistry userHomeDirServiceRegistry) {
-        this.globalListenerManager = listenerManager;
         this.userHomeDirServiceRegistry = userHomeDirServiceRegistry;
 
         // Register default loggers
         buildProgressLogger = new BuildProgressLogger(progressLoggerFactory);
         listenerManager.addListener(new BuildProgressFilter(buildProgressLogger));
-        listenerManager.useLogger(new DependencyResolutionLogger(progressLoggerFactory));
+        listenerManager.useLogger(new ProjectEvaluationLogger(progressLoggerFactory));
     }
 
-    private GradleLauncher createChildInstance(StartParameter startParameter, GradleLauncher parent, BuildSessionScopeServices buildSessionScopeServices, List<?> servicesToStop) {
+    private GradleLauncher createChildInstance(StartParameter startParameter, GradleLauncher parent, BuildTreeScopeServices buildTreeScopeServices, List<?> servicesToStop) {
         ServiceRegistry services = parent.getGradle().getServices();
         BuildRequestMetaData requestMetaData = new DefaultBuildRequestMetaData(services.get(BuildClientMetaData.class));
         BuildCancellationToken cancellationToken = services.get(BuildCancellationToken.class);
         BuildEventConsumer buildEventConsumer = services.get(BuildEventConsumer.class);
-
-        return doNewInstance(startParameter, parent, cancellationToken, requestMetaData, buildEventConsumer, buildSessionScopeServices, servicesToStop);
+        return doNewInstance(startParameter, parent, cancellationToken, requestMetaData, buildEventConsumer, buildTreeScopeServices, servicesToStop);
     }
 
     @Override
@@ -93,79 +98,68 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
             throw new IllegalStateException("Cannot have a current build");
         }
 
-        if (!(parentRegistry instanceof BuildSessionScopeServices)) {
-            throw new IllegalArgumentException("Service registry must be of build session scope");
+        if (!(parentRegistry instanceof BuildTreeScopeServices)) {
+            throw new IllegalArgumentException("Service registry must be of build-tree scope");
         }
-        BuildSessionScopeServices sessionScopeServices = (BuildSessionScopeServices) parentRegistry;
+        BuildTreeScopeServices buildTreeScopeServices = (BuildTreeScopeServices) parentRegistry;
 
-        DefaultGradleLauncher launcher = doNewInstance(startParameter, null, requestContext.getCancellationToken(), requestContext, requestContext.getEventConsumer(), sessionScopeServices, ImmutableList.of(new Stoppable() {
-            @Override
-            public void stop() {
-                rootBuild = null;
-            }
-        }));
+        DefaultGradleLauncher launcher = doNewInstance(startParameter, null,
+            requestContext.getCancellationToken(),
+            requestContext, requestContext.getEventConsumer(), buildTreeScopeServices,
+            ImmutableList.of(new Stoppable() {
+                @Override
+                public void stop() {
+                    rootBuild = null;
+                }
+            }));
         rootBuild = launcher;
 
-        DeploymentRegistry deploymentRegistry = parentRegistry.get(DeploymentRegistry.class);
-        deploymentRegistry.onNewBuild(launcher.getGradle());
+        final DefaultDeploymentRegistry deploymentRegistry = parentRegistry.get(DefaultDeploymentRegistry.class);
+        launcher.getGradle().addBuildListener(new BuildAdapter() {
+            @Override
+            public void buildFinished(BuildResult result) {
+                deploymentRegistry.buildFinished(result);
+            }
+        });
 
-        GlobalScopeFileTimeStampInspector timeStampInspector = sessionScopeServices.get(GlobalScopeFileTimeStampInspector.class);
-        launcher.addListener(timeStampInspector);
+        // Start collecting operations for this build invocation
+        parentRegistry.get(BuildOperationNotificationBridge.class).start(launcher.getGradle());
 
         return launcher;
     }
 
     private DefaultGradleLauncher doNewInstance(StartParameter startParameter, GradleLauncher parent,
-                                                BuildCancellationToken cancellationToken, BuildRequestMetaData requestMetaData, BuildEventConsumer buildEventConsumer, final BuildSessionScopeServices sessionScopeServices, List<?> servicesToStop) {
-        BuildScopeServices serviceRegistry = BuildScopeServices.forSession(sessionScopeServices);
+                                                BuildCancellationToken cancellationToken,
+                                                BuildRequestMetaData requestMetaData, BuildEventConsumer buildEventConsumer,
+                                                final BuildTreeScopeServices buildTreeScopeServices, List<?> servicesToStop) {
+        BuildScopeServices serviceRegistry = new BuildScopeServices(buildTreeScopeServices);
         serviceRegistry.add(BuildRequestMetaData.class, requestMetaData);
         serviceRegistry.add(BuildClientMetaData.class, requestMetaData.getClient());
         serviceRegistry.add(BuildEventConsumer.class, buildEventConsumer);
         serviceRegistry.add(BuildCancellationToken.class, cancellationToken);
-        NestedBuildFactoryImpl nestedBuildFactory = new NestedBuildFactoryImpl(sessionScopeServices);
+        NestedBuildFactoryImpl nestedBuildFactory = new NestedBuildFactoryImpl(buildTreeScopeServices);
         serviceRegistry.add(NestedBuildFactory.class, nestedBuildFactory);
 
         ListenerManager listenerManager = serviceRegistry.get(ListenerManager.class);
-        LoggingManagerInternal loggingManager = serviceRegistry.newInstance(LoggingManagerInternal.class);
-        loggingManager.setLevelInternal(startParameter.getLogLevel());
-
-        //this hooks up the ListenerManager and LoggingConfigurer so you can call Gradle.addListener() with a StandardOutputListener.
-        loggingManager.addStandardOutputListener(listenerManager.getBroadcaster(StandardOutputListener.class));
-        loggingManager.addStandardErrorListener(listenerManager.getBroadcaster(StandardOutputListener.class));
 
         LoggerProvider loggerProvider = (parent == null) ? buildProgressLogger : LoggerProvider.NO_OP;
         listenerManager.useLogger(new TaskExecutionLogger(serviceRegistry.get(ProgressLoggerFactory.class), loggerProvider));
         if (parent == null) {
-            listenerManager.useLogger(new BuildLogger(Logging.getLogger(BuildLogger.class), serviceRegistry.get(StyledTextOutputFactory.class), startParameter, requestMetaData));
+            BuildStartedTime buildStartedTime = serviceRegistry.get(BuildStartedTime.class);
+            Clock clock = serviceRegistry.get(Clock.class);
+            listenerManager.useLogger(new BuildLogger(Logging.getLogger(BuildLogger.class), serviceRegistry.get(StyledTextOutputFactory.class), startParameter, requestMetaData, buildStartedTime, clock));
         }
 
-        if (startParameter.isTaskOutputCacheEnabled()) {
-            listenerManager.addListener(serviceRegistry.get(TaskExecutionStatisticsEventAdapter.class));
-            listenerManager.addListener(new CacheStatisticsReporter(serviceRegistry.get(StyledTextOutputFactory.class)));
-        }
+        listenerManager.addListener(serviceRegistry.get(TaskExecutionStatisticsEventAdapter.class));
+        listenerManager.addListener(new TaskExecutionStatisticsReporter(serviceRegistry.get(StyledTextOutputFactory.class)));
 
         listenerManager.addListener(serviceRegistry.get(ProfileEventAdapter.class));
         if (startParameter.isProfile()) {
-            listenerManager.addListener(new ReportGeneratingProfileListener());
-        }
-        BuildScanRequest buildScanRequest = serviceRegistry.get(BuildScanRequest.class);
-        if (startParameter.isBuildScan()) {
-            if(!startParameter.getSystemPropertiesArgs().containsKey("scan")){
-                startParameter.getSystemPropertiesArgs().put("scan", "true");
-            }
-            buildScanRequest.markRequested();
-            listenerManager.addListener(new BuildScanRequestListener());
-        }
-        if (startParameter.isNoBuildScan()) {
-            if(!startParameter.getSystemPropertiesArgs().containsKey("scan")){
-                startParameter.getSystemPropertiesArgs().put("scan", "false");
-            }
-            buildScanRequest.markDisabled();
+            listenerManager.addListener(new ReportGeneratingProfileListener(serviceRegistry.get(StyledTextOutputFactory.class)));
         }
+
         ScriptUsageLocationReporter usageLocationReporter = new ScriptUsageLocationReporter();
         listenerManager.addListener(usageLocationReporter);
-        BuildOutputCleanupListener buildOutputCleanupListener = serviceRegistry.get(BuildOutputCleanupListener.class);
-        listenerManager.addListener(buildOutputCleanupListener);
         ShowStacktrace showStacktrace = startParameter.getShowStacktrace();
         switch (showStacktrace) {
             case ALWAYS:
@@ -186,9 +180,9 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
             gradle,
             serviceRegistry.get(InitScriptHandler.class),
             settingsLoader,
+            serviceRegistry.get(BuildLoader.class),
             serviceRegistry.get(BuildConfigurer.class),
             serviceRegistry.get(ExceptionAnalyser.class),
-            loggingManager,
             gradle.getBuildListenerBroadcaster(),
             listenerManager.getBroadcaster(ModelConfigurationListener.class),
             listenerManager.getBroadcaster(BuildCompletionListener.class),
@@ -196,7 +190,6 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
             gradle.getServices().get(BuildConfigurationActionExecuter.class),
             gradle.getServices().get(BuildExecuter.class),
             serviceRegistry,
-            globalListenerManager,
             servicesToStop
         );
         nestedBuildFactory.setParent(gradleLauncher);
@@ -204,31 +197,90 @@ public class DefaultGradleLauncherFactory implements GradleLauncherFactory {
     }
 
     private class NestedBuildFactoryImpl implements NestedBuildFactory {
-        private final BuildSessionScopeServices sessionScopeServices;
+        private final BuildTreeScopeServices buildTreeScopeServices;
         private DefaultGradleLauncher parent;
 
-        public NestedBuildFactoryImpl(BuildSessionScopeServices sessionScopeServices) {
-            this.sessionScopeServices = sessionScopeServices;
+        public NestedBuildFactoryImpl(BuildTreeScopeServices buildTreeScopeServices) {
+            this.buildTreeScopeServices = buildTreeScopeServices;
         }
 
         @Override
         public GradleLauncher nestedInstance(StartParameter startParameter) {
-            return createChildInstance(startParameter, parent, sessionScopeServices, ImmutableList.of());
+            return createChildInstance(startParameter, parent, buildTreeScopeServices, ImmutableList.of());
         }
 
         @Override
-        public GradleLauncher nestedInstanceWithNewSession(StartParameter startParameter) {
+        public BuildController nestedBuildController(StartParameter startParameter) {
             final ServiceRegistry userHomeServices = userHomeDirServiceRegistry.getServicesFor(startParameter.getGradleUserHomeDir());
-            BuildSessionScopeServices sessionScopeServices = new BuildSessionScopeServices(userHomeServices, startParameter, ClassPath.EMPTY);
-            return createChildInstance(startParameter, parent, sessionScopeServices, ImmutableList.of(sessionScopeServices, new Stoppable() {
+            BuildRequestMetaData buildRequestMetaData = new DefaultBuildRequestMetaData(Time.currentTimeMillis());
+            BuildSessionScopeServices sessionScopeServices = new BuildSessionScopeServices(userHomeServices, startParameter, buildRequestMetaData, ClassPath.EMPTY);
+            BuildTreeScopeServices buildTreeScopeServices = new BuildTreeScopeServices(sessionScopeServices);
+            GradleLauncher childInstance = createChildInstance(startParameter, parent, buildTreeScopeServices, ImmutableList.of(buildTreeScopeServices, sessionScopeServices, new Stoppable() {
                 @Override
                 public void stop() {
                     userHomeDirServiceRegistry.release(userHomeServices);
-                }}));
+                }
+            }));
+            return new NestedBuildController(new GradleBuildController(childInstance));
         }
 
         public void setParent(DefaultGradleLauncher parent) {
             this.parent = parent;
         }
+
+        private class NestedBuildController implements BuildController {
+            private final BuildController delegate;
+
+            NestedBuildController(BuildController delegate) {
+                this.delegate = delegate;
+            }
+
+            @Override
+            public void stop() {
+                delegate.stop();
+            }
+
+            @Override
+            public GradleInternal getGradle() {
+                return delegate.getGradle();
+            }
+
+            @Override
+            public GradleInternal run() {
+                BuildOperationExecutor executor = getGradle().getServices().get(BuildOperationExecutor.class);
+                return executor.call(new CallableBuildOperation<GradleInternal>() {
+                    @Override
+                    public GradleInternal call(BuildOperationContext context) {
+                        return delegate.run();
+                    }
+
+                    @Override
+                    public BuildOperationDescriptor.Builder description() {
+                        return BuildOperationDescriptor.displayName("Run nested build").parent(parent.getGradle().getBuildOperation());
+                    }
+                });
+            }
+
+            @Override
+            public GradleInternal configure() {
+                throw new UnsupportedOperationException();
+            }
+
+            @Override
+            public boolean hasResult() {
+                return delegate.hasResult();
+            }
+
+            @Nullable
+            @Override
+            public Object getResult() {
+                return delegate.getResult();
+            }
+
+            @Override
+            public void setResult(@Nullable Object result) {
+                delegate.setResult(result);
+            }
+        }
     }
 }
