@@ -15,14 +15,14 @@
  */
 package org.gradle.initialization;
 
-import com.google.common.collect.Maps;
 import org.gradle.StartParameter;
 import org.gradle.api.Action;
+import org.gradle.api.InvalidUserDataException;
 import org.gradle.api.UnknownProjectException;
 import org.gradle.api.initialization.ConfigurableIncludedBuild;
-import org.gradle.api.initialization.IncludedBuild;
 import org.gradle.api.initialization.ProjectDescriptor;
 import org.gradle.api.initialization.Settings;
+import org.gradle.api.initialization.dsl.ScriptHandler;
 import org.gradle.api.internal.GradleInternal;
 import org.gradle.api.internal.SettingsInternal;
 import org.gradle.api.internal.file.FileResolver;
@@ -32,16 +32,22 @@ import org.gradle.api.internal.plugins.DefaultObjectConfigurationAction;
 import org.gradle.api.internal.plugins.PluginManagerInternal;
 import org.gradle.api.internal.project.AbstractPluginAware;
 import org.gradle.api.internal.project.ProjectRegistry;
+import org.gradle.caching.configuration.BuildCacheConfiguration;
+import org.gradle.composite.internal.IncludedBuildRegistry;
 import org.gradle.configuration.ScriptPluginFactory;
 import org.gradle.groovy.scripts.ScriptSource;
 import org.gradle.internal.Actions;
-import org.gradle.internal.Cast;
+import org.gradle.internal.resource.TextResourceLoader;
+import org.gradle.internal.scripts.ScriptFileResolver;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.scopes.ServiceRegistryFactory;
+import org.gradle.plugin.management.PluginManagementSpec;
+import org.gradle.vcs.SourceControl;
 
 import javax.inject.Inject;
 import java.io.File;
-import java.util.Map;
+
+import static org.gradle.util.NameValidator.asValidName;
 
 public class DefaultSettings extends AbstractPluginAware implements SettingsInternal {
     public static final String DEFAULT_BUILD_SRC_DIR = "buildSrc";
@@ -59,20 +65,21 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
 
     private final ClassLoaderScope settingsClassLoaderScope;
     private final ClassLoaderScope buildRootClassLoaderScope;
+    private final ScriptHandler scriptHandler;
     private final ServiceRegistry services;
-    private final Map<File, ConfigurableIncludedBuild> includedBuilds = Maps.newLinkedHashMap();
 
     public DefaultSettings(ServiceRegistryFactory serviceRegistryFactory, GradleInternal gradle,
-                           ClassLoaderScope settingsClassLoaderScope, ClassLoaderScope buildRootClassLoaderScope, File settingsDir,
-                           ScriptSource settingsScript, StartParameter startParameter) {
+                           ClassLoaderScope settingsClassLoaderScope, ClassLoaderScope buildRootClassLoaderScope, ScriptHandler settingsScriptHandler,
+                           File settingsDir, ScriptSource settingsScript, StartParameter startParameter) {
         this.gradle = gradle;
         this.buildRootClassLoaderScope = buildRootClassLoaderScope;
+        this.scriptHandler = settingsScriptHandler;
         this.settingsDir = settingsDir;
         this.settingsScript = settingsScript;
         this.startParameter = startParameter;
         this.settingsClassLoaderScope = settingsClassLoaderScope;
         services = serviceRegistryFactory.createFor(this);
-        rootProjectDescriptor = createProjectDescriptor(null, settingsDir.getName(), settingsDir);
+        rootProjectDescriptor = createProjectDescriptor(null, asValidName(settingsDir.getName()), settingsDir);
     }
 
     @Override
@@ -88,8 +95,13 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         return this;
     }
 
+    @Override
+    public ScriptHandler getBuildscript() {
+        return scriptHandler;
+    }
+
     public DefaultProjectDescriptor createProjectDescriptor(DefaultProjectDescriptor parent, String name, File dir) {
-        return new DefaultProjectDescriptor(parent, name, dir, getProjectDescriptorRegistry(), getFileResolver());
+        return new DefaultProjectDescriptor(parent, name, dir, getProjectDescriptorRegistry(), getFileResolver(), getScriptFileResolver());
     }
 
     public DefaultProjectDescriptor findProject(String path) {
@@ -116,7 +128,7 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         return projectDescriptor;
     }
 
-    public void include(String[] projectPaths) {
+    public void include(String... projectPaths) {
         for (String projectPath : projectPaths) {
             String subPath = "";
             String[] pathElements = removeTrailingColon(projectPath).split(":");
@@ -133,10 +145,10 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         }
     }
 
-    public void includeFlat(String[] projectNames) {
+    public void includeFlat(String... projectNames) {
         for (String projectName : projectNames) {
             createProjectDescriptor(rootProjectDescriptor, projectName,
-                    new File(rootProjectDescriptor.getProjectDir().getParentFile(), projectName));
+                new File(rootProjectDescriptor.getProjectDir().getParentFile(), projectName));
         }
     }
 
@@ -202,7 +214,7 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
 
     @Override
     protected DefaultObjectConfigurationAction createObjectConfigurationAction() {
-        return new DefaultObjectConfigurationAction(getFileResolver(), getScriptPluginFactory(), getScriptHandlerFactory(), getRootClassLoaderScope(), this);
+        return new DefaultObjectConfigurationAction(getFileResolver(), getScriptPluginFactory(), getScriptHandlerFactory(), getRootClassLoaderScope(), getResourceLoader(), this);
     }
 
     public ClassLoaderScope getRootClassLoaderScope() {
@@ -217,6 +229,16 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
         return services;
     }
 
+    @Inject
+    protected ScriptFileResolver getScriptFileResolver() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected TextResourceLoader getResourceLoader() {
+        throw new UnsupportedOperationException();
+    }
+
     @Inject
     protected ScriptHandlerFactory getScriptHandlerFactory() {
         throw new UnsupportedOperationException();
@@ -233,7 +255,12 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
     }
 
     @Inject
-    protected IncludedBuildFactory getIncludedBuildFactory() {
+    protected IncludedBuildRegistry getIncludedBuildRegistry() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Inject
+    protected NestedBuildFactory getNestedBuildFactory() {
         throw new UnsupportedOperationException();
     }
 
@@ -249,17 +276,45 @@ public class DefaultSettings extends AbstractPluginAware implements SettingsInte
 
     @Override
     public void includeBuild(Object rootProject, Action<ConfigurableIncludedBuild> configuration) {
-        File projectDir = getFileResolver().resolve(rootProject);
-        ConfigurableIncludedBuild build = includedBuilds.get(projectDir);
-        if (build == null) {
-            build = getIncludedBuildFactory().createBuild(projectDir);
-            includedBuilds.put(projectDir, build);
+        if (gradle.getParent() == null) {
+            File projectDir = getFileResolver().resolve(rootProject);
+            ConfigurableIncludedBuild includedBuild = getIncludedBuildRegistry().addExplicitBuild(projectDir, getNestedBuildFactory());
+            configuration.execute(includedBuild);
+        } else {
+            throw new InvalidUserDataException(String.format("Included build '%s' cannot have included builds.", getRootProject().getName()));
         }
-        configuration.execute(build);
     }
 
     @Override
-    public Map<File, IncludedBuild> getIncludedBuilds() {
-        return Cast.uncheckedCast(includedBuilds);
+    public void buildCache(Action<? super BuildCacheConfiguration> action) {
+        action.execute(getBuildCache());
+    }
+
+    @Override
+    @Inject
+    public BuildCacheConfiguration getBuildCache() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void pluginManagement(Action<? super PluginManagementSpec> rule) {
+        rule.execute(getPluginManagement());
+    }
+
+    @Override
+    @Inject
+    public PluginManagementSpec getPluginManagement() {
+        throw new UnsupportedOperationException();
+    }
+
+    @Override
+    public void sourceControl(Action<? super SourceControl> configuration) {
+        configuration.execute(getSourceControl());
+    }
+
+    @Override
+    @Inject
+    public SourceControl getSourceControl() {
+        throw new UnsupportedOperationException();
     }
 }
