@@ -17,6 +17,7 @@
 package org.gradle.initialization;
 
 import org.gradle.StartParameter;
+import org.gradle.internal.invocation.BuildController;
 
 public interface NestedBuildFactory {
     /**
@@ -25,7 +26,8 @@ public interface NestedBuildFactory {
     GradleLauncher nestedInstance(StartParameter startParameter);
 
     /**
-     * Creates a nested {@link GradleLauncher} instance with the provided parameters, with a new session.
+     * Creates a {@link BuildController} for nested build instance with the provided parameters.
+     * The nested build will be created with a new session.
      */
-    GradleLauncher nestedInstanceWithNewSession(StartParameter startParameter);
+    BuildController nestedBuildController(StartParameter startParameter);
 }
