@@ -17,6 +17,7 @@
 package org.gradle.initialization;
 
 import org.gradle.StartParameter;
+import org.gradle.api.internal.StartParameterInternal;
 import org.gradle.api.logging.LogLevel;
 import org.gradle.api.logging.configuration.ConsoleOutput;
 import org.gradle.api.logging.configuration.ShowStacktrace;
@@ -24,7 +25,12 @@ import org.gradle.test.fixtures.file.TestFile;
 import org.gradle.util.WrapUtil;
 
 import java.io.File;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Set;
 
 import static org.junit.Assert.assertEquals;
 
@@ -46,7 +52,7 @@ public class CommandLineConverterTestSupport {
     protected ShowStacktrace expectedShowStackTrace = ShowStacktrace.INTERNAL_EXCEPTIONS;
     protected LogLevel expectedLogLevel = LogLevel.LIFECYCLE;
     protected ConsoleOutput expectedConsoleOutput = ConsoleOutput.Auto;
-    protected StartParameter actualStartParameter;
+    protected StartParameterInternal actualStartParameter;
     protected boolean expectedProfile;
     protected File expectedProjectCacheDir;
     protected boolean expectedRefreshDependencies;
@@ -61,7 +67,7 @@ public class CommandLineConverterTestSupport {
     protected boolean expectedContinuous;
 
     protected void checkConversion(String... args) {
-        actualStartParameter = new StartParameter();
+        actualStartParameter = new StartParameterInternal();
         actualStartParameter.setCurrentDir(currentDir);
         commandLineConverter.convert(Arrays.asList(args), actualStartParameter);
         // We check the params passed to the build factory
