@@ -17,6 +17,7 @@
 package org.gradle.api.file;
 
 import org.gradle.api.internal.file.CompositeFileCollection;
+import org.gradle.api.internal.file.TestFiles;
 import org.gradle.api.internal.file.UnionFileCollection;
 import org.gradle.api.internal.file.collections.DefaultConfigurableFileCollection;
 import org.gradle.api.internal.file.collections.DefaultFileCollectionResolveContext;
@@ -56,7 +57,7 @@ public class FileCollectionMatchers {
                 }
                 if (expected instanceof CompositeFileCollection) {
                     CompositeFileCollection collection = (CompositeFileCollection) expected;
-                    DefaultFileCollectionResolveContext context = new DefaultFileCollectionResolveContext();
+                    DefaultFileCollectionResolveContext context = new DefaultFileCollectionResolveContext(TestFiles.resolver());
                     collection.visitContents(context);
                     return context.resolveAsFileCollections();
                 }
