@@ -20,7 +20,13 @@ import org.gradle.api.internal.file.collections.DirectoryFileTreeFactory;
 import org.gradle.api.tasks.util.PatternSet;
 import org.gradle.api.tasks.util.internal.PatternSets;
 import org.gradle.internal.Factory;
+import org.gradle.internal.hash.DefaultContentHasherFactory;
+import org.gradle.internal.hash.DefaultFileHasher;
+import org.gradle.internal.hash.DefaultStreamHasher;
 import org.gradle.internal.nativeintegration.filesystem.FileSystem;
+import org.gradle.internal.reflect.DirectInstantiator;
+import org.gradle.internal.resource.local.FileResourceConnector;
+import org.gradle.internal.resource.local.FileResourceRepository;
 import org.gradle.process.internal.DefaultExecActionFactory;
 import org.gradle.process.internal.ExecActionFactory;
 import org.gradle.process.internal.ExecHandleFactory;
@@ -41,6 +47,10 @@ public class TestFiles {
         return FILE_SYSTEM;
     }
 
+    public static FileResourceRepository fileRepository() {
+        return new FileResourceConnector(FILE_SYSTEM);
+    }
+
     /**
      * Returns a resolver with no base directory.
      */
@@ -59,6 +69,18 @@ public class TestFiles {
         return new DefaultDirectoryFileTreeFactory(getPatternSetFactory(), fileSystem());
     }
 
+    public static FileOperations fileOperations(File basedDir) {
+        return new DefaultFileOperations(resolver(basedDir), null, null, DirectInstantiator.INSTANCE, fileLookup(), directoryFileTreeFactory(), streamHasher(), fileHasher());
+    }
+
+    public static DefaultStreamHasher streamHasher() {
+        return new DefaultStreamHasher(new DefaultContentHasherFactory());
+    }
+
+    public static DefaultFileHasher fileHasher() {
+        return new DefaultFileHasher(streamHasher());
+    }
+
     public static FileCollectionFactory fileCollectionFactory() {
         return new DefaultFileCollectionFactory();
     }
@@ -90,4 +112,8 @@ public class TestFiles {
     public static Factory<PatternSet> getPatternSetFactory() {
         return resolver().getPatternSetFactory();
     }
+
+    public static String systemSpecificAbsolutePath(String path) {
+        return new File(path).getAbsolutePath();
+    }
 }
