@@ -21,19 +21,20 @@ import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
 
 public class DefaultModuleVersionIdentifier implements ModuleVersionIdentifier {
 
-    private final DefaultModuleIdentifier id;
+    private final ModuleIdentifier id;
     private final String version;
 
     public DefaultModuleVersionIdentifier(String group, String name, String version) {
         assert group != null : "group cannot be null";
         assert name != null : "name cannot be null";
         assert version != null : "version cannot be null";
-        this.id = new DefaultModuleIdentifier(group, name);
+        this.id = DefaultModuleIdentifier.newId(group, name);
         this.version = version;
     }
 
     public DefaultModuleVersionIdentifier(ModuleIdentifier id, String version) {
-        this.id = new DefaultModuleIdentifier(id.getGroup(), id.getName());
+        assert version != null : "version cannot be null";
+        this.id = id;
         this.version = version;
     }
 
@@ -53,13 +54,7 @@ public class DefaultModuleVersionIdentifier implements ModuleVersionIdentifier {
     public String toString() {
         String group = id.getGroup();
         String module = id.getName();
-        StringBuilder builder = new StringBuilder(group.length() + module.length() + version.length() + 2);
-        builder.append(group);
-        builder.append(":");
-        builder.append(module);
-        builder.append(":");
-        builder.append(version);
-        return builder.toString();
+        return group + ":" + module + ":" + version;
     }
 
     @Override
@@ -82,7 +77,7 @@ public class DefaultModuleVersionIdentifier implements ModuleVersionIdentifier {
 
     @Override
     public int hashCode() {
-        return id.hashCode() ^ version.hashCode();
+        return 31 * id.hashCode() ^ version.hashCode();
     }
 
     public ModuleIdentifier getModule() {
