@@ -17,20 +17,35 @@
 package org.gradle.api.internal.artifacts.ivyservice;
 
 import com.google.common.collect.Sets;
-import org.gradle.api.Nullable;
 import org.gradle.api.artifacts.ResolvedArtifact;
-import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
+import org.gradle.api.attributes.AttributeContainer;
 import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ArtifactVisitor;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.artifact.ResolvableArtifact;
+import org.gradle.internal.UncheckedException;
 
 import java.io.File;
 import java.util.Set;
 
-class ArtifactCollectingVisitor implements ArtifactVisitor {
-    final Set<ResolvedArtifact> artifacts = Sets.newLinkedHashSet();
+public class ArtifactCollectingVisitor implements ArtifactVisitor {
+    private final Set<ResolvedArtifact> artifacts;
+
+    public ArtifactCollectingVisitor() {
+        this(Sets.<ResolvedArtifact>newLinkedHashSet());
+    }
+
+    public ArtifactCollectingVisitor(Set<ResolvedArtifact> artifacts) {
+        this.artifacts = artifacts;
+    }
 
     @Override
-    public void visitArtifact(ResolvedArtifact artifact) {
-        this.artifacts.add(artifact);
+    public void visitArtifact(AttributeContainer variant, ResolvableArtifact artifact) {
+        this.artifacts.add(artifact.toPublicView());
+    }
+
+    @Override
+    public void visitFailure(Throwable failure) {
+        throw UncheckedException.throwAsUncheckedException(failure);
     }
 
     @Override
@@ -39,7 +54,16 @@ class ArtifactCollectingVisitor implements ArtifactVisitor {
     }
 
     @Override
-    public void visitFiles(@Nullable ComponentIdentifier componentIdentifier, Iterable<File> files) {
+    public boolean requireArtifactFiles() {
+        return false;
+    }
+
+    @Override
+    public void visitFile(ComponentArtifactIdentifier artifactIdentifier, AttributeContainer variant, File file) {
         throw new UnsupportedOperationException();
     }
+
+    public Set<ResolvedArtifact> getArtifacts() {
+        return artifacts;
+    }
 }
