@@ -19,10 +19,11 @@ package org.gradle.api.internal.artifacts.ivyservice.dependencysubstitution;
 import org.gradle.api.Action;
 import org.gradle.api.artifacts.DependencyResolveDetails;
 import org.gradle.api.artifacts.DependencySubstitutions;
+import org.gradle.api.internal.artifacts.ComponentSelectorConverter;
 import org.gradle.api.internal.artifacts.configurations.MutationValidator;
 
 public interface DependencySubstitutionsInternal extends DependencySubstitutions, DependencySubstitutionRules {
-    DependencySubstitutions allWithDependencyResolveDetails(Action<? super DependencyResolveDetails> rule);
+    DependencySubstitutions allWithDependencyResolveDetails(Action<? super DependencyResolveDetails> rule, ComponentSelectorConverter componentSelectorConverter);
 
     void setMutationValidator(MutationValidator validator);
 
