@@ -15,11 +15,11 @@
  */
 package org.gradle.api.internal.artifacts.ivyservice.moduleconverter;
 
-import org.gradle.api.artifacts.Configuration;
+import org.gradle.api.internal.artifacts.configurations.ConfigurationInternal;
 import org.gradle.internal.component.local.model.BuildableLocalComponentMetadata;
 
 import java.util.Collection;
 
 public interface ConfigurationComponentMetaDataBuilder {
-    void addConfigurations(BuildableLocalComponentMetadata metaData, Collection<? extends Configuration> configurations);
+    void addConfigurations(BuildableLocalComponentMetadata metaData, Collection<? extends ConfigurationInternal> configurations);
 }
