@@ -16,21 +16,20 @@
 package org.gradle.api.internal.artifacts.ivyservice.resolveengine.oldresult;
 
 import org.gradle.api.artifacts.ModuleDependency;
-import org.gradle.api.artifacts.UnresolvedDependency;
-import org.gradle.api.internal.artifacts.ResolvedConfigurationIdentifier;
+import org.gradle.api.internal.artifacts.ivyservice.resolveengine.graph.DependencyGraphNode;
 
 //builds old model of resolved dependency graph based on the result events
 public interface ResolvedConfigurationBuilder {
 
-    void addFirstLevelDependency(ModuleDependency moduleDependency, ResolvedConfigurationIdentifier dependency);
+    void addFirstLevelDependency(ModuleDependency moduleDependency, DependencyGraphNode dependency);
 
-    void addUnresolvedDependency(UnresolvedDependency unresolvedDependency);
+    void addChild(DependencyGraphNode parent, DependencyGraphNode child, int artifactsId);
 
-    void addChild(ResolvedConfigurationIdentifier parent, ResolvedConfigurationIdentifier child, long artifactsId);
+    void addNodeArtifacts(DependencyGraphNode node, int artifactsId);
 
-    void newResolvedDependency(ResolvedConfigurationIdentifier id);
+    void newResolvedDependency(DependencyGraphNode node);
 
-    void done(ResolvedConfigurationIdentifier root);
+    void done(DependencyGraphNode root);
 
     ResolvedGraphResults complete();
 }
