@@ -15,15 +15,18 @@
  */
 package org.gradle.api.internal.artifacts.repositories.resolver;
 
-import org.gradle.api.Nullable;
-import org.gradle.internal.component.external.model.ModuleComponentArtifactMetaData;
+import org.gradle.internal.component.external.model.ModuleComponentArtifactMetadata;
+import org.gradle.internal.component.model.ModuleSource;
 import org.gradle.internal.resolve.result.ResourceAwareResolveResult;
 import org.gradle.internal.resource.local.LocallyAvailableExternalResource;
 
+import javax.annotation.Nullable;
+
 public interface ExternalResourceArtifactResolver {
+    ModuleSource getSource();
 
     @Nullable
-    LocallyAvailableExternalResource resolveArtifact(ModuleComponentArtifactMetaData artifact, ResourceAwareResolveResult result);
+    LocallyAvailableExternalResource resolveArtifact(ModuleComponentArtifactMetadata artifact, ResourceAwareResolveResult result);
 
-    boolean artifactExists(ModuleComponentArtifactMetaData artifact, ResourceAwareResolveResult result);
+    boolean artifactExists(ModuleComponentArtifactMetadata artifact, ResourceAwareResolveResult result);
 }
