@@ -15,31 +15,37 @@
  */
 package org.gradle.api.internal.artifacts.repositories.resolver;
 
-import org.gradle.internal.component.external.model.DefaultModuleComponentArtifactMetaData;
-import org.gradle.internal.component.external.model.ModuleComponentArtifactMetaData;
+import org.gradle.internal.component.external.model.DefaultModuleComponentArtifactMetadata;
+import org.gradle.internal.component.external.model.ModuleComponentArtifactMetadata;
+import org.gradle.internal.component.model.ModuleSource;
 import org.gradle.internal.resolve.result.ResourceAwareResolveResult;
 import org.gradle.internal.resource.local.LocallyAvailableExternalResource;
 
 class MavenUniqueSnapshotExternalResourceArtifactResolver implements ExternalResourceArtifactResolver {
     private final ExternalResourceArtifactResolver delegate;
-    private final String timestamp;
+    private final MavenUniqueSnapshotModuleSource snapshot;
 
-    public MavenUniqueSnapshotExternalResourceArtifactResolver(ExternalResourceArtifactResolver delegate, String timestamp) {
+    public MavenUniqueSnapshotExternalResourceArtifactResolver(ExternalResourceArtifactResolver delegate, MavenUniqueSnapshotModuleSource snapshot) {
         this.delegate = delegate;
-        this.timestamp = timestamp;
+        this.snapshot = snapshot;
     }
 
-    public boolean artifactExists(ModuleComponentArtifactMetaData artifact, ResourceAwareResolveResult result) {
+    @Override
+    public ModuleSource getSource() {
+        return snapshot;
+    }
+
+    public boolean artifactExists(ModuleComponentArtifactMetadata artifact, ResourceAwareResolveResult result) {
         return delegate.artifactExists(timestamp(artifact), result);
     }
 
-    public LocallyAvailableExternalResource resolveArtifact(ModuleComponentArtifactMetaData artifact, ResourceAwareResolveResult result) {
+    public LocallyAvailableExternalResource resolveArtifact(ModuleComponentArtifactMetadata artifact, ResourceAwareResolveResult result) {
         return delegate.resolveArtifact(timestamp(artifact), result);
     }
 
-    protected ModuleComponentArtifactMetaData timestamp(ModuleComponentArtifactMetaData artifact) {
+    protected ModuleComponentArtifactMetadata timestamp(ModuleComponentArtifactMetadata artifact) {
         MavenUniqueSnapshotComponentIdentifier snapshotComponentIdentifier =
-                new MavenUniqueSnapshotComponentIdentifier(artifact.getId().getComponentIdentifier(), timestamp);
-        return new DefaultModuleComponentArtifactMetaData(snapshotComponentIdentifier, artifact.getName());
+                new MavenUniqueSnapshotComponentIdentifier(artifact.getId().getComponentIdentifier(), snapshot.getTimestamp());
+        return new DefaultModuleComponentArtifactMetadata(snapshotComponentIdentifier, artifact.getName());
     }
 }
