@@ -16,15 +16,22 @@
 
 package org.gradle.api.internal.attributes;
 
-import org.gradle.api.attributes.AttributeContainer;
+import org.gradle.api.attributes.Attribute;
 import org.gradle.api.attributes.AttributesSchema;
-import org.gradle.api.attributes.HasAttributes;
-
-import java.util.List;
+import org.gradle.internal.component.model.AttributeMatcher;
 
 public interface AttributesSchemaInternal extends AttributesSchema {
+    /**
+     * Returns a matcher that uses the consumer rules from this schema, and the producer rules from the given schema.
+     */
+    AttributeMatcher withProducer(AttributesSchemaInternal producerSchema);
 
-    List<? extends HasAttributes> getMatches(AttributesSchema producerAttributeSchema, List<HasAttributes> candidates, AttributeContainer consumer);
+    /**
+     * Returns a matcher that uses the rules from this schema, and assumes the producer has the same rules.
+     */
+    AttributeMatcher matcher();
 
-    boolean isMatching(AttributeContainer candidate, AttributeContainer target, boolean incompleteCandidate);
+    CompatibilityRule<Object> compatibilityRules(Attribute<?> attribute);
+
+    DisambiguationRule<Object> disambiguationRules(Attribute<?> attribute);
 }
