@@ -23,9 +23,10 @@ import org.gradle.api.internal.artifacts.dsl.dependencies.DependencyFactory;
 import org.gradle.api.internal.file.FileCollectionInternal;
 import org.gradle.api.internal.file.FileResolver;
 import org.gradle.api.internal.file.collections.FileCollectionAdapter;
-import org.gradle.api.internal.impldeps.GradleImplDepsJarType;
-import org.gradle.api.internal.impldeps.GradleImplDepsProvider;
-import org.gradle.api.internal.impldeps.GradleImplDepsRelocatedJar;
+import org.gradle.api.internal.file.collections.SingletonFileSet;
+import org.gradle.api.internal.runtimeshaded.RuntimeShadedJarFactory;
+import org.gradle.api.internal.runtimeshaded.RuntimeShadedJarType;
+import org.gradle.internal.component.local.model.OpaqueComponentIdentifier;
 import org.gradle.internal.exceptions.DiagnosticsVisitor;
 import org.gradle.internal.installation.CurrentGradleInstallation;
 import org.gradle.internal.reflect.Instantiator;
@@ -34,6 +35,7 @@ import org.gradle.internal.typeconversion.NotationConverter;
 import org.gradle.internal.typeconversion.TypeConversionException;
 
 import java.io.File;
+import java.util.ArrayList;
 import java.util.Collection;
 import java.util.List;
 import java.util.Map;
@@ -47,7 +49,7 @@ public class DependencyClassPathNotationConverter implements NotationConverter<D
     private final ClassPathRegistry classPathRegistry;
     private final Instantiator instantiator;
     private final FileResolver fileResolver;
-    private final GradleImplDepsProvider gradleImplDepsProvider;
+    private final RuntimeShadedJarFactory runtimeShadedJarFactory;
     private final CurrentGradleInstallation currentGradleInstallation;
     private final Map<DependencyFactory.ClassPathNotation, SelfResolvingDependency> internCache = Maps.newEnumMap(DependencyFactory.ClassPathNotation.class);
     private final Lock internCacheWriteLock = new ReentrantLock();
@@ -56,12 +58,12 @@ public class DependencyClassPathNotationConverter implements NotationConverter<D
         Instantiator instantiator,
         ClassPathRegistry classPathRegistry,
         FileResolver fileResolver,
-        GradleImplDepsProvider gradleImplDepsProvider,
+        RuntimeShadedJarFactory runtimeShadedJarFactory,
         CurrentGradleInstallation currentGradleInstallation) {
         this.instantiator = instantiator;
         this.classPathRegistry = classPathRegistry;
         this.fileResolver = fileResolver;
-        this.gradleImplDepsProvider = gradleImplDepsProvider;
+        this.runtimeShadedJarFactory = runtimeShadedJarFactory;
         this.currentGradleInstallation = currentGradleInstallation;
     }
 
@@ -90,14 +92,14 @@ public class DependencyClassPathNotationConverter implements NotationConverter<D
             Collection<File> classpath = classPathRegistry.getClassPath(notation.name()).getAsFiles();
             boolean runningFromInstallation = currentGradleInstallation.getInstallation() != null;
             FileCollectionInternal files;
-            if (runningFromInstallation && notation.equals(FAT_GRADLE_API)) {
+            if (runningFromInstallation && notation.equals(GRADLE_API)) {
                 files = gradleApiFileCollection(classpath);
-            } else if (runningFromInstallation && notation.equals(FAT_GRADLE_TEST_KIT)) {
+            } else if (runningFromInstallation && notation.equals(GRADLE_TEST_KIT)) {
                 files = gradleTestKitFileCollection(classpath);
             } else {
                 files = fileResolver.resolveFiles(classpath);
             }
-            dependency = instantiator.newInstance(DefaultSelfResolvingDependency.class, files);
+            dependency = instantiator.newInstance(DefaultSelfResolvingDependency.class, new OpaqueComponentIdentifier(notation.displayName), files);
             internCache.put(notation, dependency);
         }
         return dependency;
@@ -106,25 +108,39 @@ public class DependencyClassPathNotationConverter implements NotationConverter<D
     private FileCollectionInternal gradleApiFileCollection(Collection<File> apiClasspath) {
         // Don't inline the Groovy jar as the Groovy “tools locator” searches for it by name
         List<File> groovyImpl = classPathRegistry.getClassPath(LOCAL_GROOVY.name()).getAsFiles();
+        List<File> installationBeacon = classPathRegistry.getClassPath("GRADLE_INSTALLATION_BEACON").getAsFiles();
         apiClasspath.removeAll(groovyImpl);
+        apiClasspath.removeAll(installationBeacon);
+        removeGradleScriptKotlin(apiClasspath);
 
-        return (FileCollectionInternal) relocatedDepsJar(apiClasspath, "fatGradleApi()", GradleImplDepsJarType.API)
-            .plus(fileResolver.resolveFiles(
-                groovyImpl,
-                classPathRegistry.getClassPath("GRADLE_INSTALLATION_BEACON").getAsFiles())
-            );
+        return (FileCollectionInternal) relocatedDepsJar(apiClasspath, "gradleApi()", RuntimeShadedJarType.API)
+            .plus(fileResolver.resolveFiles(groovyImpl, installationBeacon));
+    }
+
+    /**
+     * Gradle script kotlin should not be part of the public Gradle API
+     * We remove this in a very hacky way for 3.0. Going forward, there
+     * will be a cleaner solution
+     */
+    private void removeGradleScriptKotlin(Collection<File> apiClasspath) {
+        for (File file : new ArrayList<File>(apiClasspath)) {
+            // TODO: replace by something cleaner
+            if (file.getName().contains("kotlin")) {
+                apiClasspath.remove(file);
+            }
+        }
     }
 
     private FileCollectionInternal gradleTestKitFileCollection(Collection<File> testKitClasspath) {
         List<File> gradleApi = classPathRegistry.getClassPath(GRADLE_API.name()).getAsFiles();
         testKitClasspath.removeAll(gradleApi);
 
-        return (FileCollectionInternal) relocatedDepsJar(testKitClasspath, "fatGradleTestKit()", GradleImplDepsJarType.TEST_KIT)
+        return (FileCollectionInternal) relocatedDepsJar(testKitClasspath, "gradleTestKit()", RuntimeShadedJarType.TEST_KIT)
             .plus(gradleApiFileCollection(gradleApi));
     }
 
-    private FileCollectionInternal relocatedDepsJar(Collection<File> classpath, String displayName, GradleImplDepsJarType gradleImplDepsJarType) {
-        File gradleImplDepsJar = gradleImplDepsProvider.getFile(classpath, gradleImplDepsJarType);
-        return new FileCollectionAdapter(new GradleImplDepsRelocatedJar(displayName, gradleImplDepsJar));
+    private FileCollectionInternal relocatedDepsJar(Collection<File> classpath, String displayName, RuntimeShadedJarType runtimeShadedJarType) {
+        File gradleImplDepsJar = runtimeShadedJarFactory.get(runtimeShadedJarType, classpath);
+        return new FileCollectionAdapter(new SingletonFileSet(gradleImplDepsJar, displayName));
     }
 }
