@@ -16,13 +16,15 @@
 
 package org.gradle.internal.component.external.model;
 
-import org.gradle.api.Nullable;
 import org.gradle.api.artifacts.component.ModuleComponentIdentifier;
+import org.gradle.internal.DisplayName;
 import org.gradle.internal.component.model.DefaultIvyArtifactName;
 import org.gradle.internal.component.model.IvyArtifactName;
 import org.gradle.util.GUtil;
 
-public class DefaultModuleComponentArtifactIdentifier implements ModuleComponentArtifactIdentifier {
+import javax.annotation.Nullable;
+
+public class DefaultModuleComponentArtifactIdentifier implements ModuleComponentArtifactIdentifier, DisplayName {
     private final ModuleComponentIdentifier componentIdentifier;
     private final IvyArtifactName name;
 
@@ -65,6 +67,11 @@ public class DefaultModuleComponentArtifactIdentifier implements ModuleComponent
         return builder.toString();
     }
 
+    @Override
+    public String getCapitalizedDisplayName() {
+        return getDisplayName();
+    }
+
     public IvyArtifactName getName() {
         return name;
     }
