@@ -17,14 +17,20 @@
 package org.gradle.internal.component.local.model;
 
 import org.gradle.api.artifacts.ModuleDependency;
-import org.gradle.internal.component.model.LocalOriginDependencyMetadata;
+import org.gradle.internal.component.model.DependencyMetadata;
 
 /**
  * A dependency declared using the dependency DSL.
  */
-public interface DslOriginDependencyMetadata extends LocalOriginDependencyMetadata {
+public interface DslOriginDependencyMetadata extends DependencyMetadata {
+
     /**
-     * Remove this.
+     * Provide access to the source `Dependency` instance for this `DependencyMetadata`.
+     * This is used for:
+     * - Accessing the `ClientModule` instance for a 'client module' dependency.
+     * - Binding the source dependency to the first-order resolved components in `ResolvedConfiguration`.
+     *
+     * The goal is to eventually replace these uses, and remove this type.
      */
     ModuleDependency getSource();
 }
