@@ -16,13 +16,11 @@
 
 package org.gradle.internal.component.local.model;
 
-import org.gradle.api.Nullable;
-import org.gradle.api.attributes.AttributesSchema;
 import org.gradle.internal.component.model.ComponentResolveMetadata;
 
-public interface LocalComponentMetadata extends ComponentResolveMetadata {
-    AttributesSchema getAttributesSchema();
+import javax.annotation.Nullable;
 
+public interface LocalComponentMetadata extends ComponentResolveMetadata {
     @Nullable
     @Override
     LocalConfigurationMetadata getConfiguration(String name);
