@@ -16,11 +16,12 @@
 
 package org.gradle.internal.component.local.model;
 
-import org.gradle.api.Nullable;
 import org.gradle.api.artifacts.FileCollectionDependency;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
 import org.gradle.api.file.FileCollection;
 
+import javax.annotation.Nullable;
+
 /**
  * Represents a local file dependency. Should be modelled as a regular dependency, but is treated separately as a migration step.
  */
