@@ -18,6 +18,7 @@ package org.gradle.internal.component.model;
 
 import org.gradle.api.artifacts.component.ComponentArtifactIdentifier;
 import org.gradle.api.artifacts.component.ComponentIdentifier;
+import org.gradle.api.tasks.TaskDependency;
 
 /**
  * Meta-data for an artifact that belongs to some component.
@@ -37,4 +38,9 @@ public interface ComponentArtifactMetadata {
      * Returns this artifact as an Ivy artifact. This method is here to allow the artifact to be exposed in a backward-compatible way.
      */
     IvyArtifactName getName();
+
+    /**
+     * Collects the build dependencies of this artifact, which are required to build this artifact
+     */
+    TaskDependency getBuildDependencies();
 }
