@@ -17,25 +17,33 @@
 package org.gradle.internal.component.model;
 
 import org.gradle.api.internal.attributes.AttributeContainerInternal;
+import org.gradle.internal.DisplayName;
 
-import java.util.Set;
+import java.util.List;
 
 public class DefaultVariantMetadata implements VariantMetadata {
+    private final DisplayName displayName;
     private final AttributeContainerInternal attributes;
-    private final Set<? extends ComponentArtifactMetadata> artifacts;
+    private final List<? extends ComponentArtifactMetadata> artifacts;
 
-    public DefaultVariantMetadata(AttributeContainerInternal attributes, Set<? extends ComponentArtifactMetadata> artifacts) {
+    public DefaultVariantMetadata(DisplayName displayName, AttributeContainerInternal attributes, List<? extends ComponentArtifactMetadata> artifacts) {
+        this.displayName = displayName;
         this.attributes = attributes;
         this.artifacts = artifacts;
     }
 
+    @Override
+    public DisplayName asDescribable() {
+        return displayName;
+    }
+
     @Override
     public AttributeContainerInternal getAttributes() {
         return attributes;
     }
 
     @Override
-    public Set<? extends ComponentArtifactMetadata> getArtifacts() {
+    public List<? extends ComponentArtifactMetadata> getArtifacts() {
         return artifacts;
     }
 }
