@@ -32,22 +32,24 @@ import org.gradle.api.artifacts.component.ProjectComponentIdentifier;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.ConventionMapping;
 import org.gradle.api.internal.IConventionAware;
+import org.gradle.api.internal.artifacts.ivyservice.projectmodule.LocalComponentRegistry;
 import org.gradle.api.internal.artifacts.ivyservice.projectmodule.ProjectLocalComponentProvider;
 import org.gradle.api.internal.artifacts.publish.DefaultPublishArtifact;
 import org.gradle.api.internal.project.ProjectInternal;
-import org.gradle.api.invocation.Gradle;
 import org.gradle.api.plugins.JavaBasePlugin;
 import org.gradle.api.plugins.JavaPlugin;
 import org.gradle.api.plugins.JavaPluginConvention;
 import org.gradle.api.plugins.WarPlugin;
 import org.gradle.api.plugins.scala.ScalaBasePlugin;
 import org.gradle.api.tasks.SourceSetContainer;
+import org.gradle.api.tasks.TaskDependency;
+import org.gradle.initialization.ProjectPathRegistry;
 import org.gradle.internal.component.local.model.LocalComponentArtifactMetadata;
 import org.gradle.internal.component.local.model.PublishArtifactLocalArtifactMetadata;
 import org.gradle.internal.reflect.Instantiator;
+import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.language.scala.plugins.ScalaLanguagePlugin;
 import org.gradle.plugins.ide.api.XmlFileContentMerger;
-import org.gradle.plugins.ide.idea.internal.IdeaNameDeduper;
 import org.gradle.plugins.ide.idea.internal.IdeaScalaConfigurer;
 import org.gradle.plugins.ide.idea.model.IdeaLanguageLevel;
 import org.gradle.plugins.ide.idea.model.IdeaModel;
@@ -59,6 +61,9 @@ import org.gradle.plugins.ide.idea.model.PathFactory;
 import org.gradle.plugins.ide.idea.model.internal.GeneratedIdeaScope;
 import org.gradle.plugins.ide.idea.model.internal.IdeaDependenciesProvider;
 import org.gradle.plugins.ide.internal.IdePlugin;
+import org.gradle.plugins.ide.internal.configurer.UniqueProjectNameProvider;
+import org.gradle.util.Path;
+import org.gradle.util.SingleMessageLogger;
 
 import javax.inject.Inject;
 import java.io.File;
@@ -99,10 +104,12 @@ public class IdeaPlugin extends IdePlugin {
     private final Instantiator instantiator;
     private IdeaModel ideaModel;
     private List<Project> allJavaProjects;
+    private final UniqueProjectNameProvider uniqueProjectNameProvider;
 
     @Inject
-    public IdeaPlugin(Instantiator instantiator) {
+    public IdeaPlugin(Instantiator instantiator, UniqueProjectNameProvider uniqueProjectNameProvider) {
         this.instantiator = instantiator;
+        this.uniqueProjectNameProvider = uniqueProjectNameProvider;
     }
 
     public IdeaModel getModel() {
@@ -115,7 +122,7 @@ public class IdeaPlugin extends IdePlugin {
     }
 
     @Override
-    protected void onApply(Project project) {
+    protected void onApply(final Project project) {
         getLifecycleTask().setDescription("Generates IDEA project files (IML, IPR, IWS)");
         getCleanTask().setDescription("Cleans IDEA project files (IML, IPR)");
 
@@ -127,43 +134,26 @@ public class IdeaPlugin extends IdePlugin {
         configureForJavaPlugin(project);
         configureForWarPlugin(project);
         configureForScalaPlugin();
-        postProcess("idea", new Action<Gradle>() {
-            @Override
-            public void execute(Gradle gradle) {
-                performPostEvaluationActions();
-            }
-        });
+        registerImlArtifact(project);
+        linkCompositeBuildDependencies((ProjectInternal) project);
     }
 
+    // No one should be calling this.
+    @Deprecated
     public void performPostEvaluationActions() {
-        makeSureModuleNamesAreUnique();
-        // This needs to happen after de-duplication
-        registerImlArtifacts();
+        SingleMessageLogger.nagUserOfDiscontinuedMethod("performPostEvaluationActions");
     }
 
-    private void makeSureModuleNamesAreUnique() {
-        new IdeaNameDeduper().configureRoot(project.getRootProject());
-    }
-
-    private void registerImlArtifacts() {
-        Set<Project> projectsWithIml = Sets.filter(project.getRootProject().getAllprojects(), new Predicate<Project>() {
-            @Override
-            public boolean apply(Project project) {
-                return project.getPlugins().hasPlugin(IdeaPlugin.class);
-            }
-        });
-        for (Project project : projectsWithIml) {
-            ProjectLocalComponentProvider projectComponentProvider = ((ProjectInternal) project).getServices().get(ProjectLocalComponentProvider.class);
-            ProjectComponentIdentifier projectId = newProjectId(project);
-            projectComponentProvider.registerAdditionalArtifact(projectId, createImlArtifact(projectId, project));
-        }
+    private void registerImlArtifact(Project project) {
+        ProjectLocalComponentProvider projectComponentProvider = ((ProjectInternal) project).getServices().get(ProjectLocalComponentProvider.class);
+        ProjectComponentIdentifier projectId = newProjectId(project);
+        projectComponentProvider.registerAdditionalArtifact(projectId, createImlArtifact(projectId, project));
     }
 
     private static LocalComponentArtifactMetadata createImlArtifact(ProjectComponentIdentifier projectId, Project project) {
-        String moduleName = project.getExtensions().getByType(IdeaModel.class).getModule().getName();
-        File imlFile = new File(project.getProjectDir(), moduleName + ".iml");
+        IdeaModule module = project.getExtensions().getByType(IdeaModel.class).getModule();
         Task byName = project.getTasks().getByName("ideaModule");
-        PublishArtifact publishArtifact = new DefaultPublishArtifact(moduleName, "iml", "iml", null, null, imlFile, byName);
+        PublishArtifact publishArtifact = new ImlArtifact(module, byName);
         return new PublishArtifactLocalArtifactMetadata(projectId, publishArtifact);
     }
 
@@ -178,7 +168,6 @@ public class IdeaPlugin extends IdePlugin {
             task.setOutputFile(new File(project.getProjectDir(), project.getName() + ".iws"));
             addWorker(task, false);
         }
-
     }
 
     private void configureIdeaProject(final Project project) {
@@ -273,8 +262,11 @@ public class IdeaPlugin extends IdePlugin {
         IdeaModuleIml iml = new IdeaModuleIml(task.getXmlTransformer(), project.getProjectDir());
         final IdeaModule module = instantiator.newInstance(IdeaModule.class, project, iml);
         task.setModule(module);
-
         ideaModel.setModule(module);
+
+        final String defaultModuleName = uniqueProjectNameProvider.getUniqueName(project);
+        module.setName(defaultModuleName);
+
         ConventionMapping conventionMapping = ((IConventionAware) module).getConventionMapping();
         conventionMapping.map("sourceDirs", new Callable<Set<File>>() {
             @Override
@@ -282,12 +274,6 @@ public class IdeaPlugin extends IdePlugin {
                 return Sets.newHashSet();
             }
         });
-        conventionMapping.map("name", new Callable<String>() {
-            @Override
-            public String call() throws Exception {
-                return project.getName();
-            }
-        });
         conventionMapping.map("contentRoot", new Callable<File>() {
             @Override
             public File call() throws Exception {
@@ -487,8 +473,60 @@ public class IdeaPlugin extends IdePlugin {
         project.getTasks().findByName("ideaModule").dependsOn(project.getRootProject().getTasks().findByName("ideaProject"));
     }
 
+    private void linkCompositeBuildDependencies(final ProjectInternal project) {
+        if (isRoot(project)) {
+            getLifecycleTask().dependsOn(new Callable<List<TaskDependency>>() {
+                @Override
+                public List<TaskDependency> call() throws Exception {
+                    return allImlArtifactsInComposite(project);
+                }
+            });
+        }
+    }
+
+    private List<TaskDependency> allImlArtifactsInComposite(ProjectInternal project) {
+        List<TaskDependency> dependencies = Lists.newArrayList();
+        ServiceRegistry services = project.getServices();
+        ProjectPathRegistry projectPathRegistry = services.get(ProjectPathRegistry.class);
+        LocalComponentRegistry localComponentRegistry = services.get(LocalComponentRegistry.class);
+        ProjectComponentIdentifier thisProjectId = projectPathRegistry.getProjectComponentIdentifier(project.getIdentityPath());
+        for (Path projectPath : projectPathRegistry.getAllProjectPaths()) {
+            final ProjectComponentIdentifier otherProjectId = projectPathRegistry.getProjectComponentIdentifier(projectPath);
+            if (thisProjectId.getBuild().equals(otherProjectId.getBuild())) {
+                // IDEA Module for project in current build: handled via `modules` model elements.
+                continue;
+            }
+            LocalComponentArtifactMetadata imlArtifact = localComponentRegistry.findAdditionalArtifact(otherProjectId, "iml");
+            if (imlArtifact != null) {
+                dependencies.add(imlArtifact.getBuildDependencies());
+            }
+        }
+        return dependencies;
+    }
+
     private static boolean isRoot(Project project) {
         return project.getParent() == null;
     }
 
+    private static class ImlArtifact extends DefaultPublishArtifact {
+        private final IdeaModule module;
+        private final File projectDir;
+
+        public ImlArtifact(IdeaModule module, Object... tasks) {
+            super(null, "iml", "iml", null, null, null, tasks);
+            this.module = module;
+            this.projectDir = module.getProject().getProjectDir();
+        }
+
+        @Override
+        public String getName() {
+            return module.getName();
+        }
+
+        @Override
+        public File getFile() {
+            return new File(projectDir, getName() + ".iml");
+        }
+    }
+
 }
