@@ -21,6 +21,7 @@ import com.google.common.collect.Iterables;
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
 import groovy.lang.Closure;
+import org.gradle.api.Action;
 import org.gradle.api.Incubating;
 import org.gradle.api.JavaVersion;
 import org.gradle.api.Project;
@@ -29,20 +30,21 @@ import org.gradle.api.internal.project.ProjectInternal;
 import org.gradle.language.scala.ScalaPlatform;
 import org.gradle.plugins.ide.idea.model.internal.IdeaDependenciesProvider;
 import org.gradle.plugins.ide.internal.resolver.UnresolvedDependenciesLogger;
-import org.gradle.util.ConfigureUtil;
 
 import java.io.File;
 import java.util.Collection;
 import java.util.Map;
 import java.util.Set;
 
+import static org.gradle.util.ConfigureUtil.configure;
+
 /**
  * Enables fine-tuning module details (*.iml file) of the IDEA plugin.
  * <p>
  * Example of use with a blend of most possible properties.
  * Typically you don't have to configure this model directly because Gradle configures it for you.
  *
- * <pre autoTested=''>
+ * <pre class='autoTested'>
  * apply plugin: 'java'
  * apply plugin: 'idea'
  *
@@ -108,7 +110,7 @@ import java.util.Set;
  * <p>
  * Examples of advanced configuration:
  *
- * <pre autoTested=''>
+ * <pre class='autoTested'>
  * apply plugin: 'java'
  * apply plugin: 'idea'
  *
@@ -127,14 +129,14 @@ import java.util.Set;
  *
  *       //closure executed after *.iml content is loaded from existing file
  *       //but before gradle build information is merged
- *       beforeMerged { module ->
+ *       beforeMerged { module -&gt;
  *         //if you want skip merging exclude dirs
  *         module.excludeFolders.clear()
  *       }
  *
  *       //closure executed after *.iml content is loaded from existing file
  *       //and after gradle build information is merged
- *       whenMerged { module ->
+ *       whenMerged { module -&gt;
  *         //you can tinker with {@link Module}
  *       }
  *     }
@@ -240,7 +242,7 @@ public class IdeaModule {
      * plus configurations are added minus the files from the minus configurations. See example below...
      * <p>
      * Example how to use scopes property to enable 'performanceTestCompile' dependencies in the output *.iml file:
-     * <pre autoTested=''>
+     * <pre class='autoTested'>
      * apply plugin: 'java'
      * apply plugin: 'idea'
      *
@@ -313,7 +315,7 @@ public class IdeaModule {
     }
 
     /**
-     * {@link org.gradle.api.dsl.ConventionProperty} for the directories to be excluded. <p> For example see docs for {@link IdeaModule}
+     * Directories to be excluded. <p> For example see docs for {@link IdeaModule}
      */
     public Set<File> getExcludeDirs() {
         return excludeDirs;
@@ -441,7 +443,7 @@ public class IdeaModule {
     }
 
     /**
-     * See {@link #iml(Closure)}
+     * See {@link #iml(Action)}
      */
     public IdeaModuleIml getIml() {
         return iml;
@@ -490,7 +492,18 @@ public class IdeaModule {
      * For example see docs for {@link IdeaModule}.
      */
     public void iml(Closure closure) {
-        ConfigureUtil.configure(closure, getIml());
+        configure(closure, getIml());
+    }
+
+    /**
+     * Enables advanced configuration like tinkering with the output XML or affecting the way existing *.iml content is merged with gradle build information.
+     * <p>
+     * For example see docs for {@link IdeaModule}.
+     *
+     * @since 3.5
+     */
+    public void iml(Action<? super IdeaModuleIml> action) {
+        action.execute(getIml());
     }
 
     /**
