@@ -15,40 +15,52 @@
  */
 package org.gradle.integtests.fixtures.executer;
 
+import com.google.common.base.Preconditions;
+import com.google.common.base.Strings;
 import com.google.common.collect.ImmutableList;
 import com.google.common.collect.Sets;
 import com.google.common.io.CharSource;
 import groovy.lang.Closure;
 import groovy.lang.DelegatesTo;
 import org.gradle.api.Action;
+import org.gradle.api.JavaVersion;
 import org.gradle.api.Transformer;
 import org.gradle.api.UncheckedIOException;
 import org.gradle.api.internal.ClosureBackedAction;
 import org.gradle.api.internal.initialization.DefaultClassLoaderScope;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
+import org.gradle.api.logging.configuration.ConsoleOutput;
 import org.gradle.integtests.fixtures.daemon.DaemonLogsAnalyzer;
+import org.gradle.internal.ImmutableActionSet;
+import org.gradle.internal.MutableActionSet;
 import org.gradle.internal.UncheckedException;
 import org.gradle.internal.featurelifecycle.LoggingDeprecatedFeatureHandler;
 import org.gradle.internal.jvm.Jvm;
+import org.gradle.internal.jvm.UnsupportedJavaRuntimeException;
+import org.gradle.internal.logging.LoggingManagerInternal;
+import org.gradle.internal.logging.services.DefaultLoggingManagerFactory;
 import org.gradle.internal.logging.services.LoggingServiceRegistry;
+import org.gradle.internal.logging.sink.ConsoleStateUtil;
+import org.gradle.internal.logging.sink.OutputEventRenderer;
 import org.gradle.internal.nativeintegration.services.NativeServices;
 import org.gradle.internal.service.ServiceRegistry;
 import org.gradle.internal.service.ServiceRegistryBuilder;
 import org.gradle.internal.service.scopes.GlobalScopeServices;
-import org.gradle.launcher.daemon.configuration.DaemonParameters;
-import org.gradle.launcher.daemon.configuration.GradleProperties;
-import org.gradle.listener.ActionBroadcast;
+import org.gradle.internal.time.Clock;
+import org.gradle.launcher.daemon.configuration.DaemonBuildOptions;
 import org.gradle.process.internal.streams.SafeStreams;
 import org.gradle.test.fixtures.file.TestDirectoryProvider;
 import org.gradle.test.fixtures.file.TestFile;
 import org.gradle.testfixtures.internal.NativeServicesTestFixture;
 import org.gradle.util.CollectionUtils;
+import org.gradle.util.GUtil;
 import org.gradle.util.GradleVersion;
 
 import java.io.File;
 import java.io.IOException;
 import java.io.InputStream;
+import java.io.OutputStream;
 import java.io.PipedInputStream;
 import java.io.PipedOutputStream;
 import java.nio.charset.Charset;
@@ -57,10 +69,10 @@ import java.util.Arrays;
 import java.util.HashMap;
 import java.util.HashSet;
 import java.util.LinkedHashMap;
-import java.util.LinkedHashSet;
 import java.util.List;
 import java.util.Locale;
 import java.util.Map;
+import java.util.Properties;
 import java.util.Set;
 
 import static org.gradle.integtests.fixtures.executer.AbstractGradleExecuter.CliDaemonArgument.*;
@@ -72,11 +84,12 @@ import static org.gradle.util.CollectionUtils.join;
 public abstract class AbstractGradleExecuter implements GradleExecuter {
     protected static final ServiceRegistry GLOBAL_SERVICES = ServiceRegistryBuilder.builder()
         .displayName("Global services")
-        .parent(LoggingServiceRegistry.newCommandLineProcessLogging())
+        .parent(newCommandLineProcessLogging())
         .parent(NativeServicesTestFixture.getInstance())
         .provider(new GlobalScopeServices(true))
         .build();
     protected final static Set<String> PROPAGATED_SYSTEM_PROPERTIES = Sets.newHashSet();
+    private static final List<String> JDK7_PATHS = Arrays.asList("1.7", "jdk7", "-7-", "jdk-7", "7u");
 
     public static void propagateSystemProperty(String name) {
         PROPAGATED_SYSTEM_PROPERTIES.add(name);
@@ -86,6 +99,20 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private static final String LAUNCHER_DEBUG_SYSPROP = "org.gradle.integtest.launcher.debug";
     private static final String PROFILE_SYSPROP = "org.gradle.integtest.profile";
 
+    private static final List<String> LOW_LEVELS = Arrays.asList(
+        "--info",
+        "--debug",
+        "--warn",
+        "-Dorg.gradle.logging.level=lifecycle",
+        "-Dorg.gradle.logging.level=info",
+        "-Dorg.gradle.logging.level=debug",
+        "-Dorg.gradle.logging.level=warn");
+
+    private static final List<String> HIGH_LEVELS = Arrays.asList(
+        "-q",
+        "--quiet",
+        "-Dorg.gradle.logging.level=quiet");
+
     protected static final List<String> DEBUG_ARGS = ImmutableList.of(
         "-agentlib:jdwp=transport=dt_socket,server=y,suspend=y,address=5005"
     );
@@ -95,6 +122,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     protected final IntegrationTestBuildContext buildContext;
 
     private final Set<File> isolatedDaemonBaseDirs = new HashSet<File>();
+    private final Set<GradleHandle> running = new HashSet<GradleHandle>();
     private final List<String> args = new ArrayList<String>();
     private final List<String> tasks = new ArrayList<String>();
     private boolean allowExtraLogging = true;
@@ -122,13 +150,16 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     private final List<String> commandLineJvmOpts = new ArrayList<String>();
     private boolean useOnlyRequestedJvmOpts;
     private boolean requiresGradleDistribution;
+    private boolean useOwnUserHomeServices;
+    private ConsoleOutput consoleType;
+    private boolean showStacktrace = true;
 
     private int expectedDeprecationWarnings;
     private boolean eagerClassLoaderCreationChecksOn = true;
     private boolean stackTraceChecksOn = true;
 
-    private final ActionBroadcast<GradleExecuter> beforeExecute = new ActionBroadcast<GradleExecuter>();
-    private final Set<Action<? super GradleExecuter>> afterExecute = new LinkedHashSet<Action<? super GradleExecuter>>();
+    private final MutableActionSet<GradleExecuter> beforeExecute = new MutableActionSet<GradleExecuter>();
+    private ImmutableActionSet<GradleExecuter> afterExecute = ImmutableActionSet.empty();
 
     private final TestDirectoryProvider testDirectoryProvider;
     protected final GradleVersion gradleVersion;
@@ -170,6 +201,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return logger;
     }
 
+    @Override
     public GradleExecuter reset() {
         args.clear();
         tasks.clear();
@@ -199,33 +231,41 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         interactive = false;
         checkDeprecations = true;
         durationMeasurement = null;
+        consoleType = null;
         return this;
     }
 
+    @Override
     public GradleDistribution getDistribution() {
         return distribution;
     }
 
+    @Override
     public TestDirectoryProvider getTestDirectoryProvider() {
         return testDirectoryProvider;
     }
 
+    @Override
     public void beforeExecute(Action<? super GradleExecuter> action) {
         beforeExecute.add(action);
     }
 
+    @Override
     public void beforeExecute(@DelegatesTo(GradleExecuter.class) Closure action) {
         beforeExecute.add(new ClosureBackedAction<GradleExecuter>(action));
     }
 
+    @Override
     public void afterExecute(Action<? super GradleExecuter> action) {
-        afterExecute.add(action);
+        afterExecute = afterExecute.add(action);
     }
 
+    @Override
     public void afterExecute(@DelegatesTo(GradleExecuter.class) Closure action) {
-        afterExecute.add(new ClosureBackedAction<GradleExecuter>(action));
+        afterExecute(new ClosureBackedAction<GradleExecuter>(action));
     }
 
+    @Override
     public GradleExecuter inDirectory(File directory) {
         workingDir = directory;
         return this;
@@ -235,6 +275,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return workingDir == null ? getTestDirectoryProvider().getTestDirectory() : workingDir;
     }
 
+    @Override
     public GradleExecuter copyTo(GradleExecuter executer) {
         executer.withGradleUserHomeDir(gradleUserHomeDir);
         executer.withDaemonIdleTimeoutSecs(daemonIdleTimeoutSecs);
@@ -298,12 +339,12 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         executer.withCommandLineGradleOpts(commandLineJvmOpts);
         executer.withBuildJvmOpts(buildJvmOpts);
         if (useOnlyRequestedJvmOpts) {
-            executer.useDefaultBuildJvmArgs();
+            executer.useOnlyRequestedJvmOpts();
         }
         executer.noExtraLogging();
 
-        for (int i = 0; i < expectedDeprecationWarnings; i++) {
-            executer.expectDeprecationWarning();
+        if (expectedDeprecationWarnings > 0) {
+            executer.expectDeprecationWarnings(expectedDeprecationWarnings);
         }
         if (!eagerClassLoaderCreationChecksOn) {
             executer.withEagerClassLoaderCreationCheckDisabled();
@@ -314,9 +355,15 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         if (requiresGradleDistribution) {
             executer.requireGradleDistribution();
         }
+        if (useOwnUserHomeServices) {
+            executer.withOwnUserHomeServices();
+        }
         if (requireDaemon) {
             executer.requireDaemon();
         }
+        if (searchUpwards) {
+            executer.withSearchUpwards();
+        }
 
         executer.startBuildProcessInDebugger(debug);
         executer.startLauncherInDebugger(debugLauncher);
@@ -331,38 +378,52 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             executer.withDurationMeasurement(durationMeasurement);
         }
 
+        if (consoleType != null) {
+            executer.withConsole(consoleType);
+        }
+
+        if (!showStacktrace) {
+            executer.withStacktraceDisabled();
+        }
         return executer;
     }
 
+    @Override
     public GradleExecuter usingBuildScript(File buildScript) {
         this.buildScript = buildScript;
         return this;
     }
 
+    @Override
     public GradleExecuter usingProjectDirectory(File projectDir) {
         this.projectDir = projectDir;
         return this;
     }
 
+    @Override
     public GradleExecuter usingSettingsFile(File settingsFile) {
         this.settingsFile = settingsFile;
         return this;
     }
 
+    @Override
     public GradleExecuter usingInitScript(File initScript) {
         initScripts.add(initScript);
         return this;
     }
 
+    @Override
     public TestFile getGradleUserHomeDir() {
         return gradleUserHomeDir;
     }
 
+    @Override
     public GradleExecuter withGradleUserHomeDir(File userHomeDir) {
         this.gradleUserHomeDir = userHomeDir == null ? null : new TestFile(userHomeDir);
         return this;
     }
 
+    @Override
     public GradleExecuter requireOwnGradleUserHomeDir() {
         return withGradleUserHomeDir(testDirectoryProvider.getTestDirectory().file("user-home"));
     }
@@ -455,6 +516,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return buildJvmOpts;
     }
 
+    @Override
     public GradleExecuter withUserHomeDir(File userHomeDir) {
         this.userHomeDir = userHomeDir;
         return this;
@@ -464,11 +526,13 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return javaHome == null ? Jvm.current().getJavaHome() : javaHome;
     }
 
+    @Override
     public GradleExecuter withJavaHome(File javaHome) {
         this.javaHome = javaHome;
         return this;
     }
 
+    @Override
     public GradleExecuter usingExecutable(String script) {
         this.executable = script;
         return this;
@@ -501,6 +565,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return stdinPipe;
     }
 
+    @Override
     public GradleExecuter withDefaultCharacterEncoding(String defaultCharacterEncoding) {
         this.defaultCharacterEncoding = defaultCharacterEncoding;
         return this;
@@ -510,6 +575,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return defaultCharacterEncoding == null ? Charset.defaultCharset().name() : defaultCharacterEncoding;
     }
 
+    @Override
     public GradleExecuter withDefaultLocale(Locale defaultLocale) {
         this.defaultLocale = defaultLocale;
         return this;
@@ -519,6 +585,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return defaultLocale;
     }
 
+    @Override
     public GradleExecuter withSearchUpwards() {
         searchUpwards = true;
         return this;
@@ -528,36 +595,43 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return quiet;
     }
 
+    @Override
     public GradleExecuter withQuietLogging() {
         quiet = true;
         return this;
     }
 
+    @Override
     public GradleExecuter withTaskList() {
         taskList = true;
         return this;
     }
 
+    @Override
     public GradleExecuter withDependencyList() {
         dependencyList = true;
         return this;
     }
 
+    @Override
     public GradleExecuter withArguments(String... args) {
         return withArguments(Arrays.asList(args));
     }
 
+    @Override
     public GradleExecuter withArguments(List<String> args) {
         this.args.clear();
         this.args.addAll(args);
         return this;
     }
 
+    @Override
     public GradleExecuter withArgument(String arg) {
         this.args.add(arg);
         return this;
     }
 
+    @Override
     public GradleExecuter withEnvironmentVars(Map<String, ?> environment) {
         environmentVars.clear();
         for (Map.Entry<String, ?> entry : environment.entrySet()) {
@@ -585,46 +659,59 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return result.toString();
     }
 
+    @Override
     public GradleExecuter withTasks(String... names) {
         return withTasks(Arrays.asList(names));
     }
 
+    @Override
     public GradleExecuter withTasks(List<String> names) {
         tasks.clear();
         tasks.addAll(names);
         return this;
     }
 
+    @Override
     public GradleExecuter withDaemonIdleTimeoutSecs(int secs) {
         daemonIdleTimeoutSecs = secs;
         return this;
     }
 
-    public GradleExecuter useDefaultBuildJvmArgs() {
+    @Override
+    public GradleExecuter useOnlyRequestedJvmOpts() {
         useOnlyRequestedJvmOpts = true;
         return this;
     }
 
+    @Override
     public GradleExecuter withDaemonBaseDir(File daemonBaseDir) {
         this.daemonBaseDir = daemonBaseDir;
         return this;
     }
 
+    @Override
     public GradleExecuter requireIsolatedDaemons() {
         return withDaemonBaseDir(testDirectoryProvider.getTestDirectory().file("daemon"));
     }
 
+    @Override
+    public GradleExecuter withWorkerDaemonsExpirationDisabled() {
+        return withArgument("-Dorg.gradle.workers.internal.disable-daemons-expiration=true");
+    }
+
+    @Override
     public boolean usesSharedDaemons() {
         return isSharedDaemons();
     }
 
+    @Override
     public File getDaemonBaseDir() {
         return daemonBaseDir;
     }
 
     @Override
     public GradleExecuter requireDaemon() {
-        requireDaemon = true;
+        this.requireDaemon = true;
         return this;
     }
 
@@ -632,22 +719,51 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return daemonBaseDir.equals(buildContext.getDaemonBaseDir());
     }
 
+    @Override
     public boolean isUseDaemon() {
         CliDaemonArgument cliDaemonArgument = resolveCliDaemonArgument();
-        if (cliDaemonArgument == DAEMON) {
-            return true;
+        if (cliDaemonArgument == NO_DAEMON) {
+            return false;
         }
-        return requireDaemon;
+        return requireDaemon || cliDaemonArgument == DAEMON;
+    }
+
+    @Override
+    public GradleExecuter withOwnUserHomeServices() {
+        useOwnUserHomeServices = true;
+        return this;
+    }
+
+    @Override
+    public GradleExecuter withConsole(ConsoleOutput consoleType) {
+        this.consoleType = consoleType;
+        return this;
+    }
+
+    public GradleExecuter withStacktraceDisabled() {
+        showStacktrace = false;
+        return this;
     }
 
     /**
      * Performs cleanup at completion of the test.
      */
     public void cleanup() {
+        stopRunningBuilds();
         cleanupIsolatedDaemons();
     }
 
-    protected void cleanupIsolatedDaemons() {
+    private void stopRunningBuilds() {
+        for (GradleHandle handle : running) {
+            try {
+                handle.abort().waitForExit();
+            } catch (Exception e) {
+                getLogger().warn("Problem stopping running build", e);
+            }
+        }
+    }
+
+    private void cleanupIsolatedDaemons() {
         for (File baseDir : isolatedDaemonBaseDirs) {
             try {
                 new DaemonLogsAnalyzer(baseDir, gradleVersion.getVersion()).killAll();
@@ -712,7 +828,9 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                 allArgs.add("--no-daemon");
             }
         }
-        allArgs.add("--stacktrace");
+        if (showStacktrace) {
+            allArgs.add("--stacktrace");
+        }
         if (taskList) {
             allArgs.add("tasks");
         }
@@ -746,6 +864,10 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             allArgs.add(getGradleUserHomeDir().getAbsolutePath());
         }
 
+        if (consoleType != null) {
+            allArgs.add("--console=" + consoleType.toString().toLowerCase());
+        }
+
         allArgs.addAll(args);
         allArgs.addAll(tasks);
         return allArgs;
@@ -761,14 +883,14 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
             properties.put("user.home", getUserHomeDir().getAbsolutePath());
         }
 
-        properties.put(GradleProperties.IDLE_TIMEOUT_PROPERTY, "" + (daemonIdleTimeoutSecs * 1000));
-        properties.put(GradleProperties.DAEMON_BASE_DIR_PROPERTY, daemonBaseDir.getAbsolutePath());
+        properties.put(DaemonBuildOptions.IdleTimeoutOption.GRADLE_PROPERTY, "" + (daemonIdleTimeoutSecs * 1000));
+        properties.put(DaemonBuildOptions.BaseDirOption.GRADLE_PROPERTY, daemonBaseDir.getAbsolutePath());
         if (!noExplicitNativeServicesDir) {
             properties.put(NativeServices.NATIVE_DIR_OVERRIDE, buildContext.getNativeServicesDir().getAbsolutePath());
         }
         properties.put(LoggingDeprecatedFeatureHandler.ORG_GRADLE_DEPRECATION_TRACE_PROPERTY_NAME, Boolean.toString(fullDeprecationStackTrace));
 
-        if (gradleUserHomeDir != null && !gradleUserHomeDir.equals(buildContext.getGradleUserHomeDir())) {
+        if (useOwnUserHomeServices || (gradleUserHomeDir != null && !gradleUserHomeDir.equals(buildContext.getGradleUserHomeDir()))) {
             properties.put(REUSE_USER_HOME_SERVICES, "false");
         }
         if (!noExplicitTmpDir) {
@@ -794,7 +916,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
 
         if (interactive) {
-            properties.put(DaemonParameters.INTERACTIVE_TOGGLE, "true");
+            properties.put(ConsoleStateUtil.INTERACTIVE_TOGGLE, "true");
         }
 
         return properties;
@@ -804,18 +926,26 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return true;
     }
 
+    @Override
     public final GradleHandle start() {
         assert afterExecute.isEmpty() : "afterExecute actions are not implemented for async execution";
+        return startHandle();
+    }
+
+    protected GradleHandle startHandle() {
         fireBeforeExecute();
         assertCanExecute();
         collectStateBeforeExecution();
         try {
-            return doStart();
+            GradleHandle handle = createGradleHandle();
+            running.add(handle);
+            return handle;
         } finally {
             reset();
         }
     }
 
+    @Override
     public final ExecutionResult run() {
         fireBeforeExecute();
         assertCanExecute();
@@ -827,14 +957,65 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         }
     }
 
-    private void finished() {
+    private boolean java7DeprecationWarningShouldExist() {
+        if (org.apache.commons.collections.CollectionUtils.containsAny(args, LOW_LEVELS)) {
+            return currentOrTargetIsJava7();
+        }
+        if (org.apache.commons.collections.CollectionUtils.containsAny(args, HIGH_LEVELS)) {
+            return false;
+        }
+        return currentOrTargetIsJava7();
+    }
+
+    private boolean currentOrTargetIsJava7() {
+        String javaHomeInProperties = javaHomeInProperties();
+        if (javaHomeInProperties != null) {
+            return isJava7Home(javaHomeInProperties);
+        } else if (getJavaHome().equals(Jvm.current().getJavaHome())) {
+            return JavaVersion.current().isJava7();
+        } else {
+            return isJava7Home(getJavaHome().toString());
+        }
+    }
+
+    private String javaHomeInProperties() {
+        File gradleProperties = new File(getWorkingDir(), "gradle.properties");
+        if (gradleProperties.isFile()) {
+            Properties properties = GUtil.loadProperties(gradleProperties);
+            if (properties.getProperty("org.gradle.java.home") != null) {
+                return properties.getProperty("org.gradle.java.home");
+            }
+        }
+        return null;
+    }
+
+    private boolean isJava7Home(String path){
+        for(String jdk7Path: JDK7_PATHS){
+            if(path.contains(jdk7Path)){
+                return true;
+            }
+        }
+        return false;
+    }
+
+    private boolean isJava7DeprecationTruncatedDaemonLog(List<String> lines, int i, String line) {
+        // for forkingIntegrationTests running on Java 7, daemon exit log will fail the tests with following stdout:
+        // ----- Last  20 lines from daemon log file - daemon-6753.out.log -----
+        //    at org.gradle.launcher.daemon.server.exec.ForwardClientInput.execute(ForwardClientInput.java:72)
+        //    at org.gradle.launcher.daemon.server.api.DaemonCommandExecution.proceed(DaemonCommandExecution.java:120)
+        //    ...
+        return line.startsWith("----- Last  ") && i < lines.size() - 1 && STACK_TRACE_ELEMENT.matcher(lines.get(i + 1)).matches();
+    }
+
+    protected void finished() {
         try {
-            new ActionBroadcast<GradleExecuter>(afterExecute).execute(this);
+            afterExecute.execute(this);
         } finally {
             reset();
         }
     }
 
+    @Override
     public final ExecutionFailure runWithFailure() {
         fireBeforeExecute();
         assertCanExecute();
@@ -856,7 +1037,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         beforeExecute.execute(this);
     }
 
-    protected GradleHandle doStart() {
+    protected GradleHandle createGradleHandle() {
         throw new UnsupportedOperationException(String.format("%s does not support running asynchronously.", getClass().getSimpleName()));
     }
 
@@ -890,12 +1071,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
     @Override
     public GradleExecuter withBuildCacheEnabled() {
-        return withArgument("-Dorg.gradle.cache.tasks=true");
-    }
-
-    @Override
-    public GradleExecuter withLocalBuildCache(File cacheDir) {
-        return withBuildCacheEnabled().withArgument("-Dorg.gradle.cache.tasks.directory=" + cacheDir.getAbsolutePath());
+        return withArgument("--build-cache");
     }
 
     protected Action<ExecutionResult> getResultAssertion() {
@@ -906,16 +1082,45 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
 
             @Override
             public void execute(ExecutionResult executionResult) {
-                validate(executionResult.getNormalizedOutput(), "Standard output");
+                String normalizedOutput = executionResult.getNormalizedOutput();
                 String error = executionResult.getError();
-                if (executionResult instanceof ExecutionFailure) {
-                    // Axe everything after the expected exception
-                    int pos = error.indexOf("* Exception is:\n");
-                    if (pos >= 0) {
-                        error = error.substring(0, pos);
-                    }
+                boolean executionFailure = isExecutionFailure(executionResult);
+
+                // for tests using rich console standard out and error are combined in output of execution result
+                if (executionFailure && isErrorOutEmpty(error)) {
+                    normalizedOutput = removeExceptionStackTraceForFailedExecution(normalizedOutput);
                 }
+
+                validate(normalizedOutput, "Standard output");
+
+                if (executionFailure) {
+                    error = removeExceptionStackTraceForFailedExecution(error);
+                }
+
                 validate(error, "Standard error");
+
+                if (expectedDeprecationWarnings > 0) {
+                    throw new AssertionError(String.format("Expected %d more deprecation warnings", expectedDeprecationWarnings));
+                }
+            }
+
+            private boolean isErrorOutEmpty(String error) {
+                //remove SLF4J error out like 'Class path contains multiple SLF4J bindings.'
+                //See: https://github.com/gradle/performance/issues/375#issuecomment-315103861
+                return Strings.isNullOrEmpty(error.replaceAll("(?m)^SLF4J: .*", "").trim());
+            }
+
+            private boolean isExecutionFailure(ExecutionResult executionResult) {
+                return executionResult instanceof ExecutionFailure;
+            }
+
+            // Axe everything after the expected exception
+            private String removeExceptionStackTraceForFailedExecution(String text) {
+                int pos = text.indexOf("* Exception is:");
+                if (pos >= 0) {
+                    text = text.substring(0, pos);
+                }
+                return text;
             }
 
             private void validate(String output, String displayName) {
@@ -931,9 +1136,26 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                     if (line.matches(".*use(s)? or override(s)? a deprecated API\\.")) {
                         // A javac warning, ignore
                         i++;
+                    } else if (line.contains(UnsupportedJavaRuntimeException.JAVA7_DEPRECATION_WARNING)) {
+                        if (!java7DeprecationWarningShouldExist()) {
+                            throw new AssertionError(String.format("%s line %d contains unexpected deprecation warning: %s%n=====%n%s%n=====%n", displayName, i + 1, line, output));
+                        }
+                        // skip over stack trace
+                        i++;
+                        while (i < lines.size() && STACK_TRACE_ELEMENT.matcher(lines.get(i)).matches()) {
+                            i++;
+                        }
+                    } else if (isJava7DeprecationTruncatedDaemonLog(lines, i, line)) {
+                        // skip over stack trace
+                        i++;
+                        while (i < lines.size() && STACK_TRACE_ELEMENT.matcher(lines.get(i)).matches()) {
+                            i++;
+                        }
+                    } else if (isDeprecationMessageInHelpDescription(line)) {
+                        i++;
                     } else if (line.matches(".*\\s+deprecated.*")) {
                         if (checkDeprecations && expectedDeprecationWarnings <= 0) {
-                            throw new AssertionError(String.format("%s line %d contains a deprecation warning: %s%n=====%n%s%n=====%n", displayName, i+1, line, output));
+                            throw new AssertionError(String.format("%s line %d contains a deprecation warning: %s%n=====%n%s%n=====%n", displayName, i + 1, line, output));
                         }
                         expectedDeprecationWarnings--;
                         // skip over stack trace
@@ -941,31 +1163,47 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
                         while (i < lines.size() && STACK_TRACE_ELEMENT.matcher(lines.get(i)).matches()) {
                             i++;
                         }
-                    } else if (!expectStackTraces && STACK_TRACE_ELEMENT.matcher(line).matches() && i < lines.size()-1 && STACK_TRACE_ELEMENT.matcher(lines.get(i+1)).matches()) {
+                    } else if (!expectStackTraces && STACK_TRACE_ELEMENT.matcher(line).matches() && i < lines.size() - 1 && STACK_TRACE_ELEMENT.matcher(lines.get(i + 1)).matches()) {
                         // 2 or more lines that look like stack trace elements
-                        throw new AssertionError(String.format("%s line %d contains an unexpected stack trace: %s%n=====%n%s%n=====%n", displayName, i+1, line, output));
+                        throw new AssertionError(String.format("%s line %d contains an unexpected stack trace: %s%n=====%n%s%n=====%n", displayName, i + 1, line, output));
                     } else {
                         i++;
                     }
                 }
             }
+
+            private boolean isDeprecationMessageInHelpDescription(String s) {
+                return s.matches(".*\\[deprecated.*]");
+            }
         };
     }
 
+    @Override
     public GradleExecuter expectDeprecationWarning() {
-        expectedDeprecationWarnings++;
+        return expectDeprecationWarnings(1);
+    }
+
+    @Override
+    public GradleExecuter expectDeprecationWarnings(int count) {
+        Preconditions.checkState(expectedDeprecationWarnings == 0, "expected deprecation count is already set for this execution");
+        Preconditions.checkArgument(count > 0, "expected deprecation count must be positive");
+        expectedDeprecationWarnings = count;
         return this;
     }
+
+    @Override
     public GradleExecuter noDeprecationChecks() {
         checkDeprecations = false;
         return this;
     }
 
+    @Override
     public GradleExecuter withEagerClassLoaderCreationCheckDisabled() {
         eagerClassLoaderCreationChecksOn = false;
         return this;
     }
 
+    @Override
     public GradleExecuter withStackTraceChecksDisabled() {
         stackTraceChecksOn = false;
         return this;
@@ -975,6 +1213,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return buildContext.getTmpDir().createDir();
     }
 
+    @Override
     public GradleExecuter noExtraLogging() {
         this.allowExtraLogging = false;
         return this;
@@ -988,6 +1227,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return requiresGradleDistribution;
     }
 
+    @Override
     public GradleExecuter requireGradleDistribution() {
         this.requiresGradleDistribution = true;
         return this;
@@ -1010,6 +1250,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return debugLauncher;
     }
 
+    @Override
     public GradleExecuter withProfiler(String args) {
         profiler = args;
         return this;
@@ -1033,6 +1274,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return this;
     }
 
+    @Override
     public GradleExecuter withFullDeprecationStackTraceDisabled() {
         fullDeprecationStackTrace = false;
         return this;
@@ -1043,6 +1285,7 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
         return debug;
     }
 
+    @Override
     public boolean isProfile() {
         return !profiler.isEmpty();
     }
@@ -1084,4 +1327,51 @@ public abstract class AbstractGradleExecuter implements GradleExecuter {
     protected DurationMeasurement getDurationMeasurement() {
         return durationMeasurement;
     }
+
+    private static LoggingServiceRegistry newCommandLineProcessLogging() {
+        LoggingServiceRegistry loggingServices = new LoggingServiceRegistry() {
+            @Override
+            protected OutputEventRenderer createOutputEventRenderer(Clock clock) {
+                return new VerboseAwareOutputEventRenderer(clock);
+            }
+        };
+        LoggingManagerInternal rootLoggingManager = loggingServices.get(DefaultLoggingManagerFactory.class).getRoot();
+        rootLoggingManager.captureSystemSources();
+        rootLoggingManager.attachSystemOutAndErr();
+        return loggingServices;
+    }
+
+    private static class VerboseAwareOutputEventRenderer extends OutputEventRenderer {
+        VerboseAwareOutputEventRenderer(Clock clock) {
+            super(clock);
+        }
+
+        @Override
+        public void attachAnsiConsole(OutputStream outputStream) {
+            if (outputStream instanceof VerboseAwareAnsiOutputStream) {
+                attachAnsiConsole(outputStream, VerboseAwareAnsiOutputStream.class.cast(outputStream).isVerbose());
+            } else {
+                super.attachAnsiConsole(outputStream);
+            }
+        }
+    }
+
+    protected static class VerboseAwareAnsiOutputStream extends OutputStream {
+        private final OutputStream delegate;
+        private boolean verbose;
+
+        VerboseAwareAnsiOutputStream(OutputStream delegate, boolean verbose) {
+            this.delegate = delegate;
+            this.verbose = verbose;
+        }
+
+        public boolean isVerbose() {
+            return verbose;
+        }
+
+        @Override
+        public void write(int b) throws IOException {
+            delegate.write(b);
+        }
+    }
 }
