@@ -23,11 +23,11 @@ import org.gradle.api.Task;
 import org.gradle.api.execution.TaskExecutionGraph;
 import org.gradle.api.execution.TaskExecutionGraphListener;
 import org.gradle.api.execution.TaskExecutionListener;
-import org.gradle.api.internal.changedetection.state.InMemoryTaskArtifactCache;
+import org.gradle.api.internal.StartParameterInternal;
 import org.gradle.api.internal.classpath.ModuleRegistry;
 import org.gradle.api.internal.file.TestFiles;
 import org.gradle.api.logging.StandardOutputListener;
-import org.gradle.api.logging.configuration.ShowStacktrace;
+import org.gradle.api.logging.configuration.ConsoleOutput;
 import org.gradle.api.tasks.TaskState;
 import org.gradle.cli.CommandLineParser;
 import org.gradle.configuration.GradleLauncherMetaData;
@@ -38,12 +38,16 @@ import org.gradle.initialization.DefaultBuildRequestContext;
 import org.gradle.initialization.DefaultBuildRequestMetaData;
 import org.gradle.initialization.NoOpBuildEventConsumer;
 import org.gradle.initialization.ReportedException;
+import org.gradle.initialization.layout.BuildLayoutFactory;
+import org.gradle.integtests.fixtures.logging.GroupedOutputFixture;
 import org.gradle.internal.Factory;
 import org.gradle.internal.SystemProperties;
 import org.gradle.internal.classpath.ClassPath;
 import org.gradle.internal.event.ListenerManager;
 import org.gradle.internal.exceptions.LocationAwareException;
 import org.gradle.internal.invocation.BuildAction;
+import org.gradle.internal.io.LineBufferingOutputStream;
+import org.gradle.internal.io.TextStream;
 import org.gradle.internal.jvm.Jvm;
 import org.gradle.internal.logging.LoggingManagerInternal;
 import org.gradle.internal.nativeintegration.ProcessEnvironment;
@@ -64,12 +68,12 @@ import org.gradle.util.SetSystemProperties;
 import org.hamcrest.Matcher;
 import org.hamcrest.Matchers;
 
+import javax.annotation.Nullable;
 import java.io.File;
 import java.io.InputStream;
 import java.io.StringWriter;
 import java.nio.charset.Charset;
 import java.util.ArrayList;
-import java.util.Arrays;
 import java.util.Collection;
 import java.util.HashMap;
 import java.util.HashSet;
@@ -83,6 +87,7 @@ import java.util.concurrent.CopyOnWriteArrayList;
 import java.util.concurrent.CopyOnWriteArraySet;
 import java.util.regex.Pattern;
 
+import static org.gradle.integtests.fixtures.executer.OutputScrapingExecutionResult.flattenTaskPaths;
 import static org.gradle.util.Matchers.*;
 import static org.hamcrest.Matchers.*;
 import static org.junit.Assert.*;
@@ -114,7 +119,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
     @Override
     protected ExecutionResult doRun() {
         if (isForkRequired()) {
-            return doStart().waitForFinish();
+            return createGradleHandle().waitForFinish();
         }
 
         StandardOutputListener outputListener = new OutputListenerImpl();
@@ -127,13 +132,13 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             throw new UnexpectedBuildFailure(e);
         }
         return assertResult(new InProcessExecutionResult(buildListener.executedTasks, buildListener.skippedTasks,
-                new OutputScrapingExecutionResult(outputListener.toString(), errorListener.toString())));
+            new OutputScrapingExecutionResult(outputListener.toString(), errorListener.toString())));
     }
 
     @Override
     protected ExecutionFailure doRunWithFailure() {
         if (isForkRequired()) {
-            return doStart().waitForFailure();
+            return createGradleHandle().waitForFailure();
         }
 
         StandardOutputListener outputListener = new OutputListenerImpl();
@@ -144,7 +149,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             throw new AssertionError("expected build to fail but it did not.");
         } catch (GradleException e) {
             return assertResult(new InProcessExecutionFailure(buildListener.executedTasks, buildListener.skippedTasks,
-                    new OutputScrapingExecutionFailure(outputListener.toString(), errorListener.toString()), e));
+                new OutputScrapingExecutionFailure(outputListener.toString(), errorListener.toString()), e));
         }
     }
 
@@ -169,7 +174,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
     }
 
     @Override
-    protected GradleHandle doStart() {
+    protected GradleHandle createGradleHandle() {
         return new ForkingGradleHandle(getStdinPipe(), isUseDaemon(), getResultAssertion(), getDefaultCharacterEncoding(), getJavaExecBuilder(), getDurationMeasurement()).start();
     }
 
@@ -241,7 +246,32 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         SetSystemProperties.resetTempDirLocation();
     }
 
-    private BuildResult executeBuild(GradleInvocation invocation, StandardOutputListener outputListener, StandardOutputListener errorListener, BuildListenerImpl listener) {
+    private LoggingManagerInternal createLoggingManager(StartParameter startParameter, final StandardOutputListener outputListener, final StandardOutputListener errorListener) {
+        LoggingManagerInternal loggingManager =
+            GLOBAL_SERVICES.getFactory(LoggingManagerInternal.class).create();
+
+        if (startParameter.getConsoleOutput() == ConsoleOutput.Rich || startParameter.getConsoleOutput() == ConsoleOutput.Verbose) {
+            // Explicitly enabling rich console forces everything into the stdout listener
+            loggingManager.attachAnsiConsole(new VerboseAwareAnsiOutputStream(new LineBufferingOutputStream(new TextStream() {
+                @Override
+                public void text(String text) {
+                    outputListener.onOutput(text);
+                }
+
+                @Override
+                public void endOfStream(@Nullable Throwable failure) {
+
+                }
+            }), startParameter.getConsoleOutput() == ConsoleOutput.Verbose));
+        } else {
+            loggingManager.addStandardOutputListener(outputListener);
+            loggingManager.addStandardErrorListener(errorListener);
+        }
+
+        return loggingManager;
+    }
+
+    private BuildResult executeBuild(GradleInvocation invocation, final StandardOutputListener outputListener, final StandardOutputListener errorListener, BuildListenerImpl listener) {
         // Augment the environment for the execution
         System.setIn(connectStdIn());
         processEnvironment.maybeSetProcessDir(getWorkingDir());
@@ -253,19 +283,16 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         resetTempDirLocation();
 
         // TODO: Fix tests that rely on this being set before we process arguments like this...
-        StartParameter startParameter = new StartParameter();
+        StartParameter startParameter = new StartParameterInternal();
         startParameter.setCurrentDir(getWorkingDir());
-        startParameter.setShowStacktrace(ShowStacktrace.ALWAYS);
 
+        // TODO: Reuse more of CommandlineActionFactory
         CommandLineParser parser = new CommandLineParser();
-        ParametersConverter parametersConverter = new ParametersConverter();
+        BuildLayoutFactory buildLayoutFactory = BuildLayoutFactory.forDefaultScriptingLanguages();
+        ParametersConverter parametersConverter = new ParametersConverter(buildLayoutFactory);
         parametersConverter.configure(parser);
         final Parameters parameters = new Parameters(startParameter);
         parametersConverter.convert(parser.parse(getAllArgs()), parameters);
-        if (parameters.getDaemonParameters().isStop()) {
-            // --stop should simulate stopping the daemon
-            GLOBAL_SERVICES.get(InMemoryTaskArtifactCache.class).invalidateAll();
-        }
 
         BuildActionExecuter<BuildActionParameters> actionExecuter = GLOBAL_SERVICES.get(BuildActionExecuter.class);
 
@@ -276,14 +303,25 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             // TODO: Reuse more of BuildActionsFactory
             BuildAction action = new ExecuteBuildAction(startParameter);
             BuildActionParameters buildActionParameters = createBuildActionParameters(startParameter);
-            BuildRequestContext buildRequestContext = createBuildRequestContext(outputListener, errorListener);
-            startMeasurement();
-            actionExecuter.execute(action, buildRequestContext, buildActionParameters, GLOBAL_SERVICES);
+            BuildRequestContext buildRequestContext = createBuildRequestContext();
+
+            LoggingManagerInternal loggingManager = createLoggingManager(startParameter, outputListener, errorListener);
+            loggingManager.start();
+
+            try {
+                startMeasurement();
+                try {
+                    actionExecuter.execute(action, buildRequestContext, buildActionParameters, GLOBAL_SERVICES);
+                } finally {
+                    stopMeasurement();
+                }
+            } finally {
+                loggingManager.stop();
+            }
             return new BuildResult(null, null);
         } catch (ReportedException e) {
             return new BuildResult(null, e.getCause());
         } finally {
-            stopMeasurement();
             listenerManager.removeListener(listener);
         }
     }
@@ -301,12 +339,11 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         );
     }
 
-    private BuildRequestContext createBuildRequestContext(StandardOutputListener outputListener, StandardOutputListener errorListener) {
+    private BuildRequestContext createBuildRequestContext() {
         return new DefaultBuildRequestContext(
             new DefaultBuildRequestMetaData(new GradleLauncherMetaData()),
             new DefaultBuildCancellationToken(),
-            new NoOpBuildEventConsumer(),
-            outputListener, errorListener);
+            new NoOpBuildEventConsumer());
     }
 
     public void assertCanExecute() {
@@ -396,6 +433,7 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
         private final List<String> plannedTasks;
         private final Set<String> skippedTasks;
         private final OutputScrapingExecutionResult outputResult;
+        private GroupedOutputFixture groupedOutputFixture;
 
         public InProcessExecutionResult(List<String> plannedTasks, Set<String> skippedTasks, OutputScrapingExecutionResult outputResult) {
             this.plannedTasks = plannedTasks;
@@ -412,6 +450,14 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return outputResult.getNormalizedOutput();
         }
 
+        @Override
+        public GroupedOutputFixture getGroupedOutput() {
+            if (groupedOutputFixture == null) {
+                this.groupedOutputFixture = new GroupedOutputFixture(getOutput());
+            }
+            return groupedOutputFixture;
+        }
+
         public ExecutionResult assertOutputEquals(String expectedOutput, boolean ignoreExtraLines, boolean ignoreLineOrder) {
             outputResult.assertOutputEquals(expectedOutput, ignoreExtraLines, ignoreLineOrder);
             return this;
@@ -431,10 +477,25 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return new ArrayList<String>(plannedTasks);
         }
 
-        public ExecutionResult assertTasksExecuted(String... taskPaths) {
-            List<String> expected = Arrays.asList(taskPaths);
-            assertThat(plannedTasks, equalTo(expected));
-            outputResult.assertTasksExecuted(taskPaths);
+        public ExecutionResult assertTasksExecutedInOrder(Object... taskPaths) {
+            Set<String> expected = TaskOrderSpecs.exact(taskPaths).getTasks();
+            assertThat(plannedTasks, containsInAnyOrder(expected.toArray()));
+            outputResult.assertTasksExecutedInOrder(taskPaths);
+            return this;
+        }
+
+        public ExecutionResult assertTasksExecuted(Object... taskPaths) {
+            List<String> flattenedTasks = flattenTaskPaths(taskPaths);
+            assertThat(plannedTasks, containsInAnyOrder(flattenedTasks.toArray()));
+            outputResult.assertTasksExecuted(flattenedTasks);
+            return this;
+        }
+
+        @Override
+        public ExecutionResult assertTaskOrder(Object... taskPaths) {
+            Set<String> expected = TaskOrderSpecs.exact(taskPaths).getTasks();
+            assertThat(plannedTasks, hasItems(expected.toArray(new String[]{})));
+            outputResult.assertTaskOrder(taskPaths);
             return this;
         }
 
@@ -442,13 +503,14 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return new HashSet<String>(skippedTasks);
         }
 
-        public ExecutionResult assertTasksSkipped(String... taskPaths) {
+        @Override
+        public ExecutionResult assertTasksSkipped(Object... taskPaths) {
             if (GradleContextualExecuter.isParallel()) {
                 return this;
             }
-            Set<String> expected = new HashSet<String>(Arrays.asList(taskPaths));
+            Set<String> expected = new HashSet<String>(flattenTaskPaths(taskPaths));
             assertThat(skippedTasks, equalTo(expected));
-            outputResult.assertTasksSkipped(taskPaths);
+            outputResult.assertTasksSkipped(expected);
             return this;
         }
 
@@ -461,14 +523,15 @@ public class InProcessGradleExecuter extends AbstractGradleExecuter {
             return this;
         }
 
-        public ExecutionResult assertTasksNotSkipped(String... taskPaths) {
+        @Override
+        public ExecutionResult assertTasksNotSkipped(Object... taskPaths) {
             if (GradleContextualExecuter.isParallel()) {
                 return this;
             }
-            Set<String> expected = new HashSet<String>(Arrays.asList(taskPaths));
+            Set<String> expected = new HashSet<String>(flattenTaskPaths(taskPaths));
             Set<String> notSkipped = getNotSkippedTasks();
             assertThat(notSkipped, equalTo(expected));
-            outputResult.assertTasksNotSkipped(taskPaths);
+            outputResult.assertTasksNotSkipped(expected);
             return this;
         }
 
