@@ -34,12 +34,20 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
     private static final Pattern DESCRIPTION_PATTERN = Pattern.compile("(?ms)^\\* What went wrong:$(.+?)^\\* Try:$");
     private static final Pattern LOCATION_PATTERN = Pattern.compile("(?ms)^\\* Where:((.+)'.+') line: (\\d+)$");
     private static final Pattern RESOLUTION_PATTERN = Pattern.compile("(?ms)^\\* Try:$(.+?)^\\* Exception is:$");
+    private static final Pattern EXCEPTION_PATTERN = Pattern.compile("(?ms)^\\* Exception is:$(.+?):(.+?)$");
+    private static final Pattern EXCEPTION_CAUSE_PATTERN = Pattern.compile("(?ms)^Caused by: (.+?):(.+?)$");
     private final String description;
     private final String lineNumber;
     private final String fileName;
     private final String resolution;
+    private final Exception exception;
+    // with normalized line endings
     private final List<String> causes = new ArrayList<String>();
 
+    static boolean hasFailure(String error) {
+        return FAILURE_PATTERN.matcher(error).find();
+    }
+
     public OutputScrapingExecutionFailure(String output, String error) {
         super(output, error);
 
@@ -80,6 +88,16 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
         } else {
             resolution = matcher.group(1).trim();
         }
+
+        matcher = EXCEPTION_PATTERN.matcher(error);
+        if (!matcher.find()) {
+            exception = null;
+        } else {
+            String exceptionClass = matcher.group(1).trim();
+            String exceptionMessage = matcher.group(2).trim();
+            matcher = EXCEPTION_CAUSE_PATTERN.matcher(error);
+            exception = recreateException(exceptionClass, exceptionMessage, matcher);
+        }
     }
 
     private Problem extract(String problem) {
@@ -107,6 +125,24 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
         return new Problem(description, causes);
     }
 
+    private Exception recreateException(String className, String message, java.util.regex.Matcher exceptionCauseMatcher) {
+        Exception causedBy = null;
+        if (exceptionCauseMatcher.find()) {
+            String causedByClass = exceptionCauseMatcher.group(1).trim();
+            String causedByMessage = exceptionCauseMatcher.group(2).trim();
+            causedBy = recreateException(causedByClass, causedByMessage, exceptionCauseMatcher);
+        }
+        try {
+            if (causedBy == null) {
+                return (Exception) Class.forName(className).getConstructor(String.class).newInstance(message);
+            } else {
+                return (Exception) Class.forName(className).getConstructor(String.class, Throwable.class).newInstance(message, causedBy);
+            }
+        } catch (Exception e) {
+            return new Exception(message);
+        }
+    }
+
     private String toPrefixPattern(int prefix) {
         StringBuilder builder = new StringBuilder("(?m)^");
         for (int i = 0; i < prefix; i++) {
@@ -169,6 +205,10 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
         return new DependencyResolutionFailure(this, configurationPath);
     }
 
+    public Exception getException() {
+        return exception;
+    }
+
     private static class Problem {
         final String description;
         final List<String> causes;
@@ -178,4 +218,4 @@ public class OutputScrapingExecutionFailure extends OutputScrapingExecutionResul
             this.causes = causes;
         }
     }
-}
\ No newline at end of file
+}
