@@ -37,6 +37,8 @@ import java.util.concurrent.TimeUnit;
 
 /**
  * Allows the test process and a single build process to synchronize.
+ *
+ * TODO - merge into {@link BlockingHttpServer} as they now have overlapping APIs.
  */
 public class CyclicBarrierHttpServer extends ExternalResource {
     private ExecutorService executor;
@@ -168,17 +170,40 @@ public class CyclicBarrierHttpServer extends ExternalResource {
         }
     }
 
+    public boolean waitFor() {
+        return waitFor(true, 20);
+    }
+
+    public boolean waitFor(boolean failAtTimeout) {
+        return waitFor(failAtTimeout, 20);
+    }
+
+    public boolean waitFor(int timeoutSeconds) {
+        return waitFor(true, timeoutSeconds);
+    }
+
     /**
      * Blocks until a connection to the URI has been received. No response is returned to the client until
      * {@link #release()} is called.
+     *
+     * @param failAtTimeout if client connection timeout occurs: should the build fail directly or return with 'false'? (if it fails, it might hide the original error)
+     * @param timeoutSeconds timeout in seconds
+     *
+     * @return false on timeout
      */
-    public void waitFor() {
-        long expiry = monotonicClockMillis() + 20000;
+    public boolean waitFor(boolean failAtTimeout, int timeoutSeconds) {
+        long expiry = monotonicClockMillis() + timeoutSeconds * 1000;
         synchronized (lock) {
             while (!connected && !stopped) {
                 long delay = expiry - monotonicClockMillis();
                 if (delay <= 0) {
-                    throw new AssertionFailedError(String.format("Timeout waiting for client to connect to %s.", getUri()));
+                    String message = String.format("Timeout waiting for client to connect to %s.", getUri());
+                    if (failAtTimeout) {
+                        throw new AssertionFailedError(message);
+                    } else {
+                        System.out.println(message);
+                        return false;
+                    }
                 }
                 System.out.println("waiting for client to connect");
                 try {
@@ -191,6 +216,7 @@ public class CyclicBarrierHttpServer extends ExternalResource {
                 throw new AssertionFailedError(String.format("Server was stopped while waiting for client to connect to %s.", getUri()));
             }
             System.out.println("client connected - unblocking");
+            return true;
         }
     }
 
@@ -223,8 +249,12 @@ public class CyclicBarrierHttpServer extends ExternalResource {
      * <p>Note that this method will generally return before the client has received the response.
      */
     public void sync() {
+        sync(20);
+    }
+
+    public void sync(final int timeoutSeconds) {
         synchronized (lock) {
-            waitFor();
+            waitFor(timeoutSeconds);
             release();
         }
     }
