@@ -18,7 +18,11 @@ package org.gradle.performance.fixture;
 
 import org.gradle.performance.measure.MeasuredOperation;
 
+import java.io.File;
+
 public interface BuildExperimentListener {
+    void beforeExperiment(BuildExperimentSpec experimentSpec, File projectDir);
+
     void beforeInvocation(BuildExperimentInvocationInfo invocationInfo);
 
     void afterInvocation(BuildExperimentInvocationInfo invocationInfo, MeasuredOperation operation, MeasurementCallback measurementCallback);
