@@ -26,4 +26,11 @@ public interface TestExecutionResult {
      * Returns the result for the given test class.
      */
     TestClassExecutionResult testClass(String testClass);
+
+    /**
+     * Returns the result for the first test class whose name starts with the given string.
+     */
+    TestClassExecutionResult testClassStartsWith(String testClass);
+
+    int getTotalNumberOfTestClassesExecuted();
 }
