@@ -16,22 +16,43 @@
 
 package org.gradle.test.fixtures.file;
 
+import com.google.common.collect.Lists;
+import com.google.common.io.Files;
 import groovy.lang.Closure;
 import groovy.lang.DelegatesTo;
 import org.apache.commons.io.FileUtils;
 import org.apache.tools.ant.Project;
 import org.apache.tools.ant.taskdefs.Zip;
-import org.codehaus.groovy.runtime.DefaultGroovyMethods;
+import org.codehaus.groovy.runtime.ResourceGroovyMethods;
+import org.gradle.api.UncheckedIOException;
+import org.gradle.internal.hash.HashCode;
+import org.gradle.internal.hash.Hashing;
+import org.gradle.internal.hash.HashingOutputStream;
+import org.gradle.internal.io.NullOutputStream;
 import org.gradle.internal.nativeintegration.filesystem.FileSystem;
 import org.gradle.internal.nativeintegration.services.NativeServices;
+import org.gradle.testing.internal.util.RetryUtil;
 import org.hamcrest.Matcher;
 
-import java.io.*;
+import java.io.BufferedReader;
+import java.io.File;
+import java.io.FileInputStream;
+import java.io.FileOutputStream;
+import java.io.FileReader;
+import java.io.IOException;
+import java.io.ObjectStreamException;
 import java.net.URI;
 import java.net.URISyntaxException;
 import java.net.URL;
-import java.security.MessageDigest;
-import java.util.*;
+import java.util.ArrayList;
+import java.util.Arrays;
+import java.util.Formatter;
+import java.util.HashMap;
+import java.util.List;
+import java.util.Map;
+import java.util.Properties;
+import java.util.Set;
+import java.util.TreeSet;
 import java.util.jar.JarFile;
 import java.util.jar.Manifest;
 
@@ -132,7 +153,17 @@ public class TestFile extends File {
     public TestFile leftShift(Object content) {
         getParentFile().mkdirs();
         try {
-            DefaultGroovyMethods.leftShift(this, content);
+            ResourceGroovyMethods.leftShift(this, content);
+            return this;
+        } catch (IOException e) {
+            throw new RuntimeException(String.format("Could not append to test file '%s'", this), e);
+        }
+    }
+
+    public TestFile setText(String content) {
+        getParentFile().mkdirs();
+        try {
+            ResourceGroovyMethods.setText(this, content);
             return this;
         } catch (IOException e) {
             throw new RuntimeException(String.format("Could not append to test file '%s'", this), e);
@@ -141,6 +172,9 @@ public class TestFile extends File {
 
     public TestFile[] listFiles() {
         File[] children = super.listFiles();
+        if (children == null) {
+            return null;
+        }
         TestFile[] files = new TestFile[children.length];
         for (int i = 0; i < children.length; i++) {
             File child = children[i];
@@ -229,7 +263,7 @@ public class TestFile extends File {
                 FileUtils.copyDirectory(this, target);
             } catch (IOException e) {
                 throw new RuntimeException(String.format("Could not copy test directory '%s' to '%s'", this,
-                        target), e);
+                    target), e);
             }
         } else {
             try {
@@ -244,12 +278,18 @@ public class TestFile extends File {
         new TestFile(target).copyTo(this);
     }
 
-    public void copyFrom(URL resource) {
-        try {
-            FileUtils.copyURLToFile(resource, this);
-        } catch (IOException e) {
-            throw new RuntimeException(e);
-        }
+    public void copyFrom(final URL resource) {
+        final TestFile testFile = this;
+        RetryUtil.retry(new Closure(null, null) {
+            @SuppressWarnings("UnusedDeclaration")
+            void doCall() {
+                try {
+                    FileUtils.copyURLToFile(resource, testFile);
+                } catch (IOException e) {
+                    throw new UncheckedIOException(e);
+                }
+            }
+        });
     }
 
     public void moveToDirectory(File target) {
@@ -277,11 +317,18 @@ public class TestFile extends File {
      * Changes the last modified time for this file so that it is different to and smaller than its current last modified time, within file system resolution.
      */
     public TestFile makeOlder() {
-        // Just move back 2 seconds
-        setLastModified(lastModified() - 2000L);
+        makeOlder(this);
         return this;
     }
 
+    /**
+     * Changes the last modified time for the given file so that it is different to and smaller than its current last modified time, within file system resolution.
+     */
+    public static void makeOlder(File file) {
+        // Just move back 2 seconds
+        assert file.setLastModified(file.lastModified() - 2000L);
+    }
+
     /**
      * Creates a directory structure specified by the given closure.
      * <pre>
@@ -317,6 +364,18 @@ public class TestFile extends File {
         return write(formatter);
     }
 
+    /**
+     * Replaces the given text in the file with new value, asserting that the change was actually applied (ie the text was present).
+     */
+    public void replace(String oldText, String newText) {
+        String original = getText();
+        String newContent = original.replace(oldText, newText);
+        if (original.equals(newContent)) {
+            throw new AssertionError("File " + this + " does not contain the expected text.");
+        }
+        setText(newContent);
+    }
+
     public TestFile assertExists() {
         assertTrue(String.format("%s does not exist", this), exists());
         return this;
@@ -350,7 +409,7 @@ public class TestFile extends File {
         assertIsFile();
         other.assertIsFile();
         assertEquals(String.format("%s is not the same length as %s", this, other), other.length(), this.length());
-        assertTrue(String.format("%s does not have the same content as %s", this, other), Arrays.equals(getHash("MD5"), other.getHash("MD5")));
+        assertTrue(String.format("%s does not have the same content as %s", this, other), getMd5Hash().equals(other.getMd5Hash()));
         return this;
     }
 
@@ -361,14 +420,18 @@ public class TestFile extends File {
         return this;
     }
 
-    private byte[] getHash(String algorithm) {
+    public String getMd5Hash() {
+        return md5(this).toString();
+    }
+
+    public static HashCode md5(File file) {
+        HashingOutputStream hashingStream = new HashingOutputStream(Hashing.md5(), NullOutputStream.INSTANCE);
         try {
-            MessageDigest messageDigest = MessageDigest.getInstance(algorithm);
-            messageDigest.update(FileUtils.readFileToByteArray(this));
-            return messageDigest.digest();
-        } catch (Exception e) {
-            throw new RuntimeException(e);
+            Files.copy(file, hashingStream);
+        } catch (IOException e) {
+            throw new UncheckedIOException(e);
         }
+        return hashingStream.hash();
     }
 
     public void createLink(File target) {
@@ -377,6 +440,15 @@ public class TestFile extends File {
 
     public void createLink(String target) {
         NativeServices.getInstance().get(FileSystem.class).createSymbolicLink(this, new File(target));
+        clearCanonCaches();
+    }
+
+    private void clearCanonCaches() {
+        try {
+            File.createTempFile("doesnt", "matter").delete();
+        } catch (IOException e) {
+            throw new UncheckedIOException(e);
+        }
     }
 
     public String readLink() {
@@ -410,10 +482,17 @@ public class TestFile extends File {
      * Asserts that this file contains exactly the given set of descendants.
      */
     public TestFile assertHasDescendants(String... descendants) {
+        return assertHasDescendants(Arrays.asList(descendants));
+    }
+
+    /**
+     * Convenience method for {@link #assertHasDescendants(String...)}.
+     */
+    public TestFile assertHasDescendants(Iterable<String> descendants) {
         Set<String> actual = new TreeSet<String>();
         assertIsDir();
         visit(actual, "", this);
-        Set<String> expected = new TreeSet<String>(Arrays.asList(descendants));
+        Set<String> expected = new TreeSet<String>(Lists.<String>newArrayList(descendants));
 
         Set<String> extras = new TreeSet<String>(actual);
         extras.removeAll(expected);
@@ -423,6 +502,7 @@ public class TestFile extends File {
         assertEquals(String.format("For dir: %s, extra files: %s, missing files: %s, expected: %s", this, extras, missing, expected), expected, actual);
 
         return this;
+
     }
 
     /**
@@ -433,7 +513,7 @@ public class TestFile extends File {
         Set<String> actual = new TreeSet<String>();
         visit(actual, "", this);
 
-        Set<String> expected = new TreeSet<String>(Arrays.asList(descendants));
+        Set<String> expected = new TreeSet<String>(Lists.newArrayList(descendants));
 
         Set<String> missing = new TreeSet<String>(expected);
         missing.removeAll(actual);
@@ -444,10 +524,8 @@ public class TestFile extends File {
     }
 
     public TestFile assertIsEmptyDir() {
-        if (exists()) {
-            assertIsDir();
-            assertHasDescendants();
-        }
+        assertIsDir();
+        assertHasDescendants();
         return this;
     }
 
@@ -484,7 +562,7 @@ public class TestFile extends File {
             return this;
         }
         throw new AssertionError("Problems creating dir: " + this
-                + ". Diagnostics: exists=" + this.exists() + ", isFile=" + this.isFile() + ", isDirectory=" + this.isDirectory());
+            + ". Diagnostics: exists=" + this.exists() + ", isFile=" + this.isFile() + ", isDirectory=" + this.isDirectory());
     }
 
     public TestFile createDir(Object path) {
@@ -498,6 +576,7 @@ public class TestFile extends File {
 
     /**
      * Attempts to delete this directory, ignoring failures to do so.
+     *
      * @return this
      */
     public TestFile maybeDeleteDir() {
@@ -583,28 +662,28 @@ public class TestFile extends File {
 
     public Snapshot snapshot() {
         assertIsFile();
-        return new Snapshot(lastModified(), getHash("MD5"));
+        return new Snapshot(lastModified(), md5(this));
     }
 
     public void assertHasChangedSince(Snapshot snapshot) {
         Snapshot now = snapshot();
-        assertTrue(now.modTime != snapshot.modTime || !Arrays.equals(now.hash, snapshot.hash));
+        assertTrue(String.format("contents or modification time of %s have not changed", this), now.modTime != snapshot.modTime || !now.hash.equals(snapshot.hash));
     }
 
     public void assertContentsHaveChangedSince(Snapshot snapshot) {
         Snapshot now = snapshot();
-        assertTrue(String.format("contents of %s have not changed", this), !Arrays.equals(now.hash, snapshot.hash));
+        assertNotEquals(String.format("contents of %s have not changed", this), snapshot.hash, now.hash);
     }
 
     public void assertContentsHaveNotChangedSince(Snapshot snapshot) {
         Snapshot now = snapshot();
-        assertArrayEquals(String.format("contents of %s has changed", this), snapshot.hash, now.hash);
+        assertEquals(String.format("contents of %s has changed", this), snapshot.hash, now.hash);
     }
 
     public void assertHasNotChangedSince(Snapshot snapshot) {
         Snapshot now = snapshot();
         assertEquals(String.format("last modified time of %s has changed", this), snapshot.modTime, now.modTime);
-        assertArrayEquals(String.format("contents of %s has changed", this), snapshot.hash, now.hash);
+        assertEquals(String.format("contents of %s has changed", this), snapshot.hash, now.hash);
     }
 
     public void writeProperties(Map<?, ?> properties) {
@@ -627,20 +706,39 @@ public class TestFile extends File {
     }
 
     public ExecOutput exec(Object... args) {
-        return new TestFileHelper(this).execute(Arrays.asList(args), null);
+        return new TestFileHelper(this).executeSuccess(Arrays.asList(args), null);
+    }
+
+    public ExecOutput execWithFailure(List args, List env) {
+        return new TestFileHelper(this).executeFailure(args, env);
     }
 
     public ExecOutput execute(List args, List env) {
-        return new TestFileHelper(this).execute(args, env);
+        return new TestFileHelper(this).executeSuccess(args, env);
+    }
+
+    /**
+     * Relativizes the URI of this file according to the base directory.
+     */
+    public URI relativizeFrom(TestFile baseDir) {
+        return baseDir.toURI().relativize(toURI());
     }
 
     public class Snapshot {
         private final long modTime;
-        private final byte[] hash;
+        private final HashCode hash;
 
-        public Snapshot(long modTime, byte[] hash) {
+        public Snapshot(long modTime, HashCode hash) {
             this.modTime = modTime;
             this.hash = hash;
         }
+
+        public long getModTime() {
+            return modTime;
+        }
+
+        public HashCode getHash() {
+            return hash;
+        }
     }
 }
