@@ -15,9 +15,10 @@
  */
 package org.gradle.api.internal.tasks.compile;
 
-import org.gradle.api.internal.tasks.SimpleWorkResult;
+import org.gradle.api.JavaVersion;
+import org.gradle.api.internal.tasks.compile.reflect.SourcepathIgnoringProxy;
 import org.gradle.api.tasks.WorkResult;
-import org.gradle.api.tasks.compile.CompileOptions;
+import org.gradle.api.tasks.WorkResults;
 import org.gradle.internal.Factory;
 import org.gradle.language.base.internal.compile.Compiler;
 import org.slf4j.Logger;
@@ -28,6 +29,7 @@ import javax.tools.JavaFileObject;
 import javax.tools.StandardJavaFileManager;
 import java.io.Serializable;
 import java.nio.charset.Charset;
+import java.util.Iterator;
 import java.util.List;
 
 public class JdkJavaCompiler implements Compiler<JavaCompileSpec>, Serializable {
@@ -48,15 +50,30 @@ public class JdkJavaCompiler implements Compiler<JavaCompileSpec>, Serializable
             throw new CompilationFailedException();
         }
 
-        return new SimpleWorkResult(true);
+        return WorkResults.didWork(true);
     }
 
     private JavaCompiler.CompilationTask createCompileTask(JavaCompileSpec spec) {
         List<String> options = new JavaCompilerArgumentsBuilder(spec).build();
         JavaCompiler compiler = javaHomeBasedJavaCompilerFactory.create();
-        CompileOptions compileOptions = spec.getCompileOptions();
-        StandardJavaFileManager fileManager = compiler.getStandardFileManager(null, null, compileOptions.getEncoding() != null ? Charset.forName(compileOptions.getEncoding()) : null);
-        Iterable<? extends JavaFileObject> compilationUnits = fileManager.getJavaFileObjectsFromFiles(spec.getSource());
-        return compiler.getTask(null, null, null, options, null, compilationUnits);
+        MinimalJavaCompileOptions compileOptions = spec.getCompileOptions();
+        StandardJavaFileManager standardFileManager = compiler.getStandardFileManager(null, null, compileOptions.getEncoding() != null ? Charset.forName(compileOptions.getEncoding()) : null);
+        Iterable<? extends JavaFileObject> compilationUnits = standardFileManager.getJavaFileObjectsFromFiles(spec.getSource());
+        StandardJavaFileManager fileManager = standardFileManager;
+        if (JavaVersion.current().isJava9Compatible() && emptySourcepathIn(options)) {
+            fileManager = (StandardJavaFileManager) SourcepathIgnoringProxy.proxy(standardFileManager, StandardJavaFileManager.class);
+        }
+        return compiler.getTask(null, fileManager, null, options, null, compilationUnits);
+    }
+
+    private static boolean emptySourcepathIn(List<String> options) {
+        Iterator<String> optionsIter = options.iterator();
+        while (optionsIter.hasNext()) {
+            String current = optionsIter.next();
+            if (current.equals("-sourcepath") || current.equals("--source-path")) {
+                return optionsIter.next().isEmpty();
+            }
+        }
+        return false;
     }
 }
