@@ -18,10 +18,10 @@ package org.gradle.api.internal.tasks.compile.incremental.jar;
 
 import com.google.common.collect.Maps;
 import com.google.common.collect.Sets;
-import com.google.common.hash.HashCode;
+import org.gradle.internal.hash.HashCode;
 
 import java.io.File;
-import java.util.Map;
+import java.util.LinkedHashMap;
 import java.util.Set;
 
 public class JarClasspathSnapshotFactory {
@@ -33,8 +33,8 @@ public class JarClasspathSnapshotFactory {
     }
 
     JarClasspathSnapshot createSnapshot(Iterable<JarArchive> jarArchives) {
-        Map<File, JarSnapshot> jarSnapshots = Maps.newHashMap();
-        Map<File, HashCode> jarHashes = Maps.newHashMap();
+        LinkedHashMap<File, JarSnapshot> jarSnapshots = Maps.newLinkedHashMap();
+        LinkedHashMap<File, HashCode> jarHashes = Maps.newLinkedHashMap();
         Set<String> allClasses = Sets.newHashSet();
         Set<String> duplicateClasses = Sets.newHashSet();
 
