@@ -18,7 +18,8 @@ package org.gradle.language.jvm.plugins;
 
 import org.gradle.api.*;
 import org.gradle.internal.service.ServiceRegistry;
-import org.gradle.jvm.JvmBinarySpec;
+import org.gradle.jvm.internal.JvmAssembly;
+import org.gradle.jvm.internal.WithJvmAssembly;
 import org.gradle.language.base.LanguageSourceSet;
 import org.gradle.language.base.internal.SourceTransformTaskConfig;
 import org.gradle.language.base.internal.registry.LanguageTransform;
@@ -30,12 +31,14 @@ import org.gradle.language.jvm.tasks.ProcessResources;
 import org.gradle.model.Mutate;
 import org.gradle.model.RuleSource;
 import org.gradle.platform.base.BinarySpec;
-import org.gradle.platform.base.LanguageType;
-import org.gradle.platform.base.LanguageTypeBuilder;
+import org.gradle.platform.base.ComponentType;
+import org.gradle.platform.base.TypeBuilder;
 
 import java.util.Collections;
 import java.util.Map;
 
+import static org.gradle.util.CollectionUtils.first;
+
 /**
  * Plugin for packaging JVM resources. Applies the {@link org.gradle.language.base.plugins.ComponentModelBasePlugin}. Registers "resources" language support with the {@link
  * org.gradle.language.jvm.JvmResourceSet}.
@@ -43,15 +46,15 @@ import java.util.Map;
 @Incubating
 public class JvmResourcesPlugin implements Plugin<Project> {
 
+    @Override
     public void apply(final Project project) {
         project.getPluginManager().apply(ComponentModelBasePlugin.class);
     }
 
     @SuppressWarnings("UnusedDeclaration")
     static class Rules extends RuleSource {
-        @LanguageType
-        void registerLanguage(LanguageTypeBuilder<JvmResourceSet> builder) {
-            builder.setLanguageName("resources");
+        @ComponentType
+        void registerLanguage(TypeBuilder<JvmResourceSet> builder) {
             builder.defaultImplementation(DefaultJvmResourceLanguageSourceSet.class);
         }
 
@@ -62,18 +65,27 @@ public class JvmResourcesPlugin implements Plugin<Project> {
     }
 
     private static class JvmResources implements LanguageTransform<JvmResourceSet, org.gradle.jvm.JvmResources> {
+        @Override
+        public String getLanguageName() {
+            return "resources";
+        }
+
+        @Override
         public Class<JvmResourceSet> getSourceSetType() {
             return JvmResourceSet.class;
         }
 
+        @Override
         public Map<String, Class<?>> getBinaryTools() {
             return Collections.emptyMap();
         }
 
+        @Override
         public Class<org.gradle.jvm.JvmResources> getOutputType() {
             return org.gradle.jvm.JvmResources.class;
         }
 
+        @Override
         public SourceTransformTaskConfig getTransformTask() {
             return new SourceTransformTaskConfig() {
                 public String getTaskPrefix() {
@@ -87,15 +99,20 @@ public class JvmResourcesPlugin implements Plugin<Project> {
                 public void configureTask(Task task, BinarySpec binary, LanguageSourceSet sourceSet, ServiceRegistry serviceRegistry) {
                     ProcessResources resourcesTask = (ProcessResources) task;
                     JvmResourceSet resourceSet = (JvmResourceSet) sourceSet;
-                    JvmBinarySpec jvmBinary = (JvmBinarySpec) binary;
                     resourcesTask.from(resourceSet.getSource());
-                    resourcesTask.setDestinationDir(jvmBinary.getResourcesDir());
-                    jvmBinary.getTasks().getJar().dependsOn(resourcesTask);
+
+                    // The first directory is the one created by JvmComponentPlugin.configureJvmBinaries()
+                    // to be used as the default output directory for processed resources
+                    JvmAssembly assembly = ((WithJvmAssembly) binary).getAssembly();
+                    resourcesTask.setDestinationDir(first(assembly.getResourceDirectories()));
+
+                    assembly.builtBy(resourcesTask);
                 }
             };
         }
+        @Override
         public boolean applyToBinary(BinarySpec binary) {
-            return binary instanceof JvmBinarySpec;
+            return binary instanceof WithJvmAssembly;
         }
     }
 }
