@@ -17,14 +17,17 @@
 package org.gradle.api.internal.tasks.scala;
 
 import com.google.common.collect.ImmutableList;
-import com.typesafe.zinc.*;
-import org.gradle.api.internal.tasks.SimpleWorkResult;
+import com.typesafe.zinc.IncOptions;
+import com.typesafe.zinc.Inputs;
 import org.gradle.api.internal.tasks.compile.CompilationFailedException;
-import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.api.internal.tasks.compile.JavaCompilerArgumentsBuilder;
 import org.gradle.api.logging.Logger;
 import org.gradle.api.logging.Logging;
 import org.gradle.api.tasks.WorkResult;
+import org.gradle.api.tasks.WorkResults;
+import org.gradle.internal.time.Time;
+import org.gradle.internal.time.Timer;
+import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.util.GFileUtils;
 import scala.Option;
 
@@ -57,7 +60,9 @@ public class ZincScalaCompiler implements Compiler<ScalaJavaJointCompileSpec>, S
 
             final xsbti.Logger logger = new SbtLoggerAdapter();
 
+            Timer timer = Time.startTimer();
             com.typesafe.zinc.Compiler compiler = ZincScalaCompilerFactory.createParallelSafeCompiler(scalaClasspath, zincClasspath, logger, gradleUserHome);
+            LOGGER.info("Initialized Zinc Scala compiler: {}", timer.getElapsed());
 
             List<String> scalacOptions = new ZincScalaCompilerArgumentsGenerator().generate(spec);
             List<String> javacOptions = new JavaCompilerArgumentsBuilder(spec).includeClasspath(false).noEmptySourcePath().build();
@@ -70,14 +75,16 @@ public class ZincScalaCompiler implements Compiler<ScalaJavaJointCompileSpec>, S
             if (spec.getScalaCompileOptions().isForce()) {
                 GFileUtils.deleteDirectory(spec.getDestinationDir());
             }
+            LOGGER.info("Prepared Zinc Scala inputs: {}", timer.getElapsed());
 
             try {
                 compiler.compile(inputs, logger);
             } catch (xsbti.CompileFailed e) {
                 throw new CompilationFailedException(e);
             }
+            LOGGER.info("Completed Scala compilation: {}", timer.getElapsed());
 
-            return new SimpleWorkResult(true);
+            return WorkResults.didWork(true);
         }
 
         private static IncOptions getIncOptions() {
