@@ -18,10 +18,11 @@ package org.gradle.language.scala.tasks;
 
 import org.gradle.api.Incubating;
 import org.gradle.api.internal.tasks.scala.ScalaJavaJointCompileSpec;
+import org.gradle.api.tasks.Nested;
 import org.gradle.language.base.internal.compile.Compiler;
 import org.gradle.language.base.internal.compile.CompilerUtil;
-import org.gradle.language.scala.internal.toolchain.ScalaToolChainInternal;
 import org.gradle.language.scala.ScalaPlatform;
+import org.gradle.language.scala.internal.toolchain.ScalaToolChainInternal;
 
 import javax.inject.Inject;
 
@@ -38,6 +39,7 @@ public class PlatformScalaCompile extends AbstractScalaCompile {
         super(new BaseScalaCompileOptions());
     }
 
+    @Nested
     public ScalaPlatform getPlatform() {
         return platform;
     }
