@@ -16,6 +16,7 @@
 package org.gradle.tooling.internal.provider;
 
 import org.gradle.StartParameter;
+import org.gradle.tooling.internal.protocol.ModelIdentifier;
 
 public class BuildModelAction extends SubscribableBuildAction {
     private final StartParameter startParameter;
@@ -42,4 +43,8 @@ public class BuildModelAction extends SubscribableBuildAction {
         return runTasks;
     }
 
+    public boolean isModelRequest() {
+        return !ModelIdentifier.NULL_MODEL.equals(modelName);
+    }
+
 }
