@@ -16,7 +16,7 @@
 package org.gradle.tooling.internal.provider;
 
 import org.gradle.api.JavaVersion;
-import org.gradle.api.Nullable;
+import org.gradle.api.UncheckedIOException;
 import org.gradle.initialization.BuildCancellationToken;
 import org.gradle.initialization.BuildLayoutParameters;
 import org.gradle.initialization.DefaultBuildCancellationToken;
@@ -40,7 +40,9 @@ import org.gradle.tooling.internal.protocol.ConnectionParameters;
 import org.gradle.tooling.internal.protocol.ConnectionVersion4;
 import org.gradle.tooling.internal.protocol.InternalBuildAction;
 import org.gradle.tooling.internal.protocol.InternalBuildActionExecutor;
+import org.gradle.tooling.internal.protocol.InternalBuildActionVersion2;
 import org.gradle.tooling.internal.protocol.InternalCancellableConnection;
+import org.gradle.tooling.internal.protocol.InternalParameterAcceptingConnection;
 import org.gradle.tooling.internal.protocol.InternalCancellationToken;
 import org.gradle.tooling.internal.protocol.InternalConnection;
 import org.gradle.tooling.internal.protocol.InternalUnsupportedModelException;
@@ -57,20 +59,26 @@ import org.gradle.tooling.internal.provider.connection.ProviderBuildResult;
 import org.gradle.tooling.internal.provider.connection.ProviderConnectionParameters;
 import org.gradle.tooling.internal.provider.connection.ProviderOperationParameters;
 import org.gradle.tooling.internal.provider.test.ProviderInternalTestExecutionRequest;
+import org.gradle.util.DeprecationLogger;
 import org.gradle.util.GradleVersion;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
+import javax.annotation.Nullable;
 import java.io.File;
+import java.io.IOException;
 
 public class DefaultConnection implements ConnectionVersion4, InternalConnection, BuildActionRunner,
-    ConfigurableConnection, ModelBuilder, InternalBuildActionExecutor, InternalCancellableConnection, StoppableConnection, InternalTestExecutionConnection {
+    ConfigurableConnection, ModelBuilder, InternalBuildActionExecutor, InternalCancellableConnection, InternalParameterAcceptingConnection, StoppableConnection, InternalTestExecutionConnection {
     private static final Logger LOGGER = LoggerFactory.getLogger(DefaultConnection.class);
+    private static final String UNSUPPORTED_MESSAGE = "Support for clients using a tooling API version older than 2.0 was removed in Gradle 3.0. %sYou should upgrade your tooling API client to version 3.0 or later.";
+    private static final String DEPRECATION_MESSAGE = "Support for clients using a tooling API version older than 3.0 was deprecated and will be removed in Gradle 5.0. %sYou should upgrade your tooling API client to version 3.0 or later.\n";
+
     private static final GradleVersion MIN_CLIENT_VERSION = GradleVersion.version("2.0");
+    private static final GradleVersion MIN_LTS_CLIENT_VERSION = GradleVersion.version("3.0");
     private ProtocolToModelAdapter adapter;
     private ServiceRegistry services;
     private ProviderConnection connection;
-    private boolean supportedConsumerVersion;
     @Nullable // not provided by older client versions
     private GradleVersion consumerVersion;
 
@@ -85,7 +93,7 @@ public class DefaultConnection implements ConnectionVersion4, InternalConnection
      * This is used by consumers 1.2-rc-1 and later.
      */
     public void configure(ConnectionParameters parameters) {
-        UnsupportedJavaRuntimeException.assertUsingVersion("Gradle", JavaVersion.VERSION_1_7);
+        assertUsingJava7();
         ProviderConnectionParameters providerConnectionParameters = new ProtocolToModelAdapter().adapt(ProviderConnectionParameters.class, parameters);
         File gradleUserHomeDir = providerConnectionParameters.getGradleUserHomeDir(null);
         if (gradleUserHomeDir == null) {
@@ -94,7 +102,15 @@ public class DefaultConnection implements ConnectionVersion4, InternalConnection
         initializeServices(gradleUserHomeDir);
         connection.configure(providerConnectionParameters);
         consumerVersion = GradleVersion.version(providerConnectionParameters.getConsumerVersion());
-        supportedConsumerVersion = consumerVersion.compareTo(MIN_CLIENT_VERSION) >= 0;
+    }
+
+    private void assertUsingJava7() {
+        try {
+            UnsupportedJavaRuntimeException.assertUsingVersion("Gradle", JavaVersion.VERSION_1_7);
+        } catch (IllegalArgumentException e) {
+            // https://github.com/gradle/gradle/issues/3317
+            DeprecationLogger.nagUserWith(e.getMessage());
+        }
     }
 
     private void initializeServices(File gradleUserHomeDir) {
@@ -199,7 +215,7 @@ public class DefaultConnection implements ConnectionVersion4, InternalConnection
     }
 
     /**
-     * This is used by consumers 2.1-rc-1 and later.
+     * This is used by consumers 2.1-rc-1 to 4.3
      */
     public <T> BuildResult<T> run(InternalBuildAction<T> action, InternalCancellationToken cancellationToken, BuildParameters operationParameters)
         throws BuildExceptionVersion1, InternalUnsupportedBuildArgumentException, IllegalStateException {
@@ -209,6 +225,17 @@ public class DefaultConnection implements ConnectionVersion4, InternalConnection
         return new ProviderBuildResult<T>((T) results);
     }
 
+    /**
+     * This is used by consumers 4.4 and later
+     */
+    public <T> BuildResult<T> run(InternalBuildActionVersion2<T> action, InternalCancellationToken cancellationToken, BuildParameters operationParameters)
+        throws BuildExceptionVersion1, InternalUnsupportedBuildArgumentException, IllegalStateException {
+        ProviderOperationParameters providerParameters = validateAndConvert(operationParameters);
+        BuildCancellationToken buildCancellationToken = new InternalCancellationTokenAdapter(cancellationToken);
+        Object results = connection.run(action, buildCancellationToken, providerParameters);
+        return new ProviderBuildResult<T>((T) results);
+    }
+
     /**
      * This is used by consumers 2.6-rc-1 and later
      */
@@ -221,24 +248,44 @@ public class DefaultConnection implements ConnectionVersion4, InternalConnection
         return new ProviderBuildResult<Object>(results);
     }
 
-    private UnsupportedVersionException unsupportedConnectionException() {
-        StringBuilder message = new StringBuilder("Support for clients using a tooling API version older than 2.0 was removed in Gradle 3.0. ");
-        if (consumerVersion != null) {
-            // Consumer version is provided by client 1.2 and later
-            message.append("You are currently using tooling API version ");
-            message.append(consumerVersion.getVersion());
-            message.append(". ");
-        }
-        message.append("You should upgrade your tooling API client to version 2.0 or later.");
-        return new UnsupportedVersionException(message.toString());
-    }
-
     private ProviderOperationParameters validateAndConvert(BuildParameters buildParameters) {
         LOGGER.info("Tooling API is using target Gradle version: {}.", GradleVersion.current().getVersion());
-        UnsupportedJavaRuntimeException.assertUsingVersion("Gradle", JavaVersion.VERSION_1_7);
-        if (!supportedConsumerVersion) {
+        assertUsingJava7();
+
+        checkUnsupportedTapiVersion();
+        ProviderOperationParameters parameters = adapter.builder(ProviderOperationParameters.class).mixInTo(ProviderOperationParameters.class, BuildLogLevelMixIn.class).build(buildParameters);
+        checkDeprecatedTapiVersion(parameters);
+
+        DeprecationLogger.reset();
+        return parameters;
+    }
+
+    private UnsupportedVersionException unsupportedConnectionException() {
+        return new UnsupportedVersionException(String.format(UNSUPPORTED_MESSAGE, createCurrentVersionMessage()));
+    }
+
+    private String createCurrentVersionMessage() {
+        if (consumerVersion == null) {
+            return "";
+        } else {
+            // Consumer version is provided by client 1.2 and later
+            return String.format("You are currently using tooling API version %s. ", consumerVersion.getVersion());
+        }
+    }
+
+    private void checkUnsupportedTapiVersion() {
+        if (consumerVersion == null || consumerVersion.compareTo(MIN_CLIENT_VERSION) < 0) {
             throw unsupportedConnectionException();
         }
-        return adapter.builder(ProviderOperationParameters.class).mixInTo(ProviderOperationParameters.class, BuildLogLevelMixIn.class).build(buildParameters);
+    }
+
+    private void checkDeprecatedTapiVersion(ProviderOperationParameters parameters) {
+        if (consumerVersion.compareTo(MIN_LTS_CLIENT_VERSION) < 0 && parameters.getStandardOutput() != null) {
+            try {
+                parameters.getStandardOutput().write(String.format(DEPRECATION_MESSAGE, createCurrentVersionMessage()).getBytes());
+            } catch (IOException e) {
+                throw new UncheckedIOException(e);
+            }
+        }
     }
 }
