@@ -17,24 +17,18 @@
 package org.gradle.model.internal.core;
 
 import com.google.common.base.Optional;
-import org.gradle.api.Nullable;
 import org.gradle.internal.Cast;
 import org.gradle.internal.reflect.DirectInstantiator;
-import org.gradle.model.collection.internal.ChildNodeInitializerStrategyAccessor;
 import org.gradle.model.internal.core.rule.describe.ModelRuleDescriptor;
-import org.gradle.model.internal.manage.instance.ManagedInstance;
 import org.gradle.model.internal.type.ModelType;
 
+import javax.annotation.Nullable;
 import java.util.Collections;
-import java.util.List;
 
 /**
  * Should be used along with {@code PolymorphicModelMapProjection}.
  */
-// TODO:DAZ Removed `P extends ModelMap<E>` : find a generics expert to help me put it back
 public class SpecializedModelMapProjection<P, E> implements ModelProjection {
-    private static final ModelType<ManagedInstance> MANAGED_INSTANCE_TYPE = ModelType.of(ManagedInstance.class);
-
     private final ModelType<P> publicType;
     private final ModelType<E> elementType;
 
@@ -49,19 +43,14 @@ public class SpecializedModelMapProjection<P, E> implements ModelProjection {
     }
 
     @Override
-    public Iterable<String> getReadableTypeDescriptions(MutableModelNode node) {
-        return getWritableTypeDescriptions(node);
-    }
-
-    @Override
-    public Iterable<String> getWritableTypeDescriptions(MutableModelNode node) {
+    public Iterable<String> getTypeDescriptions(MutableModelNode node) {
         return Collections.singleton(publicType.toString());
     }
 
     @Nullable
     @Override
     public <T> ModelView<? extends T> asImmutable(ModelType<T> type, MutableModelNode node, @Nullable ModelRuleDescriptor ruleDescriptor) {
-        if (canBeViewedAsImmutable(type)) {
+        if (canBeViewedAs(type)) {
             return Cast.uncheckedCast(toView(node, ruleDescriptor, false));
         } else {
             return null;
@@ -70,8 +59,8 @@ public class SpecializedModelMapProjection<P, E> implements ModelProjection {
 
     @Nullable
     @Override
-    public <T> ModelView<? extends T> asMutable(ModelType<T> type, MutableModelNode node, ModelRuleDescriptor ruleDescriptor, List<ModelView<?>> implicitDependencies) {
-        if (canBeViewedAsMutable(type)) {
+    public <T> ModelView<? extends T> asMutable(ModelType<T> type, MutableModelNode node, ModelRuleDescriptor ruleDescriptor) {
+        if (canBeViewedAs(type)) {
             return Cast.uncheckedCast(toView(node, ruleDescriptor, true));
         } else {
             return null;
@@ -80,44 +69,14 @@ public class SpecializedModelMapProjection<P, E> implements ModelProjection {
 
     private ModelView<P> toView(MutableModelNode modelNode, ModelRuleDescriptor ruleDescriptor, boolean mutable) {
         ChildNodeInitializerStrategy<? super E> creatorStrategy = creatorStrategyAccessor.getStrategy(modelNode);
-        DefaultModelViewState state = new DefaultModelViewState(publicType, ruleDescriptor, mutable, true);
-        String description = publicType.getDisplayName() + " '" + modelNode.getPath() + "'";
-        P instance = DirectInstantiator.instantiate(viewImpl, description, elementType, ruleDescriptor, modelNode, false, state, creatorStrategy);
+        DefaultModelViewState state = new DefaultModelViewState(modelNode.getPath(), publicType, ruleDescriptor, mutable, !mutable);
+        P instance = DirectInstantiator.instantiate(viewImpl, publicType, elementType, ruleDescriptor, modelNode, state, creatorStrategy);
         return InstanceModelView.of(modelNode.getPath(), publicType, instance, state.closer());
     }
 
     @Override
-    public boolean equals(Object o) {
-        if (this == o) {
-            return true;
-        }
-        if (o == null || getClass() != o.getClass()) {
-            return false;
-        }
-        if (!super.equals(o)) {
-            return false;
-        }
-
-        SpecializedModelMapProjection<?, ?> that = (SpecializedModelMapProjection<?, ?>) o;
-        return publicType.equals(that.publicType) && viewImpl.equals(that.viewImpl);
-    }
-
-    @Override
-    public int hashCode() {
-        int result = super.hashCode();
-        result = 31 * result + publicType.hashCode();
-        result = 31 * result + viewImpl.hashCode();
-        return result;
-    }
-
-    @Override
-    public <T> boolean canBeViewedAsMutable(ModelType<T> targetType) {
-        return targetType.equals(publicType) || targetType.equals(ModelType.UNTYPED) || targetType.equals(MANAGED_INSTANCE_TYPE);
-    }
-
-    @Override
-    public <T> boolean canBeViewedAsImmutable(ModelType<T> targetType) {
-        return canBeViewedAsMutable(targetType);
+    public <T> boolean canBeViewedAs(ModelType<T> targetType) {
+        return targetType.isAssignableFrom(publicType);
     }
 
     @Override
