@@ -23,7 +23,7 @@ import org.gradle.api.UncheckedIOException;
 import java.io.IOException;
 
 @ThreadSafe
-public class NestedModelRuleDescriptor extends AbstractModelRuleDescriptor {
+class NestedModelRuleDescriptor extends AbstractModelRuleDescriptor {
 
     private final ModelRuleDescriptor parent;
     private final ModelRuleDescriptor child;
@@ -33,10 +33,7 @@ public class NestedModelRuleDescriptor extends AbstractModelRuleDescriptor {
         this.child = child;
     }
 
-    public NestedModelRuleDescriptor(ModelRuleDescriptor parent, String child) {
-        this(parent, new SimpleModelRuleDescriptor(child));
-    }
-
+    @Override
     public void describeTo(Appendable appendable) {
         parent.describeTo(appendable);
         try {
@@ -64,12 +61,4 @@ public class NestedModelRuleDescriptor extends AbstractModelRuleDescriptor {
     public int hashCode() {
         return Objects.hashCode(parent, child);
     }
-
-    public static ModelRuleDescriptor append(ModelRuleDescriptor parent, String str) {
-        return new NestedModelRuleDescriptor(parent, new SimpleModelRuleDescriptor(str));
-    }
-
-    public static ModelRuleDescriptor append(ModelRuleDescriptor parent, String str, Object... args) {
-        return append(parent, String.format(str, args));
-    }
 }
