@@ -16,15 +16,18 @@
 
 package org.gradle.model.internal.inspect;
 
-import org.gradle.api.Nullable;
-import org.gradle.api.specs.Spec;
-import org.gradle.model.internal.core.ExtractedModelRule;
+import javax.annotation.Nullable;
 
 public interface MethodModelRuleExtractor {
-    Spec<MethodRuleDefinition<?, ?>> getSpec();
+    boolean isSatisfiedBy(MethodRuleDefinition<?, ?> definition);
 
     String getDescription();
 
+    /**
+     * Should only be called when {@link #isSatisfiedBy(MethodRuleDefinition)} return true.
+     *
+     * @return The extracted rule, or null when there are problems with the rule definition. Problems should be collected using the given context.
+     */
     @Nullable
-    <R, S> ExtractedModelRule registration(MethodRuleDefinition<R, S> ruleDefinition);
+    <R, S> ExtractedModelRule registration(MethodRuleDefinition<R, S> ruleDefinition, MethodModelRuleExtractionContext context);
 }
