@@ -18,12 +18,14 @@ package org.gradle.platform.base;
 
 import org.gradle.api.Action;
 import org.gradle.api.DomainObjectSet;
+import org.gradle.api.Incubating;
 import org.gradle.api.Task;
 import org.gradle.model.internal.core.UnmanagedStruct;
 
 /**
  * A collection of tasks associated to a binary
- * */
+ */
+@Incubating
 @UnmanagedStruct
 public interface BinaryTasksCollection extends DomainObjectSet<Task> {
     /**
@@ -37,10 +39,15 @@ public interface BinaryTasksCollection extends DomainObjectSet<Task> {
     String taskName(String verb, String object);
 
     /**
-     * The 'lifecycle' task, that can be used to construct this binary.
+     * The task that can be used to assemble this binary.
      */
     Task getBuild();
 
+    /**
+     * The task that can be used to check this binary.
+     */
+    Task getCheck();
+
     <T extends Task> void create(String name, Class<T> type, Action<? super T> config);
 
 }
