@@ -16,9 +16,10 @@
 
 package org.gradle.platform.base.internal.dependents;
 
-import org.gradle.api.Nullable;
 import org.gradle.platform.base.internal.BinarySpecInternal;
 
+import javax.annotation.Nullable;
+
 /**
  * Resolve dependent binaries.
  */
