@@ -18,17 +18,15 @@ package org.gradle.jvm.internal.services;
 
 import org.gradle.api.internal.artifacts.ResolveContext;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ComponentResolvers;
-import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.DelegatingComponentResolvers;
 import org.gradle.api.internal.artifacts.ivyservice.ivyresolve.ResolverProviderFactory;
 import org.gradle.api.internal.project.taskfactory.ClasspathPropertyAnnotationHandler;
 import org.gradle.api.internal.project.taskfactory.CompileClasspathPropertyAnnotationHandler;
 import org.gradle.api.internal.resolve.DefaultLocalLibraryResolver;
 import org.gradle.api.internal.resolve.LocalLibraryDependencyResolver;
 import org.gradle.api.internal.resolve.ProjectModelResolver;
-import org.gradle.api.internal.resolve.VariantSelector;
+import org.gradle.api.internal.resolve.VariantBinarySelector;
 import org.gradle.internal.service.ServiceRegistration;
-import org.gradle.internal.service.ServiceRegistry;
-import org.gradle.internal.service.scopes.PluginServiceRegistry;
+import org.gradle.internal.service.scopes.AbstractPluginServiceRegistry;
 import org.gradle.jvm.JvmBinarySpec;
 import org.gradle.jvm.internal.JarBinaryRenderer;
 import org.gradle.jvm.internal.resolve.DefaultJavaPlatformVariantAxisCompatibility;
@@ -44,7 +42,9 @@ import org.gradle.jvm.toolchain.internal.JavaInstallationProbe;
 import org.gradle.model.internal.manage.schema.ModelSchemaStore;
 import org.gradle.process.internal.ExecActionFactory;
 
-public class PlatformJvmServices implements PluginServiceRegistry {
+import java.util.List;
+
+public class PlatformJvmServices extends AbstractPluginServiceRegistry {
     @Override
     public void registerGlobalServices(ServiceRegistration registration) {
         registration.add(JarBinaryRenderer.class);
@@ -53,26 +53,14 @@ public class PlatformJvmServices implements PluginServiceRegistry {
         registration.add(CompileClasspathPropertyAnnotationHandler.class);
     }
 
-    @Override
-    public void registerBuildSessionServices(ServiceRegistration registration) {
-    }
-
     @Override
     public void registerBuildServices(ServiceRegistration registration) {
         registration.addProvider(new BuildScopeServices());
     }
 
-    @Override
-    public void registerGradleServices(ServiceRegistration registration) {
-    }
-
-    @Override
-    public void registerProjectServices(ServiceRegistration registration) {
-    }
-
     private class BuildScopeServices {
-        LocalLibraryDependencyResolverFactory createResolverProviderFactory(ProjectModelResolver projectModelResolver, ServiceRegistry registry) {
-            return new LocalLibraryDependencyResolverFactory(projectModelResolver, registry);
+        LocalLibraryDependencyResolverFactory createResolverProviderFactory(ProjectModelResolver projectModelResolver, ModelSchemaStore schemaStore, List<VariantAxisCompatibilityFactory> factories) {
+            return new LocalLibraryDependencyResolverFactory(projectModelResolver, schemaStore, factories);
         }
 
         JavaInstallationProbe createJavaInstallationProbe(ExecActionFactory factory) {
@@ -82,11 +70,13 @@ public class PlatformJvmServices implements PluginServiceRegistry {
 
     public static class LocalLibraryDependencyResolverFactory implements ResolverProviderFactory {
         private final ProjectModelResolver projectModelResolver;
-        private final ServiceRegistry registry;
+        private final ModelSchemaStore schemaStore;
+        private final List<VariantAxisCompatibilityFactory> factories;
 
-        public LocalLibraryDependencyResolverFactory(ProjectModelResolver projectModelResolver, ServiceRegistry registry) {
+        public LocalLibraryDependencyResolverFactory(ProjectModelResolver projectModelResolver, ModelSchemaStore schemaStore, List<VariantAxisCompatibilityFactory> factories) {
             this.projectModelResolver = projectModelResolver;
-            this.registry = registry;
+            this.schemaStore = schemaStore;
+            this.factories = factories;
         }
 
         @Override
@@ -96,20 +86,17 @@ public class PlatformJvmServices implements PluginServiceRegistry {
 
         @Override
         public ComponentResolvers create(ResolveContext context) {
-            final ModelSchemaStore schemaStore = registry.get(ModelSchemaStore.class);
             VariantsMetaData variants = ((JvmLibraryResolveContext) context).getVariants();
-            VariantSelector variantSelector = new JvmVariantSelector(registry.getAll(VariantAxisCompatibilityFactory.class), JvmBinarySpec.class, schemaStore, variants);
+            VariantBinarySelector variantSelector = new JvmVariantSelector(factories, JvmBinarySpec.class, schemaStore, variants);
             JvmLocalLibraryMetaDataAdapter libraryMetaDataAdapter = new JvmLocalLibraryMetaDataAdapter();
-            LocalLibraryDependencyResolver delegate =
-                    new LocalLibraryDependencyResolver(
-                            JvmBinarySpec.class,
-                            projectModelResolver,
-                            new DefaultLocalLibraryResolver(),
-                            variantSelector,
-                            libraryMetaDataAdapter,
-                            new DefaultLibraryResolutionErrorMessageBuilder(variants, schemaStore)
-                    );
-            return DelegatingComponentResolvers.of(delegate);
+            return new LocalLibraryDependencyResolver(
+                    JvmBinarySpec.class,
+                    projectModelResolver,
+                    new DefaultLocalLibraryResolver(),
+                    variantSelector,
+                    libraryMetaDataAdapter,
+                    new DefaultLibraryResolutionErrorMessageBuilder(variants, schemaStore)
+            );
         }
     }
 }
