@@ -17,39 +17,40 @@
 package org.gradle.nativeplatform.fixtures;
 
 import com.google.common.base.Joiner;
+import com.google.common.collect.ImmutableSet;
 import com.google.common.collect.Lists;
-import net.rubygrapefruit.platform.SystemInfo;
-import net.rubygrapefruit.platform.WindowsRegistry;
-import org.gradle.api.Nullable;
 import org.gradle.api.internal.file.TestFiles;
 import org.gradle.api.specs.Spec;
 import org.gradle.internal.nativeintegration.ProcessEnvironment;
 import org.gradle.internal.os.OperatingSystem;
+import org.gradle.nativeplatform.fixtures.msvcpp.VisualStudioLocatorTestFixture;
+import org.gradle.nativeplatform.fixtures.msvcpp.VisualStudioVersion;
 import org.gradle.nativeplatform.platform.internal.DefaultNativePlatform;
 import org.gradle.nativeplatform.toolchain.Clang;
 import org.gradle.nativeplatform.toolchain.Gcc;
+import org.gradle.nativeplatform.toolchain.Swiftc;
 import org.gradle.nativeplatform.toolchain.VisualCpp;
-import org.gradle.nativeplatform.toolchain.internal.gcc.version.GccVersionDeterminer;
-import org.gradle.nativeplatform.toolchain.internal.gcc.version.GccVersionResult;
-import org.gradle.nativeplatform.toolchain.internal.msvcpp.DefaultVisualStudioLocator;
+import org.gradle.nativeplatform.toolchain.internal.gcc.metadata.GccMetadata;
+import org.gradle.nativeplatform.toolchain.internal.gcc.metadata.GccMetadataProvider;
 import org.gradle.nativeplatform.toolchain.internal.msvcpp.VisualStudioInstall;
 import org.gradle.nativeplatform.toolchain.internal.msvcpp.VisualStudioLocator;
 import org.gradle.nativeplatform.toolchain.plugins.ClangCompilerPlugin;
 import org.gradle.nativeplatform.toolchain.plugins.GccCompilerPlugin;
 import org.gradle.nativeplatform.toolchain.plugins.MicrosoftVisualCppCompilerPlugin;
+import org.gradle.nativeplatform.toolchain.plugins.SwiftCompilerPlugin;
 import org.gradle.test.fixtures.file.TestFile;
 import org.gradle.testfixtures.internal.NativeServicesTestFixture;
 import org.gradle.util.CollectionUtils;
 import org.gradle.util.VersionNumber;
 
+import javax.annotation.Nullable;
 import java.io.File;
 import java.util.ArrayList;
 import java.util.Collections;
 import java.util.List;
+import java.util.Set;
 
-import static org.gradle.nativeplatform.fixtures.VisualStudioVersion.VISUALSTUDIO_2012;
-import static org.gradle.nativeplatform.fixtures.VisualStudioVersion.VISUALSTUDIO_2013;
-import static org.gradle.nativeplatform.fixtures.VisualStudioVersion.VISUALSTUDIO_2015;
+import static org.gradle.nativeplatform.fixtures.msvcpp.VisualStudioVersion.*;
 
 public class AvailableToolChains {
     private static List<ToolChainCandidate> toolChains;
@@ -93,9 +94,14 @@ public class AvailableToolChains {
                 compilers.addAll(findVisualCpps());
                 compilers.add(findMinGW());
                 compilers.add(findCygwin());
-            } else {
-                compilers.add(findGcc());
+            } else if (OperatingSystem.current().isMacOsX()) {
                 compilers.add(findClang());
+                compilers.addAll(findGccs(false));
+                compilers.add(findSwiftc());
+            } else {
+                compilers.addAll(findGccs(true));
+                compilers.add(findClang());
+                compilers.add(findSwiftc());
             }
             toolChains = compilers;
         }
@@ -118,15 +124,14 @@ public class AvailableToolChains {
         return CollectionUtils.findFirst(VisualStudioVersion.values(), new Spec<VisualStudioVersion>() {
             @Override
             public boolean isSatisfiedBy(VisualStudioVersion candidate) {
-                return candidate.getVisualCppVersion().equals(version);
+                return candidate.getVersion().getMajor() == version.getMajor();
             }
         });
     }
 
     static private List<ToolChainCandidate> findVisualCpps() {
         // Search in the standard installation locations
-        VisualStudioLocator vsLocator = new DefaultVisualStudioLocator(OperatingSystem.current(), NativeServicesTestFixture.getInstance().get(WindowsRegistry.class), NativeServicesTestFixture.getInstance().get(SystemInfo.class));
-        final List<VisualStudioLocator.SearchResult> searchResults = vsLocator.locateAllVisualStudioVersions();
+        final List<VisualStudioLocator.SearchResult> searchResults = VisualStudioLocatorTestFixture.getVisualStudioLocator().locateAllVisualStudioVersions();
 
         List<ToolChainCandidate> toolChains = Lists.newArrayList();
 
@@ -166,24 +171,44 @@ public class AvailableToolChains {
         return new UnavailableToolChain("gcc cygwin");
     }
 
-    static private ToolChainCandidate findGcc() {
-        GccVersionDeterminer versionDeterminer = GccVersionDeterminer.forGcc(TestFiles.execActionFactory());
+    static private List<ToolChainCandidate> findGccs(boolean mustFind) {
+        GccMetadataProvider versionDeterminer = GccMetadataProvider.forGcc(TestFiles.execActionFactory());
 
-        List<File> gppCandidates = OperatingSystem.current().findAllInPath("g++");
-        for (int i = 0; i < gppCandidates.size(); i++) {
-            File candidate = gppCandidates.get(i);
-            GccVersionResult version = versionDeterminer.getGccMetaData(candidate, Collections.<String>emptyList());
-            if (version.isAvailable()) {
-                InstalledGcc gcc = new InstalledGcc("gcc");
-                if (i > 0) {
-                    // Not the first g++ in the path, needs the path variable updated
-                    gcc.inPath(candidate.getParentFile());
+        Set<File> gppCandidates = ImmutableSet.copyOf(OperatingSystem.current().findAllInPath("g++"));
+        List<ToolChainCandidate> toolChains = Lists.newArrayList();
+        if (!gppCandidates.isEmpty()) {
+            File firstInPath = gppCandidates.iterator().next();
+            for (File candidate : gppCandidates) {
+                GccMetadata version = versionDeterminer.getCompilerMetaData(candidate, Collections.<String>emptyList());
+                if (version.isAvailable()) {
+                    InstalledGcc gcc = new InstalledGcc("gcc" + " " + version.getVersion());
+                    if (!candidate.equals(firstInPath)) {
+                        // Not the first g++ in the path, needs the path variable updated
+                        gcc.inPath(candidate.getParentFile());
+                    }
+                    toolChains.add(gcc);
                 }
-                return gcc;
             }
         }
+        if (mustFind && toolChains.isEmpty()) {
+            toolChains.add(new UnavailableToolChain("gcc"));
+        }
 
-        return new UnavailableToolChain("gcc");
+        return toolChains;
+    }
+
+    static ToolChainCandidate findSwiftc() {
+        File compilerExe = new File("/opt/swift/latest/usr/bin/swiftc");
+        if (compilerExe.isFile()) {
+            return new InstalledSwiftc("swiftc").inPath(compilerExe.getParentFile());
+        }
+
+        List<File> swiftcCandidates = OperatingSystem.current().findAllInPath("swiftc");
+        if (!swiftcCandidates.isEmpty()) {
+            return new InstalledSwiftc("swiftc");
+        }
+
+        return new UnavailableToolChain("swiftc");
     }
 
     public static abstract class ToolChainCandidate {
@@ -307,6 +332,15 @@ public class AvailableToolChains {
             return Collections.emptyList();
         }
 
+        protected List<String> toRuntimeEnv() {
+            if (pathEntries.isEmpty()) {
+                return Collections.emptyList();
+            }
+
+            String path = Joiner.on(File.pathSeparator).join(pathEntries) + File.pathSeparator + System.getenv(pathVarName);
+            return Collections.singletonList(pathVarName + "=" + path);
+        }
+
         public String getId() {
             return displayName.replaceAll("\\W", "");
         }
@@ -384,6 +418,11 @@ public class AvailableToolChains {
         public String getPluginClass() {
             return GccCompilerPlugin.class.getSimpleName();
         }
+
+        @Override
+        public String getId() {
+            return "gcc";
+        }
     }
 
     public static class InstalledWindowsGcc extends InstalledGcc {
@@ -394,13 +433,9 @@ public class AvailableToolChains {
         /**
          * The environment required to execute a binary created by this toolchain.
          */
+        @Override
         public List<String> getRuntimeEnv() {
-            if (pathEntries.isEmpty()) {
-                return Collections.emptyList();
-            }
-
-            String path = Joiner.on(File.pathSeparator).join(pathEntries) + File.pathSeparator + System.getenv(pathVarName);
-            return Collections.singletonList(pathVarName + "=" + path);
+            return toRuntimeEnv();
         }
 
         @Override
@@ -413,6 +448,59 @@ public class AvailableToolChains {
             }
             return "UNKNOWN";
         }
+
+        @Override
+        public String getId() {
+            return getDisplayName().replaceAll("\\W", "");
+        }
+    }
+
+    public static class InstalledSwiftc extends InstalledToolChain {
+        public InstalledSwiftc(String displayName) {
+            super(displayName);
+        }
+
+        /**
+         * The environment required to execute a binary created by this toolchain.
+         */
+        @Override
+        public List<String> getRuntimeEnv() {
+            return toRuntimeEnv();
+        }
+
+        @Override
+        public String getInstanceDisplayName() {
+            return String.format("Tool chain '%s' (Swiftc)", getId());
+        }
+
+        @Override
+        public String getBuildScriptConfig() {
+            String config = String.format("%s(%s)\n", getId(), getImplementationClass());
+            for (File pathEntry : getPathEntries()) {
+                config += String.format("%s.path file('%s')", getId(), pathEntry.toURI());
+            }
+            return config;
+        }
+
+        @Override
+        public String getImplementationClass() {
+            return Swiftc.class.getSimpleName();
+        }
+
+        @Override
+        public String getPluginClass() {
+            return SwiftCompilerPlugin.class.getSimpleName();
+        }
+
+        @Override
+        public String getUnitTestPlatform() {
+            return null;
+        }
+
+        @Override
+        public boolean meets(ToolChainRequirement requirement) {
+            return requirement == ToolChainRequirement.SWIFT || requirement == ToolChainRequirement.AVAILABLE;
+        }
     }
 
     public static class InstalledVisualCpp extends InstalledToolChain {
@@ -420,7 +508,7 @@ public class AvailableToolChains {
         private File installDir;
 
         public InstalledVisualCpp(VisualStudioVersion version) {
-            super("visual c++ " + version.getVersion() + " (" + version.getVisualCppVersion().toString() + ")");
+            super("visual c++ " + version.getYear() + " (" + version.getVersion().toString() + ")");
         }
 
         @Override
@@ -443,15 +531,19 @@ public class AvailableToolChains {
                 case VISUALCPP:
                     return true;
                 case VISUALCPP_2012_OR_NEWER:
-                    return version.compareTo(VISUALSTUDIO_2012.getVisualCppVersion()) >= 0;
+                    return version.compareTo(VISUALSTUDIO_2012.getVersion()) >= 0;
                 case VISUALCPP_2013:
-                    return version.equals(VISUALSTUDIO_2013.getVisualCppVersion());
+                    return version.equals(VISUALSTUDIO_2013.getVersion());
                 case VISUALCPP_2013_OR_NEWER:
-                    return version.compareTo(VISUALSTUDIO_2013.getVisualCppVersion()) >= 0;
+                    return version.compareTo(VISUALSTUDIO_2013.getVersion()) >= 0;
                 case VISUALCPP_2015:
-                    return version.equals(VISUALSTUDIO_2015.getVisualCppVersion());
+                    return version.equals(VISUALSTUDIO_2015.getVersion());
                 case VISUALCPP_2015_OR_NEWER:
-                    return version.compareTo(VISUALSTUDIO_2015.getVisualCppVersion()) >= 0;
+                    return version.compareTo(VISUALSTUDIO_2015.getVersion()) >= 0;
+                case VISUALCPP_2017:
+                    return version.equals(VISUALSTUDIO_2017.getVersion());
+                case VISUALCPP_2017_OR_NEWER:
+                    return version.compareTo(VISUALSTUDIO_2017.getVersion()) >= 0;
                 default:
                     return false;
             }
