@@ -17,24 +17,28 @@
 package org.gradle.play.tasks;
 
 import org.gradle.api.Incubating;
-import org.gradle.api.UncheckedIOException;
+import org.gradle.api.file.DirectoryProperty;
 import org.gradle.api.file.FileCollection;
 import org.gradle.api.internal.ConventionTask;
+import org.gradle.api.tasks.Classpath;
 import org.gradle.api.tasks.InputFile;
 import org.gradle.api.tasks.InputFiles;
+import org.gradle.api.tasks.Internal;
+import org.gradle.api.tasks.Nested;
 import org.gradle.api.tasks.TaskAction;
 import org.gradle.api.tasks.compile.BaseForkOptions;
 import org.gradle.deployment.internal.DeploymentRegistry;
-import org.gradle.logging.ProgressLogger;
-import org.gradle.logging.ProgressLoggerFactory;
-import org.gradle.play.internal.run.*;
+import org.gradle.play.internal.run.DefaultPlayRunSpec;
+import org.gradle.play.internal.run.PlayApplicationDeploymentHandle;
+import org.gradle.play.internal.run.PlayApplicationRunner;
+import org.gradle.play.internal.run.PlayRunSpec;
 import org.gradle.play.internal.toolchain.PlayToolProvider;
 import org.slf4j.Logger;
 import org.slf4j.LoggerFactory;
 
 import javax.inject.Inject;
 import java.io.File;
-import java.io.IOException;
+import java.net.InetSocketAddress;
 import java.util.Set;
 
 /**
@@ -42,10 +46,12 @@ import java.util.Set;
  */
 @Incubating
 public class PlayRun extends ConventionTask {
-    private static Logger logger = LoggerFactory.getLogger(PlayRun.class);
+    private static final Logger LOGGER = LoggerFactory.getLogger(PlayRun.class);
 
     private int httpPort;
 
+    private final DirectoryProperty workingDir = getProject().getLayout().directoryProperty();
+
     @InputFile
     private File applicationJar;
 
@@ -55,10 +61,10 @@ public class PlayRun extends ConventionTask {
     @InputFiles
     private Set<File> assetsDirs;
 
-    @InputFiles
+    @Classpath
     private FileCollection runtimeClasspath;
 
-    @InputFiles
+    @Classpath
     private FileCollection changingClasspath;
 
     private BaseForkOptions forkOptions;
@@ -68,6 +74,7 @@ public class PlayRun extends ConventionTask {
     /**
      * fork options for the running a Play application.
      */
+    @Nested
     public BaseForkOptions getForkOptions() {
         if (forkOptions == null) {
             forkOptions = new BaseForkOptions();
@@ -77,49 +84,18 @@ public class PlayRun extends ConventionTask {
 
     @TaskAction
     public void run() {
-        ProgressLoggerFactory progressLoggerFactory = getServices().get(ProgressLoggerFactory.class);
-        PlayApplicationDeploymentHandle deploymentHandle = registerOrFindDeploymentHandle(getPath());
+        String deploymentId = getPath();
+        DeploymentRegistry deploymentRegistry = getDeploymentRegistry();
+        PlayApplicationDeploymentHandle deploymentHandle = deploymentRegistry.get(deploymentId, PlayApplicationDeploymentHandle.class);
 
-        if (!deploymentHandle.isRunning()) {
-            ProgressLogger progressLogger = progressLoggerFactory.newOperation(PlayRun.class)
-                .start("Start Play server", "Starting Play");
+        if (deploymentHandle == null) {
+            PlayRunSpec spec = new DefaultPlayRunSpec(runtimeClasspath, changingClasspath, applicationJar, assetsJar, assetsDirs, workingDir.get().getAsFile(), getForkOptions(), getHttpPort());
+            PlayApplicationRunner playApplicationRunner = playToolProvider.get(PlayApplicationRunner.class);
+            deploymentHandle = deploymentRegistry.start(deploymentId, DeploymentRegistry.ChangeBehavior.BLOCK, PlayApplicationDeploymentHandle.class, spec, playApplicationRunner);
 
-            try {
-                int httpPort = getHttpPort();
-                PlayRunSpec spec = new DefaultPlayRunSpec(runtimeClasspath, changingClasspath, applicationJar, assetsJar, assetsDirs, getProject().getProjectDir(), getForkOptions(), httpPort);
-                PlayApplicationRunnerToken runnerToken = playToolProvider.get(PlayApplicationRunner.class).start(spec);
-                deploymentHandle.start(runnerToken);
-            } finally {
-                progressLogger.completed();
-            }
-        }
-
-        if (!getProject().getGradle().getStartParameter().isContinuous()) {
-            ProgressLogger progressLogger = progressLoggerFactory.newOperation(PlayRun.class)
-                .start(String.format("Run Play App at http://localhost:%d/", httpPort),
-                    String.format("Running at http://localhost:%d/", httpPort));
-            try {
-                waitForCtrlD();
-            } finally {
-                progressLogger.completed();
-            }
-        } else {
-            logger.warn(String.format("Running Play App (%s) at http://localhost:%d/", getPath(), httpPort));
-        }
-    }
-
-    private void waitForCtrlD() {
-        while (true) {
-            try {
-                int c = System.in.read();
-                if (c == -1 || c == 4) {
-                    // STOP on Ctrl-D or EOF.
-                    logger.info("received end of stream (ctrl+d)");
-                    return;
-                }
-            } catch (IOException e) {
-                throw new UncheckedIOException(e);
-            }
+            InetSocketAddress playAppAddress = deploymentHandle.getPlayAppAddress();
+            String playUrl = "http://localhost:" + playAppAddress.getPort() + "/";
+            LOGGER.warn("Running Play App ({}) at {}", getPath(), playUrl);
         }
     }
 
@@ -130,6 +106,7 @@ public class PlayRun extends ConventionTask {
      *
      * @return HTTP port
      */
+    @Internal
     public int getHttpPort() {
         return httpPort;
     }
@@ -138,6 +115,16 @@ public class PlayRun extends ConventionTask {
         this.httpPort = httpPort;
     }
 
+    /**
+     * The working directory.
+     *
+     * @since 4.4
+     */
+    @Internal
+    public DirectoryProperty getWorkingDir() {
+        return workingDir;
+    }
+
     /**
      * The Play application jar to run.
      */
@@ -188,13 +175,4 @@ public class PlayRun extends ConventionTask {
         throw new UnsupportedOperationException();
     }
 
-    private PlayApplicationDeploymentHandle registerOrFindDeploymentHandle(String deploymentId) {
-        DeploymentRegistry deploymentRegistry = getDeploymentRegistry();
-        PlayApplicationDeploymentHandle deploymentHandle = deploymentRegistry.get(PlayApplicationDeploymentHandle.class, deploymentId);
-        if (deploymentHandle == null) {
-            deploymentHandle = new PlayApplicationDeploymentHandle(deploymentId);
-            deploymentRegistry.register(deploymentId, deploymentHandle);
-        }
-        return deploymentHandle;
-    }
 }
