@@ -17,6 +17,7 @@
 package org.gradle.api.plugins;
 
 import groovy.lang.Closure;
+import org.gradle.api.Action;
 import org.gradle.api.JavaVersion;
 import org.gradle.api.internal.file.FileLookup;
 import org.gradle.api.internal.file.SourceDirectorySetFactory;
@@ -26,12 +27,14 @@ import org.gradle.api.java.archives.Manifest;
 import org.gradle.api.java.archives.internal.DefaultManifest;
 import org.gradle.api.reporting.ReportingExtension;
 import org.gradle.api.tasks.SourceSetContainer;
+import org.gradle.internal.Actions;
 import org.gradle.internal.reflect.Instantiator;
-import org.gradle.util.ConfigureUtil;
-import org.gradle.util.DeprecationLogger;
+import org.gradle.testing.base.plugins.TestingBasePlugin;
 
 import java.io.File;
 
+import static org.gradle.util.ConfigureUtil.configure;
+
 /**
  * Is mixed into the project when applying the {@link org.gradle.api.plugins.JavaBasePlugin} or the
  * {@link org.gradle.api.plugins.JavaPlugin}.
@@ -39,8 +42,6 @@ import java.io.File;
 public class JavaPluginConvention {
     private ProjectInternal project;
 
-    private String dependencyCacheDirName;
-
     private String docsDirName;
 
     private String testResultsDirName;
@@ -56,10 +57,9 @@ public class JavaPluginConvention {
         this.project = project;
         sourceSets = instantiator.newInstance(DefaultSourceSetContainer.class, project.getFileResolver(), project.getTasks(), instantiator,
             project.getServices().get(SourceDirectorySetFactory.class));
-        dependencyCacheDirName = "dependency-cache";
         docsDirName = "docs";
-        testResultsDirName = "test-results";
-        testReportDirName = "tests";
+        testResultsDirName = TestingBasePlugin.TEST_RESULTS_DIR_NAME;
+        testReportDirName = TestingBasePlugin.TESTS_DIR_NAME;
     }
 
     /**
@@ -71,7 +71,7 @@ public class JavaPluginConvention {
      * See the example below how {@link org.gradle.api.tasks.SourceSet} 'main' is accessed and how the {@link org.gradle.api.file.SourceDirectorySet} 'java'
      * is configured to exclude some package from compilation.
      *
-     * <pre autoTested=''>
+     * <pre class='autoTested'>
      * apply plugin: 'java'
      *
      * sourceSets {
@@ -84,18 +84,12 @@ public class JavaPluginConvention {
      * </pre>
      *
      * @param closure The closure to execute.
-     * @return NamedDomainObjectContainer<org.gradle.api.tasks.SourceSet>
+     * @return NamedDomainObjectContainer&lt;org.gradle.api.tasks.SourceSet&gt;
      */
     public Object sourceSets(Closure closure) {
         return sourceSets.configure(closure);
     }
 
-    @Deprecated
-    public File getDependencyCacheDir() {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("JavaPluginConvention.getDependencyCacheDir()");
-        return project.getServices().get(FileLookup.class).getFileResolver(project.getBuildDir()).resolve(dependencyCacheDirName);
-    }
-
     /**
      * Returns a file pointing to the root directory supposed to be used for all docs.
      */
@@ -131,7 +125,7 @@ public class JavaPluginConvention {
     /**
      * Sets the source compatibility used for compiling Java sources.
      *
-     * @value The value for the source compatibility as defined by {@link JavaVersion#toVersion(Object)}
+     * @param value The value for the source compatibility as defined by {@link JavaVersion#toVersion(Object)}
      */
     public void setSourceCompatibility(Object value) {
         setSourceCompatibility(JavaVersion.toVersion(value));
@@ -140,7 +134,7 @@ public class JavaPluginConvention {
     /**
      * Sets the source compatibility used for compiling Java sources.
      *
-     * @value The value for the source compatibility
+     * @param value The value for the source compatibility
      */
     public void setSourceCompatibility(JavaVersion value) {
         srcCompat = value;
@@ -156,7 +150,7 @@ public class JavaPluginConvention {
     /**
      * Sets the target compatibility used for compiling Java sources.
      *
-     * @value The value for the target compatibility as defined by {@link JavaVersion#toVersion(Object)}
+     * @param value The value for the target compatibility as defined by {@link JavaVersion#toVersion(Object)}
      */
     public void setTargetCompatibility(Object value) {
         setTargetCompatibility(JavaVersion.toVersion(value));
@@ -165,7 +159,7 @@ public class JavaPluginConvention {
     /**
      * Sets the target compatibility used for compiling Java sources.
      *
-     * @value The value for the target compatibility
+     * @param value The value for the target compatibility
      */
     public void setTargetCompatibility(JavaVersion value) {
         targetCompat = value;
@@ -175,7 +169,7 @@ public class JavaPluginConvention {
      * Creates a new instance of a {@link Manifest}.
      */
     public Manifest manifest() {
-        return manifest(null);
+        return manifest(Actions.<Manifest>doNothing());
     }
 
     /**
@@ -185,22 +179,23 @@ public class JavaPluginConvention {
      * @param closure The closure to use to configure the manifest.
      */
     public Manifest manifest(Closure closure) {
-        return ConfigureUtil.configure(closure, new DefaultManifest(project.getFileResolver()));
+        return configure(closure, createManifest());
     }
 
     /**
-     * The name of the dependency cache dir.
+     * Creates and configures a new instance of a {@link Manifest}.
+     *
+     * @param action The action to use to configure the manifest.
+     * @since 3.5
      */
-    @Deprecated
-    public String getDependencyCacheDirName() {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("JavaPluginConvention.getDependencyCacheDirName()");
-        return dependencyCacheDirName;
+    public Manifest manifest(Action<? super Manifest> action) {
+        Manifest manifest = createManifest();
+        action.execute(manifest);
+        return manifest;
     }
 
-    @Deprecated
-    public void setDependencyCacheDirName(String dependencyCacheDirName) {
-        DeprecationLogger.nagUserOfDiscontinuedMethod("JavaPluginConvention.getDependencyCacheDirName()");
-        this.dependencyCacheDirName = dependencyCacheDirName;
+    private Manifest createManifest() {
+        return new DefaultManifest(project.getFileResolver());
     }
 
     /**
@@ -246,13 +241,4 @@ public class JavaPluginConvention {
     public ProjectInternal getProject() {
         return project;
     }
-
-    /**
-     * project
-     * @deprecated Project should be considered final.
-     */
-    @Deprecated
-    public void setProject(ProjectInternal project) {
-        this.project = project;
-    }
 }
