@@ -98,6 +98,14 @@ public class SigningExtension {
         return project;
     }
 
+    /**
+     * Whether or not this task should fail if no signatory or signature type are configured at generation time.
+     * @since 4.0
+     */
+    public void setRequired(boolean required) {
+        this.required = required;
+    }
+
     /**
      * Whether or not this task should fail if no signatory or signature type are configured at generation time.
      *
@@ -374,10 +382,23 @@ public class SigningExtension {
     /**
      * Signs the POM artifact for the given Maven deployment.
      *
-     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p> <pre autoTested=''> uploadArchives { repositories { mavenDeployer {
-     * beforeDeployment { MavenDeployment deployment -> signing.signPom(deployment) } } } } </pre> <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
+     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p>
+     * <pre class='autoTested'>
+     * uploadArchives {
+     *   repositories {
+     *     mavenDeployer {
+     *       beforeDeployment { MavenDeployment deployment -&gt;
+     *         signing.signPom(deployment)
+     *       }
+     *     }
+     *   }
+     * }
+     * </pre>
+     * <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
      * operation} for the POM.</p> <p> If {@link #isRequired()} is false and the signature cannot be generated (e.g. no configured signatory), this method will silently do nothing. That is, a
      * signature for the POM file will not be uploaded.
+     * <p>
+     * <b>Note:</b> Signing the generated POM file generated by the Maven Publishing plugin is currently not supported. Future versions of Gradle might add this functionality.
      *
      * @param mavenDeployment The deployment to sign the POM of
      * @param closure the configuration of the underlying {@link SignOperation sign operation} for the POM (optional)
@@ -409,10 +430,23 @@ public class SigningExtension {
     /**
      * Signs the POM artifact for the given Maven deployment.
      *
-     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p> <pre autoTested=''> uploadArchives { repositories { mavenDeployer {
-     * beforeDeployment { MavenDeployment deployment -> signing.signPom(deployment) } } } } </pre> <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
+     * <p>You can use this method to sign the generated POM when publishing to a Maven repository with the Maven plugin. </p>
+     * <pre class='autoTested'>
+     * uploadArchives {
+     *   repositories {
+     *     mavenDeployer {
+     *       beforeDeployment { MavenDeployment deployment -&gt;
+     *         signing.signPom(deployment)
+     *       }
+     *     }
+     *   }
+     * }
+     * </pre>
+     * <p>You can optionally provide a configuration closure to fine tune the {@link SignOperation sign
      * operation} for the POM.</p> <p> If {@link #isRequired()} is false and the signature cannot be generated (e.g. no configured signatory), this method will silently do nothing. That is, a
      * signature for the POM file will not be uploaded.
+     * <p>
+     * <b>Note:</b> Signing the generated POM file generated by the Maven Publishing plugin is currently not supported. Future versions of Gradle might add this functionality.
      *
      * @param mavenDeployment The deployment to sign the POM of
      * @return the generated signature artifact
