@@ -16,25 +16,30 @@
 
 package org.gradle.api.internal.tasks.testing.processors;
 
-import org.gradle.api.Nullable;
-import org.gradle.api.internal.tasks.testing.*;
+import org.gradle.api.internal.tasks.testing.DefaultTestSuiteDescriptor;
+import org.gradle.api.internal.tasks.testing.TestClassProcessor;
+import org.gradle.api.internal.tasks.testing.TestCompleteEvent;
+import org.gradle.api.internal.tasks.testing.TestResultProcessor;
+import org.gradle.api.internal.tasks.testing.TestStartEvent;
 import org.gradle.api.internal.tasks.testing.results.AttachParentTestResultProcessor;
-import org.gradle.internal.time.TimeProvider;
+import org.gradle.internal.time.Clock;
+
+import javax.annotation.Nullable;
 
 public class TestMainAction implements Runnable {
     private final Runnable detector;
     private final TestClassProcessor processor;
     private final TestResultProcessor resultProcessor;
-    private final TimeProvider timeProvider;
+    private final Clock clock;
     private final Object testTaskOperationId;
     private final Object rootTestSuiteId;
     private final String displayName;
 
-    public TestMainAction(Runnable detector, TestClassProcessor processor, TestResultProcessor resultProcessor, TimeProvider timeProvider, Object testTaskOperationId, Object rootTestSuiteId, String displayName) {
+    public TestMainAction(Runnable detector, TestClassProcessor processor, TestResultProcessor resultProcessor, Clock clock, Object testTaskOperationId, Object rootTestSuiteId, String displayName) {
         this.detector = detector;
         this.processor = processor;
         this.resultProcessor = new AttachParentTestResultProcessor(resultProcessor);
-        this.timeProvider = timeProvider;
+        this.clock = clock;
         this.testTaskOperationId = testTaskOperationId;
         this.rootTestSuiteId = rootTestSuiteId;
         this.displayName = displayName;
@@ -43,7 +48,7 @@ public class TestMainAction implements Runnable {
     @Override
     public void run() {
         RootTestSuiteDescriptor suite = new RootTestSuiteDescriptor(rootTestSuiteId, displayName, testTaskOperationId);
-        resultProcessor.started(suite, new TestStartEvent(timeProvider.getCurrentTime()));
+        resultProcessor.started(suite, new TestStartEvent(clock.getCurrentTime()));
         try {
             processor.startProcessing(resultProcessor);
             try {
@@ -52,7 +57,7 @@ public class TestMainAction implements Runnable {
                 processor.stop();
             }
         } finally {
-            resultProcessor.completed(suite.getId(), new TestCompleteEvent(timeProvider.getCurrentTime()));
+            resultProcessor.completed(suite.getId(), new TestCompleteEvent(clock.getCurrentTime()));
         }
     }
 
