@@ -16,7 +16,8 @@
 package org.gradle.tooling.model;
 
 import org.gradle.api.Incubating;
-import org.gradle.api.Nullable;
+
+import javax.annotation.Nullable;
 
 /**
  * Represents a task which is executable by Gradle.
